<?php
namespace Controller;

use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;

class NutritionalReport extends \Zeedhi\Framework\Controller\Simple {

    protected $nutritionalReportService;

    public function __construct(\Service\NutritionalReport $nutritionalReportService){
        $this->nutritionalReportService = $nutritionalReportService;
    }

    /*Nutritional Report - General*/
    public function getStoreRelativeData(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL = $params[0]['value'];
            $DTINI = $params[1]['value'];
            $DTFIM = $params[2]['value'];
            $requiredData = $params[3]['value']; //Defines if the data needed is either store 'consumption' or store 'nutritional' data
            if (sizeof($params) > 4) $lang = $params[4]['value'];
            else $lang = "pt_br";

            $arr = array(
                'error' => false,
                'message' => 'OK',
                'data' => array()
            );
            $families = $this->nutritionalReportService->getFamilies($CDFILIAL);

            if (!empty($families)){
                foreach($families as $family){

                    if($family['CDVENDEDOR'] == NULL){
                        $CDVENDEDOR = $this->nutritionalReportService->getSalesman($CDFILIAL)['CDVENDEDOR'];
                    } else {
                        $CDVENDEDOR = $family['CDVENDEDOR'];
                    }

                    $CDCAIXA = $this->nutritionalReportService->getRegister($CDVENDEDOR)['CDCAIXA'];
                    $CDGRUPOPRODUTO = $this->nutritionalReportService->getGroupProd($CDFILIAL, $family['CDFAMILISALD']);

                    if ($requiredData == 'nutritional'){
                        $products = $this->nutritionalReportService->getProducts($CDFILIAL, $CDCAIXA, $CDGRUPOPRODUTO);

                        $familyData = array(
                            'products' => $products,
                            'family'   => $family['NMFAMILISALD']
                        );

                        array_push($arr['data'],$familyData);
                    }
                    else if($requiredData == 'consumption'){
                        $productsSold = $this->nutritionalReportService->getProductsSold($CDFILIAL, $family['CDFAMILISALD'], $CDGRUPOPRODUTO, $DTINI, $DTFIM);

                        if (!$productsSold['error']){
                            $productsSold['chartData'] = $this->_calculateProductsSoldChart($productsSold['productsSold'], $lang);
                            $productsSold['gridData']  = $this->_calculateCommonProducts($productsSold['productsSold']);
                        }

                        unset($productsSold['productsSold']);

                        $familyData = array(
                            'family'   => $family['NMFAMILISALD'],
                            'productsSold' => $productsSold
                        );

                        array_push($arr['data'],$familyData);
                    }
                }
            }


            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('NutritionalDataRepo', $arr));
        } catch (\Exception $e){
            $arr = array(
                'error' => true,
                'message' => $e->getMessage()
            );

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('NutritionalDataRepo', $arr));
        }
    }

    private function _calculateProductsSoldChart($products, $lang){
        try{
            $chartData = array();

            $highNutritionalValue = array_filter($products, array($this, 'hasHighNutritionalValue'));
            $mediumNutritionalValue = array_filter($products, array($this, 'hasMediumNutritionalValue'));
            $lowNutritionalValue = array_filter($products, array($this, 'hasLowNutritionalValue'));

            $highPercentage = (100 * sizeof($highNutritionalValue)) / sizeof($products);
            $mediumPercentage = (100 * sizeof($mediumNutritionalValue)) / sizeof($products);
            $lowPercentage = (100 * sizeof($lowNutritionalValue)) / sizeof($products);

            if ($lang == "en_us"){
                $high = "High";
                $moderate = "Moderate";
                $low = "Low";
            }
            else {
                $high = "Alto";
                $moderate = "Moderado";
                $low = "Baixo";
            }

            $high = array(
                'name' => $high,
                'quantity' => $highPercentage,
                'color' => "#41d024"
            );
            array_push($chartData, $high);

            $medium = array(
                'name' => $moderate,
                'quantity' => $mediumPercentage,
                'color' => "#f5b60a"
            );
            array_push($chartData, $medium);

            $low = array(
                'name' => $low,
                'quantity' => $lowPercentage,
                'color' => "#ef1111"
            );
            array_push($chartData, $low);

            return $chartData;

        } catch(\Exception $e){
            throw new \Exception("Erro ao calcular porcentagem dos produtos vendidos: " . $e->getMessage());
        }
    }

    /*Nutritional Report - User specific*/
    public function getUserShoppingData(Request\Filter $request, Response $response){
        try {
            $params = $request->getFilterCriteria()->getConditions();

            $CDFILIAL = $params[0]['value'];
            $CDCLIENTE = $params[1]['value'];
            $CDCONSUMIDOR = $params[2]['value'];

            $DTINI = $params[3]['value'];
            $DTFIM = $params[4]['value'];

            $arr = array(
                'error' => false,
                'message' => 'OK',
                'data' => array()
            );
            $families = $this->nutritionalReportService->getFamilies($CDFILIAL);

            if (!empty($families)){
                foreach($families as $family){

                    if ($family['CDVENDEDOR'] == NULL){
                        $CDVENDEDOR = $this->nutritionalReportService->getSalesman($CDFILIAL)['CDVENDEDOR'];
                    }
                    else {
                        $CDVENDEDOR = $family['CDVENDEDOR'];
                    }

                    $CDCAIXA = $this->nutritionalReportService->getRegister($CDVENDEDOR)['CDCAIXA'];
                    $CDGRUPOPRODUTO = $this->nutritionalReportService->getGroupProd($CDFILIAL, $family['CDFAMILISALD']);

                    $userShoppingData = $this->nutritionalReportService->getUserShoppingData($CDFILIAL, $family['CDFAMILISALD'], $CDCLIENTE, $CDCONSUMIDOR,$CDGRUPOPRODUTO, $DTINI, $DTFIM);
                    $generalData      = $this->nutritionalReportService->getProducts($CDFILIAL, $CDCAIXA, $CDGRUPOPRODUTO);

                    if (!$userShoppingData['error']){
                        $userShoppingData['comparisonData'] = $this->_calculateComparisonData($userShoppingData['shoppingData']);
                    }

                    if (!$generalData['error']){
                        $generalData['comparisonData'] = $this->_calculateComparisonData($generalData['products']);
                    }

                    if (!$generalData['error'] && !$userShoppingData['error']){
                        $userShoppingData['mainChoices'] = $this->_calculateUserMainChoices($userShoppingData['shoppingData'], $generalData['products']);
                    }

                    $familyData = array(
                        'error' => false,
                        'userData' => $userShoppingData,
                        'generalData' => $generalData
                    );

                    array_push($arr['data'],$familyData);
                }
            }

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('NutritionalDataRepo', $arr));
        } catch (\Exception $e){
            $arr = array(
                'error' => true,
                'message' => $e->getMessage()
            );

            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('NutritionalDataRepo', $arr));
        }
    }

    private function _calculateComparisonData($products){
        $highNutritionalValue = array_filter($products, array($this, 'hasHighNutritionalValue'));
        $mediumNutritionalValue = array_filter($products, array($this, 'hasMediumNutritionalValue'));
        // $lowNutritionalValue = array_filter($products, array($this, 'hasLowNutritionalValue'));

        $bothPercentage = (100 * (sizeof($highNutritionalValue) + sizeof($mediumNutritionalValue)) / sizeof($products));
        $highPercentage = (100 * sizeof($highNutritionalValue)) / (sizeof($highNutritionalValue) + sizeof($mediumNutritionalValue));
        $mediumPercentage = (100 * sizeof($mediumNutritionalValue)) / (sizeof($highNutritionalValue) + sizeof($mediumNutritionalValue));
        // $lowPercentage = (100 * sizeof($lowNutritionalValue)) / sizeof($products);

        $comparisonData = array(
            'bothPercentage' => $bothPercentage,
            'highNutritionalValue' => $highPercentage,
            'mediumNutritionalValue' => $mediumPercentage
        );

        return $comparisonData;
    }

    private function _calculateUserMainChoices($shoppingProducts, $allProducts){
        $finalSugestions = array();
        $commonProducts = $this->_calculateCommonProducts($shoppingProducts);
        $allProductsIndexedByGroup = array();

        $defaultProduct = array(
            'NMPRODUTO' => ' - ',
            'VRNUTRIPROD' => "0"
        );

        foreach ($allProducts as $product){
            $allProductsIndexedByGroup[$product['CDGRUPO']][] = $product;
        }

        foreach ($commonProducts as $commonProduct){
            if ($commonProduct['VRNUTRIPROD'] == '0'){
                $arr = array(
                    'chosenItem' => $commonProduct,
                    'sugestedItem' => $defaultProduct
                );
                array_push($finalSugestions, $arr);
            }
            else {
                if (array_key_exists($commonProduct['CDGRUPO'], $allProductsIndexedByGroup)){
                    $added = false;
                    foreach ($allProductsIndexedByGroup[$commonProduct['CDGRUPO']] as $item){
                        if ($item['VRNUTRIPROD'] != "0" && $item['VRNUTRIPROD'] < $commonProduct['VRNUTRIPROD']){
                            $arr = array(
                                'chosenItem' => $commonProduct,
                                'sugestedItem' => $item
                            );
                            array_push($finalSugestions, $arr);
                            $added = true;
                            break;
                        }
                    }
                    if (!$added){
                        $arr = array(
                            'chosenItem' => $commonProduct,
                            'sugestedItem' => $defaultProduct
                        );
                        array_push($finalSugestions, $arr);
                    }
                }
            }
        }
        return $finalSugestions;
    }

    /*Both*/
    private function _calculateCommonProducts($products){
        $commonProducts = array();

        $productData = array_column($products, null, 'CDPRODUTO');
        $mostCommonValues = array_count_values(array_column($products, 'CDPRODUTO'));
        arsort($mostCommonValues);
        $popular = array_slice(array_keys($mostCommonValues), 0, 5, true);

        foreach ($popular as $productCode) {
            $productData[$productCode]['quantity'] = (100  * $mostCommonValues[$productCode]) / sizeof($products);
            array_push($commonProducts, $productData[$productCode]);
        }

        return $commonProducts;
    }

    function hasHighNutritionalValue($var){
        return $var['VRNUTRIPROD'] == '1';
    }

    function hasMediumNutritionalValue($var){
        return $var['VRNUTRIPROD'] == '2';
    }

    function hasLowNutritionalValue($var){
        return $var['VRNUTRIPROD'] == '3';
    }
}