<?php

namespace Service;

class StoreService {

    protected $entityManager;
    protected $apiParams;
    protected $date;

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \Odhen\API\Service\Parametros $apiParams, \Util\Date $date){
        $this->entityManager = $entityManager;
        $this->apiParams = $apiParams;
        $this->date = $date;
    }

    const URL_SERVIDOR_IMAGEM = 'http://midia.teknisa.com/files/';
    const TIPO_GRUPO = '2';
    const TIPO_SUB_GRUPO = 'I';
    const OBS_TIPO_ADICIONAR = 'A';

    public function getStores(){
        $params = array();
        return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_AVAILABLE_STORES, $params);
    }

    public function getStandardClient($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );

        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_STANDARD_CLIENT, $params);
        if (!Empty($result[0]['CDCLIENTE'])){
            return $result[0]['CDCLIENTE'];
        }
        else {
            throw new \Exception('Não existe cliente padrão para a filial.');
        }
    }

    public function getTipoRec($CDFILIAL){
        $params = array(
            ':CDFILIAL' => $CDFILIAL
        );
        $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::SQL_TIPO_CONS_PADRAO, $params);
        if ($result[0]['CDTIPOCONS'] != null){
            return $result[0];
        }
        else {
            throw new \Exception('Não existe tipo de consumidor para a filial.');
        }
    }

    public function getGeneralParams($CDFILIAL, $CDLOJA, $CDCAIXA){
        $CDVENDEDOR = $this->getVendorCode($CDFILIAL);

        return array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'CDCAIXA' => $CDCAIXA,
            'CDVENDEDORDLV' => $CDVENDEDOR,
            'CDOPERADOR' => $this->getOperatorCode($CDFILIAL, $CDVENDEDOR),
            'IDCONTROPROD' => $this->getProductControlId($CDFILIAL, $CDLOJA)
        );
    }

    public function getVendorCode($CDFILIAL){
        try {
            $params = array(
                ':CDFILIAL' => $CDFILIAL
            );
            $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_VENDOR_CODE, $params);
            return !Empty($result) ? $result[0]['CDVENDEDOR'] : null;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getOperatorCode($CDFILIAL, $CDVENDEDOR){
        try {
            $params = array(
                ':CDFILIAL' => $CDFILIAL,
                ':CDVENDEDOR' => $CDVENDEDOR
            );
            $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_OPERATOR_CODE, $params);
            if (!Empty($result)){
                $result = $result[0]['CDOPERADOR'];
            }
            else {
                throw new \Exception('Operador não existe para este vendedor.');
            }
            return $result;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getProductControlId($CDFILIAL, $CDLOJA){
        try {
            $params = array(
                ':CDFILIAL' => $CDFILIAL,
                ':CDLOJA' => $CDLOJA
            );
            $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCT_CONTROL_ID, $params);
            return !Empty($result) ? $result[0]['IDCONTROPROD'] : null;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getWeekSchedule($CDFILIAL, $CDLOJA){
        try {
            $params = array(
                ':CDFILIAL' => $CDFILIAL,
                ':CDLOJA' => $CDLOJA
            );
            return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_WEEK_SCHEDULE, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getStorePhone($CDFILIAL){
        try {
            $params = array(
                ':CDFILIAL' => $CDFILIAL
            );
            $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_STORE_PHONE, $params);
            !Empty($result) ? $result[0] : null;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getMoney(){
        try {
            $params = array();
            $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_MONEY, $params);
            return !Empty($result) ? $result[0]['CDTIPORECE'] : null;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getRedirectsStore(){
        try {
            return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_REDIRECT_STORE);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getStoreByBranch($CDFILIAL){
        try {
            $params = array(
                ':CDFILIAL' => $CDFILIAL
            );
            return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_STORES_BY_BRANCH, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function saveRedirect($CDREDILOJA, $CDFILIAL, $CDLOJA, $CDFILDESTINO, $CDLOJDESTINO, $DTINICIAL, $DTFINAL){
        try {
            $params = array(
                ':CDREDILOJA'     => $CDREDILOJA,
                ':CDFILIAL'     => $CDFILIAL,
                ':CDLOJA'         => $CDLOJA,
                ':CDFILDESTINO' => $CDFILDESTINO,
                ':CDLOJDESTINO' => $CDLOJDESTINO,
                ':DTINICIAL'     => $DTINICIAL,
                ':DTFINAL'         => $DTFINAL
            );

            return $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_REDIRECT, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

        public function changeProductionTime($CDFILIAL, $CDLOJA, $NRTEMPPROD, $HRTEMPOENTREGA){
            $params = array(
                ':CDFILIAL'       => $CDFILIAL,
                ':CDLOJA'           => $CDLOJA,
                ':NRTEMPPROD'     => $NRTEMPPROD,
                ':HRTEMPOENTREGA' => $HRTEMPOENTREGA
            );

            $this->entityManager->getConnection()->executeQuery(\Util\Query::UPDATE_PRODUCTION_TIME, $params);
    }

    public function getProductionTime($CDFILIAL, $CDLOJA){
            $params = array(
                ':CDFILIAL'       => $CDFILIAL,
                ':CDLOJA'           => $CDLOJA
            );

            return $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_PRODUCTION_TIME, $params);
    }

     public function logChangeProductionTime($CDFILIAL, $CDLOJA, $DSCAMPOALT, $VRALTERACAO, $CDOPERADOR, $DSOBSERVACAO){
            $params = array(
                ':CDFILIAL'       => $CDFILIAL,
                ':CDLOJA'           => $CDLOJA,
                ':DSCAMPOALT'     => $DSCAMPOALT,
                ':VRALTERACAO'    => $VRALTERACAO,
                ':CDOPERADOR'     => $CDOPERADOR,
                ':DSOBSERVACAO'   => $DSOBSERVACAO
            );

            $this->entityManager->getConnection()->executeQuery(\Util\Query::INSERT_PRODUCTION_TIME_LOG, $params);
    }

    public function redirectDelete($CDREDILOJA){
        try {
            $params = array(
                ':CDREDILOJA'     => $CDREDILOJA,
            );
            return $this->entityManager->getConnection()->executeQuery(\Util\Query::REDIRECT_DELETE, $params);
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getCheckMode(){
        try {
            $params = array();
            $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_CHECK_MODE, $params);
            return !empty($result) ? $result[0] : null;
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
    }

    public function getPlaces($CDCLIENTE){
		try{
            $params = array(
                ':CDCLIENTE' => $CDCLIENTE,
            );
            $result = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_ALL_FILIAIS, $params);
            return $result;
		} catch(\Exception $e){
			$response->addMessage(new Message($e->getMessage('Erro no Banco!')));
		}
    }

    public function buildMenu($CDCLIENTE, $CDFILIAL, $NRCONFTELA, $CDLOJA, $CDCAIXA){
		try {
			$productPriceTable = self::buscaTabelaPadrao($CDFILIAL, $CDCLIENTE, $CDLOJA);
		    $menu = self::montaCardapio($CDFILIAL, $NRCONFTELA, $CDLOJA, $CDCLIENTE, $productPriceTable['precosIndexadosPorProduto']);
			return $menu;
		} catch (\Exception $e) {
			throw $e;
		}
    }

    public function buscaTabelaPadrao($CDFILIAL, $CDCLIENTE, $CDLOJA){
        $tabelaPadrao = self::buscaTabelaVigente($CDFILIAL, $CDLOJA, $CDCLIENTE);
        if ($tabelaPadrao['error'] == false){
            $params = array(
                'CDFILIAL'    => $tabelaPadrao['CDFILIAL'],
                'CDTABEPREC'  => $tabelaPadrao['CDTABEPREC'],
                'DTINIVGPREC' => $tabelaPadrao['DTINIVGPREC']
            );

            $precos = $this->entityManager->getConnection()->fetchAll(\Util\Query::ORACLE_PRECOS, $params);

            $precosDia = self::buscaPrecosDia($tabelaPadrao['CDFILIAL'], $tabelaPadrao['CDTABEPREC'], $tabelaPadrao['DTINIVGPREC']);

            if (!empty($precos)) {
                $precosIndexadosPorProduto = array();
                foreach ($precos as $produto) {
                    $precosIndexadosPorProduto[$produto['CDPRODUTO']] = $produto;
                }

                if (!empty($precosDia)) {
                    foreach ($precosDia as $produtoDia) {
                        $codProdutoDia = $produtoDia['CDPRODUTO'];
                        if (isset($precosIndexadosPorProduto[$codProdutoDia]) && $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] !== null) {
                            if ($produtoDia['IDPERVALORPR'] == 'V') {
                                if ($produtoDia['IDDESCACREPR'] == 'D') {
                                    //Desconto por valor
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] - $produtoDia['VRPRECODIA'];
                                } else {
                                    //Acrescimo por valor
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] + $produtoDia['VRPRECODIA'];
                                }
                            } else {
                                if ($produtoDia['IDDESCACREPR'] == 'D') {
                                    //Desconto por porcentagem
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] * (1 - $produtoDia['VRPRECODIA']/100);
                                } else {
                                    //Acrescimo por porcentagem
                                    $precoFinal = $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] * (1 + ($produtoDia['VRPRECODIA']/100));
                                }
                            }

                            if ($precoFinal > 0) {
                                $precosIndexadosPorProduto[$codProdutoDia]['VRPRECITEM'] = $precoFinal;
                            }
                        }
                    }
                }

                $result = array(
                    'error' => false,
                    'precosIndexadosPorProduto' => $precosIndexadosPorProduto
                );
            } else {
                $result = array(
                    'error' => true,
                    'message' => 'Não há produtos para a tabela de preços vigente.'
                );
            }
        } else {
            $result = $tabelaPadrao;
        }
        return $result;
    }

    public function buscaTabelaVigente($CDFILIAL, $CDLOJA, $CDCLIENTE) {
        // busca tabela de preço por CLIENTEFILIAL
        $params = array(
            "CDFILIAL" => $CDFILIAL,
            "CDCLIENTE" => $CDCLIENTE
        );

        $tabelaClienteFilial = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::CLIENTE_FILIAL, $params);
        if ($tabelaClienteFilial["CDTABEPREC"]) {
            // valida a vigência
            $DTINIVGPREC = self::validaVigencia($tabelaClienteFilial['CDCFILTABPRE'], $tabelaClienteFilial["CDTABEPREC"]);
            if ($DTINIVGPREC == false) {
                $retorno = array(
                    'error' => true,
                    'message' => 'Tabela de preços associada ao cliente/filial está inativa.'
                );
            } else {
                $retorno = array(
                	'error' => false,
                    'CDFILIAL' => $tabelaClienteFilial["CDCFILTABPRE"],
                    'CDTABEPREC' => $tabelaClienteFilial["CDTABEPREC"],
                    'DTINIVGPREC' => $DTINIVGPREC,
                    'error' => false
                );
            }
        } else {
            // busca a tabela de preço por CLIENTE
            $params = array(
                'CDCLIENTE' => $CDCLIENTE
            );
            $tabelaCliente = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::EXISTE_PRECO_CLIE, $params);
            if ($tabelaCliente["CDTABEPREC"]) {
                // valida a vigência
                $DTINIVGPREC = self::validaVigencia($tabelaCliente["CDFILTABPREC"], $tabelaCliente["CDTABEPREC"]);
                if ($DTINIVGPREC == false) {
                    $retorno = array(
                        'error' => true,
                        'message' => 'Tabela de preços associada ao cliente está inativa.'
                    );
                } else {
                    $retorno = array(
                        'error' => false,
                        'CDFILIAL' => $tabelaCliente["CDFILTABPREC"],
                        'CDTABEPREC' => $tabelaCliente["CDTABEPREC"],
                        'DTINIVGPREC' => $DTINIVGPREC
                    );
                }
            } else {
                // busca tabela de preco por LOJA
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA
                );
                $tabelaLoja = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::TABELA_PRECO_LOJA, $params);
                if ($tabelaLoja["CDTABEPREC"]) {
                    // valida a vigência
                    $DTINIVGPREC = self::validaVigencia($CDFILIAL, $tabelaLoja["CDTABEPREC"]);
                    if ($DTINIVGPREC == false) {
                        $retorno = array(
                            'error' => true,
                            'message' => 'Tabela de preços associada à loja está inativa.'
                        );
                    } else {
                        $retorno = array(
                            'error' => false,
                            'CDFILIAL' => $CDFILIAL,
                            'CDTABEPREC' => $tabelaLoja["CDTABEPREC"],
                            'DTINIVGPREC' => $DTINIVGPREC
                        );
                    }
                } else {
                    // busca tabela de preço por FILIAL (padrão)
                    $params = array(
                        'CDFILIAL' => $CDFILIAL
                    );
                    $paravend = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::PARAVEND, $params);
                    // se não tiver tabela de preço parametrizada retorna código de erro
                    if ((bool)$paravend["CDTABEPREC"] == false) {
                        $retorno = array(
                            'error' => true,
                            'message' => 'Não há tabela de preços cadastrada.'
                        );
                    } else {
                        // valida a vigência
                        $DTINIVGPREC = self::validaVigencia($CDFILIAL, $paravend["CDTABEPREC"]);
                        if ($DTINIVGPREC == false) {
                            $retorno = array(
                                'error' => true,
                                'message' => 'Tabela de preços está inativa.'
                            );
                        } else {
                            $retorno = array(
                                'error' => false,
                                'CDFILIAL' => $CDFILIAL,
                                'CDTABEPREC' => $paravend["CDTABEPREC"],
                                'DTINIVGPREC' => $DTINIVGPREC
                            );
                        }
                    }
                }
            }
        }
        return $retorno;
    }

    public function validaVigencia($CDFILIAL, $CDTABEPREC) {
        // valida vigência
        $params = array(
            'CDFILIAL'   => $CDFILIAL,
            'CDTABEPREC' => $CDTABEPREC
        );
        $retorno = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::TABELA_VENDA_ORACLE, $params);
        if ($retorno) {
            return $retorno["DTINIVGPREC"];
        } else {
            return false;
        }
    }

    private function buscaPrecosDia($CDFILIAL, $CDTABEPREC, $DTINIVGPREC) {
        $dataAtual   = $this->date->getDataAtual();
        $hora        = $dataAtual->format("H:i:s");
        $hora        = str_replace(":", "", $hora);
        $hora        = substr($hora, 0, 4);
        // $hora = '1800';

        // Expected: 1 (for Sunday) through 7 (for Saturday)
        // PHP returns: 1 (for Monday) through 7 (for Sunday)
        // So, this method changes the index for each day, changing from the forat of return
        // used by PHP to Delphi format.
        $dia = (((int)$dataAtual->format("N")) % 7) + 1;

        // @toDo: Fix consumer type for different prices
        $params = array(
            'CDFILIAL'    => $CDFILIAL,
            'CDTABEPREC'  => $CDTABEPREC,
            'DTINIVGPREC' => $DTINIVGPREC,
            'HORA'        => $hora,
            'NRDIASEMANPR' => (string) $dia,
            'CDTIPOCONSPD' => 'T'
        );

        $precosDia = $this->entityManager->getConnection()->fetchAll(\Util\Query::ORACLE_PRECOS_DIA, $params);

        return $precosDia;
    }

    public function montaCardapio($CDFILIAL, $NRCONFTELA, $CDLOJA, $CDCLIENTE, $precosIndexadosPorProduto, $CDCONSUMIDOR = null) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDLOJA' => $CDLOJA,
            'NRCONFTELA' => $NRCONFTELA
        );
        $grupoProdutos = $this->entityManager->getConnection()->fetchAll(\Util\Query::BUSCA_GRUPO_PRODUTOS, $params);

        if (!empty($grupoProdutos)){
            $subGrupoProdutos = $this->entityManager->getConnection()->fetchAll(\Util\Query::BUSCA_SUBGRUPO_PRODUTOS, $params);

            date_default_timezone_set('America/Sao_Paulo');
            $NRDIASEMANABLOQ = date("w") + 1;

            $params = array(
                'CDFILIAL'        => $CDFILIAL,
                'CDLOJA'          => $CDLOJA,
                'NRCONFTELA'      => $NRCONFTELA,
                'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
            );
            $produtos = $this->entityManager->getConnection()->fetchAll(\Util\Query::BUSCA_PRODUTOS_ORACLE, $params);

            $params = array(
                'CDFILIAL' => $CDFILIAL,
                'CDLOJA' => $CDLOJA,
                'NRDIASEMANABLOQ' => $NRDIASEMANABLOQ
            );
            $observacoesProduto = $this->entityManager->getConnection()->fetchAll(\Util\Query::BUSCA_OBSERVACOES_PRODUTO, $params);

            $observacoesIndexadasPorProduto = array();
            foreach ($observacoesProduto as $obsAtual) {
                if (!(array_key_exists($obsAtual['CDPRODUTO'], $observacoesIndexadasPorProduto))) {
                    $observacoesIndexadasPorProduto[$obsAtual['CDPRODUTO']] = array();
                }
                array_push($observacoesIndexadasPorProduto[$obsAtual['CDPRODUTO']], $obsAtual['CDOCORR']);
            }

            $arrayProdutos = self::montaArrayProdutos($grupoProdutos, $subGrupoProdutos, $produtos,
                            $precosIndexadosPorProduto, $observacoesIndexadasPorProduto, $CDLOJA, $CDFILIAL, $NRCONFTELA,
                            $CDCLIENTE, $CDCONSUMIDOR);

            if (!empty($arrayProdutos["error"])){
                $result = $arrayProdutos;
            }
            else {
                $result = array(
                    'error' => false,
                    'cardapio' => $arrayProdutos
                );
            }
        }
        else {
            $result = array(
                'error' => true,
                'message' => 'Não há grupo de produtos na configuração de tela do caixa.'
            );
        }
        return $result;
    }

    private function montaArrayProdutos($grupoProdutos, $subGrupoProdutos, $produtos,
                                        $precosIndexadosPorProduto, $observacoesIndexadasPorProduto,
                                        $CDLOJA, $CDFILIAL, $NRCONFTELA, $CDCLIENTE, $CDCONSUMIDOR){

        $arrayRetorno = array();

        // Processa os grupos.
        foreach ($grupoProdutos as $grupoAtual){
            $codGrupo = $grupoAtual['NRPGCONFTELA'] . $grupoAtual['NRBUTTON'];

            $arrayRetorno[$codGrupo] = array(
                'grupo' => array(
                    'CODIGO' => $codGrupo,
                    'DESC' => $grupoAtual['DSBUTTON'],
                    'COLOR' => $grupoAtual['NRCOLORBACK'],
                    'DSENDEIMG' => !empty($grupoAtual['DSENDEIMG']) ? self::URL_SERVIDOR_IMAGEM . $grupoAtual['DSENDEIMG'] : null,
                    'NRBUTTON' => $grupoAtual['NRBUTTON']
                ),
                'produtos' => array()
            );
        }

        // Processa os subgrupos.
        foreach ($subGrupoProdutos as $subGrupoAtual){
            $codGrupo = $subGrupoAtual['NRPGCONFTAUX'] . $subGrupoAtual['NRBUTTONAUX'];
            $codSubGrupo = $subGrupoAtual['NRPGCONFTELA'] . $subGrupoAtual['NRBUTTON'];

            $arrayRetorno[$codGrupo]['subgrupos'][$codSubGrupo] = array(
                'subgrupo' => array(
                    'CODIGO' => $codSubGrupo,
                    'DESC' => $subGrupoAtual['DSBUTTON'],
                    'COLOR' => $subGrupoAtual['NRCOLORBACK']
                ),
                'produtos' => array()
            );
        }

        // Processa os produtos.
        foreach ($produtos as $key => $produtoAtual){
            // Define preço do produto.
            if (array_key_exists($produtoAtual['CDPRODUTO'], $precosIndexadosPorProduto)){
                $doPreco = floatval($precosIndexadosPorProduto[$produtoAtual['CDPRODUTO']]['VRPRECITEM']);
            } else {
                $preco =  self::buscaPreco($CDFILIAL, $CDCLIENTE, $produtoAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
                if ($preco['retirarProduto']) unset($produtos[$key]);
                if ($preco['error']) continue;
                else $doPreco = $preco['PRECO'];
            }

            // Define as impressoras do produto.
            $params = array(
                'CDFILIAL' => $produtoAtual['CDFILIAL'],
                'CDLOJA' => $CDLOJA,
                'CDPRODUTO' => $produtoAtual['CDPRODUTO'],
                'NRCONFTELA' => $produtoAtual['NRCONFTELA']
            );
            $impressorasProduto = $this->entityManager->getConnection()->fetchAll(\Util\Query::SQL_GET_IMP_PRODUTOS, $params);

            // Trata promoção inteligente.
            $gruposPromocao = array();
            if ($produtoAtual['IDTIPOCOMPPROD'] == '3'){
                $params = array(
                    'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON'],
                    'CDFILIAL' => $CDFILIAL
                );
                $composicaoPadrao = $this->entityManager->getConnection()->fetchAll(\Util\Query::SQL_GRUPOS_PROMO_INT_FILI, $params);

                if (count($composicaoPadrao) > 0){
                    $usaComposicaoPadrao = false;
                    $gruposComposicao = $composicaoPadrao;
                }
                else {
                    $usaComposicaoPadrao = true;
                    $params = array(
                        'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON']
                    );
                    $gruposComposicao = $this->entityManager->getConnection()->fetchAll(\Util\Query::SQL_GRUPOS_PROMO_INT, $params);
                }

                if (!empty($gruposComposicao)){
                    foreach ($gruposComposicao as $smartGroup){
                        $gruposPromocao[$smartGroup['CDGRUPROMOC']] = array(
                            'grupo' => array(),
                            'produtos' => array()
                        );

                        $gruposPromocao[$smartGroup['CDGRUPROMOC']]['grupo'] = array(
                            'CDGRUPROMOC' => $smartGroup['CDGRUPROMOC'],
                            'NMGRUPROMOC' => $smartGroup['NMGRUPROMOC'],
                            'QTPRGRUPPROMOC' => intval($smartGroup['QTPRGRUPPROMOC'])
                        );

                        date_default_timezone_set('America/Sao_Paulo');
                        $NRDIASEMANABLOQ = date("w") + 1;

                        if ($usaComposicaoPadrao){
                            $params = array(
                                'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON'],
                                'CDGRUPROMOC' => $smartGroup['CDGRUPROMOC'],
                                'CDFILIAL' => $CDFILIAL,
                                'CDLOJA' => $CDLOJA,
                                'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
                            );
                            $produtosComposicao = $this->entityManager->getConnection()->fetchAll(\Util\Query::ORACLE_PRODUTOS_PROMO_INT, $params);
                        }
                        else {
                            $params = array(
                                'CDIDENTBUTON' => $produtoAtual['CDIDENTBUTON'],
                                'CDGRUPROMOC'  => $smartGroup['CDGRUPROMOC'],
                                'CDFILIAL' => $CDFILIAL,
                                'CDLOJA' => $CDLOJA,
                                'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
                            );
                            $produtosComposicao = $this->entityManager->getConnection()->fetchAll(\Util\Query::ORACLE_PRODUTOS_PROMO_INT_FILI, $params);
                        }

                        $doPrecoPromoc = '';

                        if (!empty($produtosComposicao)){
                            foreach ($produtosComposicao as $produtoComposicaoAtual) {
                                if (!empty($precosIndexadosPorProduto[$produtoAtual['CDIDENTBUTON']]['VRPRECITEM'])){
                                    if (!empty($precosIndexadosPorProduto[$produtoComposicaoAtual['CDPRODUTO']]['VRPRECITEM'])){
                                        $doPrecoPromoc = $precosIndexadosPorProduto[$produtoComposicaoAtual['CDPRODUTO']]['VRPRECITEM'];
                                    }
                                }
                                else $doPrecoPromoc = 0;

                                $params = array(
                                    'CDFILIAL' => $CDFILIAL,
                                    'CDLOJA' => $CDLOJA,
                                    'CDPRODUTO' => $produtoComposicaoAtual['CDPRODUTO'],
                                    'NRCONFTELA' => $NRCONFTELA
                                );
                                $promoPrinter = $this->entityManager->getConnection()->fetchAll(\Util\Query::SQL_GET_IMP_PRODUTOS, $params);

                                $gruposPromocao[$smartGroup['CDGRUPROMOC']]['produtos'][$produtoComposicaoAtual['CDPRODUTO']] = array(
                                    'CDPRODUTO' => $produtoComposicaoAtual['CDPRODUTO'],
                                    'IDIMPPRODUTO' => $produtoComposicaoAtual['IDIMPPRODUTO'],
                                    'IDAPLICADESCPR' => $produtoComposicaoAtual['IDAPLICADESCPR'],
                                    'IDPERVALORDES' => $produtoComposicaoAtual['IDPERVALORDES'],
                                    'NMPRODUTO' => empty($produtoComposicaoAtual['DSAPELIDOMOB']) ? $produtoComposicaoAtual['NMPRODUTO'] : $produtoComposicaoAtual['DSAPELIDOMOB'],
                                    'VRDESPRODPROMOC' => $produtoComposicaoAtual['VRDESPRODPROMOC'],
                                    'VRPRECITEM' => $doPrecoPromoc,
                                    'OBSERVACOES' => array_key_exists($produtoComposicaoAtual['CDPRODUTO'], $observacoesIndexadasPorProduto) ? $observacoesIndexadasPorProduto[$produtoComposicaoAtual['CDPRODUTO']] : [],
                                    'IDPRODBLOQ' => $produtoComposicaoAtual['IDPRODBLOQ'],
                                    'IMPRESSORAS' => $promoPrinter,
                                    'DSPRODVENDA' => $produtoComposicaoAtual['DSPRODVENDA'],
                                    'DSADICPROD' => $produtoComposicaoAtual['DSADICPROD'],
                                    'DSENDEIMGPROMO' => !empty($produtoComposicaoAtual['DSENDEIMGPROMO']) ? self::URL_SERVIDOR_IMAGEM . $produtoComposicaoAtual['DSENDEIMGPROMO'] : null,
                                	'NRORDPROMOPR' => $produtoComposicaoAtual['NRORDPROMOPR'] ? $produtoComposicaoAtual['NRORDPROMOPR'] : 99
                                );
                            }
                        }
                    }
                }
            }

            // Formatação final.
            $produtoConstruido = array(
                'CDARVPROD' => $produtoAtual['CDARVPROD'],
                'CDBARPRODUTO' => $produtoAtual['CDBARPRODUTO'],
                'CDPRODINTE' => $produtoAtual['CDPRODINTE'],
                'CDPRODUTO' => $produtoAtual['CDIDENTBUTON'],
                'NMPRODUTO' => $produtoAtual['DSBUTTON'],
                'DTFINVGPROMOC' => $produtoAtual['DTFINVGPROMOC'],
                'DTINIVGPROMOC' => $produtoAtual['DTINIVGPROMOC'],
                'GRUPOS' => $gruposPromocao,
                'OBSERVACOES' => array_key_exists($produtoAtual['CDIDENTBUTON'], $observacoesIndexadasPorProduto) ? $observacoesIndexadasPorProduto[$produtoAtual['CDIDENTBUTON']] : [],
                'IMPRESSORAS'  => $impressorasProduto,
                'IDIMPPRODUTO' => $produtoAtual['IDIMPPRODUTO'],
                'IDPESAPROD' => $produtoAtual['IDPESAPROD'],
                'IDTIPOCOMPPROD' => $produtoAtual['IDTIPOCOMPPROD'],
                'IDTIPOPROD' => $produtoAtual['IDTIPOPROD'],
                'VRPRECITEM' => $doPreco,
                'IDPRODBLOQ' => $produtoAtual['IDPRODBLOQ'],
                'NRCOLORBACK' => $produtoAtual['NRCOLORBACK'],
                'IDCONTROLAREFIL' => $produtoAtual['IDCONTROLAREFIL'],
                'DSENDEIMG' => !empty($produtoAtual['DSENDEIMG']) ? self::URL_SERVIDOR_IMAGEM . $produtoAtual['DSENDEIMG'] : null,
                'DSPRODVENDA' => $produtoAtual['DSPRODVENDA'],
                'DSADICPROD' => $produtoAtual['DSADICPROD'],
                'NMGRUPO' => $arrayRetorno[$produtoAtual['NRPGCONFTAUX'] . $produtoAtual['NRBUTTONAUX']]['grupo']['DESC'],
                'CDBARPRODUTO' => $produtoAtual['CDBARPRODUTO']
            );

            // Insere o produto no grupo ou subgrupo.
            if ($produtoAtual['IDTPBUTONAUX'] == self::TIPO_GRUPO) {
                $codGrupo = $produtoAtual['NRPGCONFTAUX'] . $produtoAtual['NRBUTTONAUX'];
                $arrayRetorno[$codGrupo]['produtos'][$produtoAtual['CDIDENTBUTON']] = $produtoConstruido;
            }
            else if ($produtoAtual['IDTPBUTONAUX'] == self::TIPO_SUB_GRUPO) {
                $codSubGrupo = $produtoAtual['NRPGCONFTAUX'] . $produtoAtual['NRBUTTONAUX'];
                $codGrupo = $produtoAtual['NRPGCONFTAUX2'] . $produtoAtual['NRBUTTONAUX2'];
                $arrayRetorno[$codGrupo]['subgrupos'][$codSubGrupo]['produtos'][$produtoAtual['CDIDENTBUTON']] = $produtoConstruido;
            }
        }

        // Busca produtos combinados.
        date_default_timezone_set('America/Sao_Paulo');
        $NRDIASEMANABLOQ = date("w") + 1;

        $params = array(
            'CDFILIAL'        => $CDFILIAL,
            'CDLOJA'          => $CDLOJA,
            'NRCONFTELA'      => $NRCONFTELA,
            'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
        );
        $produtosCombo = $this->entityManager->getConnection()->fetchAll(\Util\Query::GET_COMBO_PRODUCTS, $params);

        // Processa os produtos combinados.
        if (sizeof($produtosCombo) > 0){
            $formattedCombos = array();

            $i = 0; // Identificador do produto pai.
            foreach ($produtosCombo as $combo){
                // Busca a composição do produto combinado.
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDLOJA' => $CDLOJA,
                    'NRCONFTELA' => $NRCONFTELA,
                    'IDTIPOPROD' => intval($combo['NRBUTTON']),
                    'NRDIASEMANABLOQ' => strval($NRDIASEMANABLOQ)
                );
                $comboChildProducts = $this->entityManager->getConnection()->fetchAll(\Util\Query::BUSCA_PRODUTOS_COMBO_ORACLE, $params);

                // Define preços e observações.
                foreach ($comboChildProducts as &$childProduct){
                    if (!empty($precosIndexadosPorProduto[$childProduct['CDIDENTBUTON']]['VRPRECITEM'])){
                        $childProduct['VRPRECITEM'] = $precosIndexadosPorProduto[$childProduct['CDIDENTBUTON']]['VRPRECITEM'];
                    }
                    else {
                        $childProduct['VRPRECITEM'] = 0;
                    }
                    $childProduct['OBSERVACOES'] = array_key_exists($childProduct['CDIDENTBUTON'], $observacoesIndexadasPorProduto) ? $observacoesIndexadasPorProduto[$childProduct['CDIDENTBUTON']] : array();
                }

                // Container necessário do produto combinado.
                $comboCode = str_pad($i++, 5, '0');
                $standardGroup = array(
                    $comboCode => array(
                        'grupo' => array(
                            'CDGRUPROMOC' => $comboCode,
                            'NMGRUPROMOC' => '',
                            'QTPRGRUPPROMOC' => $combo['NRLIMPRODCOM']
                        ),
                        'produtos' => $comboChildProducts // Composição (produtos filhos).
                    )
                );

                // Formatação do produto combinado.
                $preparedCombo = array(
                    'CDARVPROD' => $combo['CDARVPROD'],
                    'CDBARPRODUTO' => $combo['CDBARPRODUTO'],
                    'CDPRODINTE' => $combo['CDPRODINTE'],
                    'CDPRODUTO' => $comboCode,
                    'NMPRODUTO' => $combo['DSBUTTON'],
                    'DTFINVGPROMOC' => $combo['DTFINVGPROMOC'],
                    'DTINIVGPROMOC' => $combo['DTINIVGPROMOC'],
                    'GRUPOS' => $standardGroup, // Itens do produto combinado.
                    'OBSERVACOES' => array(),
                    'IMPRESSORAS'  => array(),
                    'IDIMPPRODUTO' => '2', // Produto pai nunca será impresso.
                    'IDPESAPROD' => $combo['IDPESAPROD'],
                    'IDTIPOCOMPPROD' => 'C',
                    'IDTIPOPROD' => $combo['IDTIPOPROD'],
                    'IDTIPCOBRA' => $combo['IDTIPCOBRA'],
                    'VRPRECITEM' => 0.01,
                    'IDPRODBLOQ' => $combo['IDPRODBLOQ'],
                    'NRCOLORBACK' => $combo['NRCOLORBACK'],
                    'IDCONTROLAREFIL' => $combo['IDCONTROLAREFIL'],
                    'DSENDEIMG' => null,
                    'DSPRODVENDA' => $combo['DSPRODVENDA'],
                    'DSADICPROD' => null,
                    'NMGRUPO' => 'COMBOS'
                );
                array_push($formattedCombos, $preparedCombo);
            }

            // Cria um grupo para os produtos combinados.
            if (!empty($formattedCombos)){
                $arrayRetorno['X'] = array(
                    'grupo' => array(
                        'CODIGO' => 'X',
                        'DESC' => 'COMBOS',
                        'COLOR' => '8421376.000',
                        'DSENDEIMG' => null,
                        'NRBUTTON' => null
                    ),
                    'produtos' => $formattedCombos
                );
            }
        }

        // Busca todos os grupos promocionais presentes no cardapio com preço atualizado.
        $gruposPromocionais = self::buscaGruposPromocionais($arrayRetorno, $CDFILIAL, $CDCLIENTE, $CDLOJA, $CDCONSUMIDOR);
        self::atualizaPrecoGrupPromoc($arrayRetorno, $gruposPromocionais);

        return $arrayRetorno;
    }

    public function buscaPreco($CDFILIAL, $CDCLIENTE, $CDPRODUTO, $CDLOJA, $CDCONSUMIDOR) {
        $tabelaEscolhida = self::buscaTabelaVigente($CDFILIAL, $CDLOJA, $CDCLIENTE);
        if ($tabelaEscolhida['error'] == false) {
            $retorno = self::buscaPrecoProduto(
                $tabelaEscolhida["CDFILIAL"],
                $tabelaEscolhida["CDTABEPREC"],
                $CDPRODUTO,
                $tabelaEscolhida['DTINIVGPREC'],
                $CDCLIENTE,
                $CDCONSUMIDOR
            );
        }
        else {
            $retorno = $tabelaEscolhida;
        }
        return $retorno;
    }

    public function buscaPrecoProduto($CDFILIAL, $CDTABEPREC, $CDPRODUTO, $DTINIVGPREC, $CDCLIENTE, $CDCONSUMIDOR){
        $retorno = array(
            'error' => false,
            'PRECOCLIE' => 0,
            'PRECOVAR' => 0,
            'PRECOSUGER' => 0,
            'DESC' => 0,
            'ACRE' => 0,
            'PRECO' => 0,
            'retirarProduto' => false
        );

        // formata a data de vigência
        $DTINIVGPREC = $this->date->getDataDeString($DTINIVGPREC, "d-m-Y H:i:s")->format(\Util\Date::FORMATO_BRASILEIRO);

        // busca dados do produto
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDTABEPREC' => $CDTABEPREC,
            'DTINIVGPREC' => $DTINIVGPREC,
            'CDPRODUTO' => $CDPRODUTO
        );
        $itemTabPreco = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::ITEM_PRECO_ORACLE, $params);

        // se não tiver preço cadastrado retorna o código do erro
        if (empty($itemTabPreco)) {
            $retorno["error"] = true;
            $retorno["message"] = 'Preço não cadastrado para o produto ' . $CDPRODUTO . '.';
            $retorno['retirarProduto'] = true;
        // verifica se o preco está zerado
        } elseif ($itemTabPreco["PRECO"] + $itemTabPreco["PRECOCLIE"] === 0) {
            $retorno["error"] = true;
            $retorno["message"] = 'O preço do produto ' . $CDPRODUTO . ' está zerado.';
        } else {
            // caso o produto seja válido, busca preco do produto
            $result       = self::precoDia($CDFILIAL, $CDTABEPREC, $CDPRODUTO, $DTINIVGPREC, $CDCLIENTE, $CDCONSUMIDOR);

            $retorno["PRECOVAR"]   = floatval($itemTabPreco["IDPRECVARIA"]);
            $retorno["PRECOCLIE"]  = floatval($itemTabPreco["PRECOCLIE"]);
            $retorno["PRECOSUGER"] = floatval($itemTabPreco["PRECOSUGER"]);

            // verifica se existe preco do dia
            if (empty($precoDia)) {
                $retorno["PRECO"] = floatval($itemTabPreco["PRECO"]);
            } else {
                $precoDia     = floatval($result["VRPRECODIA"]); //VRPRECODIA
                $perc         = $result["PERC"]; //IDPERVALORPR
                $descAcre     = $result["DESCACRE"]; //IDDESCACREPR
                $idVisuaCupom = $result["IDVISUACUPOM"]; //IDVISUACUPOM
                
                // acrescenta ou desconta, o valor ou percentual, do preço cadastrado
                $retorno["PRECOVAR"]   = floatval($itemTabPreco["IDPRECVARIA"]);
                $retorno["PRECOCLIE"]  = floatval($itemTabPreco["PRECOCLIE"]);
                $retorno["PRECOSUGER"] = 0;

                // define se é valor (V) ou percentual (P)
                if ($perc == "V") {
                    // define se é desconto (D) ou acréscimo (A)
                    if ($descAcre == "A") {
                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["ACRE"] = 0;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"] + $precoDia, 2));
                        } else {
                            $retorno["ACRE"] = $precoDia;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    } else {
                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["DESC"] = 0;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"] - $precoDia, 2));
                        } else {
                            $retorno["DESC"] = $precoDia;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    }
                } else {
                    // define se é desconto (D) ou acréscimo (A)

                    if ($descAcre == "A") {
                        $acrescimoAux = floatval(round(($itemTabPreco["PRECO"] * $precoDia) / 100, 2));
                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["ACRE"] = 0;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"] + $acrescimoAux, 2));
                        } else {
                            $retorno["ACRE"] = $acrescimoAux;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    } else {
                        $itemTabPreco["PRECO"] = floor(floatval($itemTabPreco["PRECO"]) * 100) / 100;
                        $descontoAux = floor(floatval(($itemTabPreco["PRECO"] * $precoDia) / 100) * 100) / 100;

                        // define se desconto aparece no cupom ou não
                        if ($idVisuaCupom == "N") {
                            $retorno["DESC"] = 0;
                            $retorno["PRECO"] = floor(($itemTabPreco["PRECO"] - $descontoAux) * 100) / 100;

                            if ($retorno["PRECO"] < 0.01) {
                                $retorno["PRECO"] = 0.01;
                            }
                        } else {
                            $retorno["DESC"] = $descontoAux;
                            $retorno["PRECO"] = floatval(round($itemTabPreco["PRECO"], 2));
                        }
                    }
                }
            }
        }

        return $retorno;
    }

    private function buscaGruposPromocionais($cardapio, $CDFILIAL, $CDCLIENTE, $CDLOJA, $CDCONSUMIDOR) {
    	$gruposPromocionais = array();

    	foreach ($cardapio as &$grupoAtual) {
    		if (!empty($grupoAtual['subgrupos'])) {
    			foreach ($grupoAtual['subgrupos'] as $subGrupoAtual) {
    				foreach ($subGrupoAtual['produtos'] as $produtoSubGrupoAtual) {
    					if ($produtoSubGrupoAtual['IDTIPOCOMPPROD'] == '3') {
    						foreach ($produtoSubGrupoAtual['GRUPOS'] as $chaveGrupo => &$grupoPromocAtual) {
			    				$chaveExiste = array_key_exists($chaveGrupo, $gruposPromocionais);
			    				if (!$chaveExiste) {
			    					foreach ($grupoPromocAtual['produtos'] as &$prodGrupoPromocAtual) {
			    						$preco =  self::buscaPreco($CDFILIAL, $CDCLIENTE, $prodGrupoPromocAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
	                                    $prodGrupoPromocAtual['VRPRECITEM'] = $preco['PRECO'];
			    					}
			    					$gruposPromocionais[$chaveGrupo] = $grupoPromocAtual;
			    				}
			    			}
    					}
    				}
    			}
    		}
            else {
    			foreach ($grupoAtual['produtos'] as &$produtoAtual) {
		    		if ($produtoAtual['IDTIPOCOMPPROD'] == '3') {
		    			// produto é combo
		    			foreach ($produtoAtual['GRUPOS'] as $chaveGrupo => &$grupoPromocAtual) {
		    				// verifica se grupo promocional já foi inserido no array
		    				$chaveExiste = array_key_exists($chaveGrupo, $gruposPromocionais);
		    				if (!$chaveExiste) {
		    					foreach ($grupoPromocAtual['produtos'] as &$prodGrupoPromocAtual) {
		    						// carrega preço dos produtos do grupo promocional corretamente
		    						$preco =  self::buscaPreco($CDFILIAL, $CDCLIENTE, $prodGrupoPromocAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
                                    $prodGrupoPromocAtual['VRPRECITEM'] = $preco['PRECO'];
		    					}

		    					$gruposPromocionais[$chaveGrupo] = $grupoPromocAtual;
		    				}
		    			}
		    		}
    			}
    		}
    	}

    	return $gruposPromocionais;
    }

    private function atualizaPrecoGrupPromoc(&$cardapio, $gruposPromocionais) {
    	foreach ($cardapio as &$grupoAtual) {
            if(isset($grupoAtual['produtos'])) {
                foreach ($grupoAtual['produtos'] as &$produtoAtual) {
                    if ($produtoAtual['IDTIPOCOMPPROD'] == '3') {
                        // produto é combo
                        foreach ($produtoAtual['GRUPOS'] as $chaveGrupo => &$grupoPromocAtual) {
                            // percorre produtos do grupo promocional e atualiza preços
                            foreach ($grupoPromocAtual['produtos'] as $chaveProduto => &$prodGrupoPromocAtual) {
                                $prodGrupoPromocAtual['VRPRECITEM'] = $gruposPromocionais[$chaveGrupo]['produtos'][$chaveProduto]['VRPRECITEM'];
                            }
                        }
                    }
                }
            }
		}
    }

    public function precoDia($CDFILIAL, $CDTABEPREC, $CDPRODUTO, $DTINIVGPREC, $CDCLIENTE, $CDCONSUMIDOR) {
        $dataAtual   = $this->date->getDataAtual();
        $diaSeguinte = $this->date->adicionaIntervalo($dataAtual);

        $params = array(
            'CDCLIENTE'    => $CDCLIENTE,
            'CDCONSUMIDOR' => $CDCONSUMIDOR
        );

        // busca o tipo de consumidor do consumidor passado
        $retorno = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::TIPO_CONSUMIDOR, $params);
        if ($retorno) {
            $tipoConsumidor = $retorno["CDTIPOCONS"];
        } else {
            $tipoConsumidor = "T";
        }

        $params = array(
            'CDFILIAL'   => $CDFILIAL,
            'DTFERIFILI' => $dataAtual->format("d/m/Y")
        );

        // verifica se é feridado
        $retorno = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::FERIADO, $params);

        if ($retorno) {
            $dia = "F";
        } else {
            // verifica se o dia seguinte é feriado (significa que hoje é véspera de feriado)
            // comentado, Murillo do Madero falou que o desconto do happy hour aplica na véspera também

            /*
            $retorno = $this->entityManager->getConnection()->fetchAssoc (
                self::FERIADO, array($CDFILIAL, $diaSeguinte->format("d/m/Y"))
            );
            if ($retorno) {
                $dia = "V";
            } else {
            */
            // Expected: 1 (for Sunday) through 7 (for Saturday)
            // PHP returns: 1 (for Monday) through 7 (for Sunday)
            // So, this method changes the index for each day, changing from the format of return
            // used by PHP to Delphi format.
            $dia = $dataAtual->format("N") % 7 + 1;
            //}
        }

        // 1 domingo
        // 2 segunda
        // 3 terca
        // 4 quarta
        // 5 quinta
        // 6 sexta
        // 7 sabado
        // F - feriado
        // V - dia antes do feriado (véspera)

        $hora = $dataAtual->format("H:i:s");
        $hora = str_replace(":", "", $hora);
        $hora = substr($hora, 0, 4);
        // $hora = '1930';

        $params = array(
            'CDFILIAL'     => $CDFILIAL,
            'CDTABEPREC'   => $CDTABEPREC,
            'DTINIVGPREC'  => $DTINIVGPREC,
            'CDPRODUTO'    => $CDPRODUTO,
            'CDPRPAITABPR' => $CDPRODUTO,
            'NRDIASEMANPR' => (string) $dia,
            'CDTIPOCONSPD' => $tipoConsumidor,
            'HORA'         => $hora
        );

        $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::ITEM_PRECO_DIA_ORACLE, $params);

        if ($itemPrecoDia) {
            $retorno["PERC"]         = $itemPrecoDia["IDPERVALORPR"];
            $retorno["DESCACRE"]     = $itemPrecoDia["IDDESCACREPR"];
            $retorno["IDVISUACUPOM"] = $itemPrecoDia["IDVISUACUPOM"];
            $retorno["VRPRECODIA"]   = $itemPrecoDia["VRPRECODIA"];
        } else {
            $params = array(
                'CDFILIAL'     => $CDFILIAL,
                'CDTABEPREC'   => $CDTABEPREC,
                'DTINIVGPREC'  => $DTINIVGPREC,
                'CDPRODUTO'    => $CDPRODUTO,
                'CDPRPAITABPR' => $CDPRODUTO,
                'NRDIASEMANPR' => "T",
                'CDTIPOCONSPD' => $tipoConsumidor,
                'HORA'         => $hora
            );

            $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::ITEM_PRECO_DIA_ORACLE, $params);

            if ($itemPrecoDia) {
                $retorno["PERC"]         = $itemPrecoDia["IDPERVALORPR"];
                $retorno["DESCACRE"]     = $itemPrecoDia["IDDESCACREPR"];
                $retorno["IDVISUACUPOM"] = $itemPrecoDia["IDVISUACUPOM"];
                $retorno["VRPRECODIA"]   = $itemPrecoDia["VRPRECODIA"];
            } else {
                // Expected: 1 (for Sunday) through 7 (for Saturday)
                // PHP returns: 1 (for Monday) through 7 (for Sunday)
                // So, this method changes the index for each day, changing from the forat of return
                // used by PHP to Delphi format.
                $dia = (((int)$dataAtual->format("N")) % 7) + 1;

                $params = array(
                    'CDFILIAL'     => $CDFILIAL,
                    'CDTABEPREC'   => $CDTABEPREC,
                    'DTINIVGPREC'  => $DTINIVGPREC,
                    'CDPRODUTO'    => $CDPRODUTO,
                    'CDPRPAITABPR' => $CDPRODUTO,
                    'NRDIASEMANPR' => (string) $dia,
                    'CDTIPOCONSPD' => $tipoConsumidor,
                    'HORA'         => $hora
                );

                $itemPrecoDia = $this->entityManager->getConnection()->fetchAssoc(\Util\Query::ITEM_PRECO_DIA_ORACLE, $params);
                if ($itemPrecoDia) {
                    $retorno["PERC"]         = $itemPrecoDia["IDPERVALORPR"];
                    $retorno["DESCACRE"]     = $itemPrecoDia["IDDESCACREPR"];
                    $retorno["IDVISUACUPOM"] = $itemPrecoDia["IDVISUACUPOM"];
                    $retorno["VRPRECODIA"]   = $itemPrecoDia["VRPRECODIA"];
                } else {
                    $retorno = null;
                }
            }
        }

        return $retorno;
    }

}

