<?php

namespace Odhen\API\Service;

class ImpressaoSAT {

    protected $entityManager;
    protected $impressaoUtil;
    protected $databaseUtil;

    public function __construct(
        \Doctrine\ORM\EntityManager $entityManager,
        \Odhen\API\Lib\ImpressaoUtil $impressaoUtil,
        \Odhen\API\Util\Database $databaseUtil) {

        $this->entityManager = $entityManager;
        $this->impressaoUtil = $impressaoUtil;
        $this->databaseUtil = $databaseUtil;
    }

    const MOVIMENTACAO_ENTRADA = 'E';
    const MOVIMENTACAO_SAIDA = 'S';

    public function imprimeCupomNF($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $NRNOTAFISCALCE, $CDSERIESAT, $DTEMISSAONFCE, $NRACESSONFCE, $NRINSCRCONS, $NMCONSVEND, $CDSENHAPED, $DSQRCODENFCE, $impostoFederal, $impostoEstadual) {
        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'NRORG' => $NRORG
        );
        $dadosFilial = $this->entityManager->getConnection()->fetchAll(self::GET_DADOS_FILIAL, $params);
        $NMRAZSOCFILI = $dadosFilial[0]['NMRAZSOCFILI'];
        $NRINSJURFILI = $dadosFilial[0]['NRINSJURFILI'];
        $DSENDEFILI = $dadosFilial[0]['DSENDEFILI'];
        $NMBAIRFILI = $dadosFilial[0]['NMBAIRFILI'];
        $NMMUNICIPIO = $dadosFilial[0]['NMMUNICIPIO'];
        $SGESTADO = $dadosFilial[0]['SGESTADO'];
        $CDINSCESTA = $dadosFilial[0]['CDINSCESTA'];
        $NMFILIAL = $dadosFilial[0]['NMFILIAL'];
        $CDINSCMUNI = $dadosFilial[0]['CDINSCMUNI'];

        $params = array(
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA,
            'NRORG' => $NRORG
        );
        $dadosImpressora = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_IMPRESSORA, $params);
        if (!empty($dadosImpressora)) {
            $IDMODEIMPRES = $dadosImpressora[0]['IDMODEIMPRES'];
            $CDPORTAIMPR = $dadosImpressora[0]['CDPORTAIMPR'];
            $CDLOJA = $dadosImpressora[0]['CDLOJA'];
            $NMLOJA = $dadosImpressora[0]['NMLOJA'];
            $NMCAIXA = $dadosImpressora[0]['NMCAIXA'];

            $resposta = $this->impressaoUtil->iniciarPorta($IDMODEIMPRES, $CDPORTAIMPR);
            $portaUsbKiosk = $resposta['portaUsbKiosk'];

            if ($resposta['error'] == false) {
                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDCAIXA' => $CDCAIXA,
                    'NRORG' => $NRORG
                );
                $dadosCaixa = $this->entityManager->getConnection()->fetchAll(self::BUSCA_DADOS_CAIXA, $params);
                $DTABERCAIX = $dadosCaixa[0]['DTABERCAIX'];

                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDCAIXA' => $CDCAIXA,
                    'DTABERCAIX' => $DTABERCAIX,
                    'NRSEQVENDA' => $NRSEQVENDA,
                    'NRORG' => $NRORG
                );
                if ($this->databaseUtil->databaseIsOracle()) {
                    $movimentacao = $this->entityManager->getConnection()->fetchAll(self::BUSCA_RECEBIMENTOS_ORACLE, $params);
                } else {
                    $movimentacao = $this->entityManager->getConnection()->fetchAll(self::BUSCA_RECEBIMENTOS_MSDE, $params);
                }

                $params = array(
                    'CDFILIAL' => $CDFILIAL,
                    'CDCAIXA' => $CDCAIXA,
                    'NRSEQVENDA' => $NRSEQVENDA,
                    'NRORG' => $NRORG
                );
                $produtos = $this->entityManager->getConnection()->fetchAll(self::BUSCA_PRODUTOS, $params);

                $troco = 0;
                $recebimentos = array();
                foreach ($movimentacao as $movAtual) {
                    if ($movAtual['IDTIPOMOVIVE'] == self::MOVIMENTACAO_ENTRADA) {
                        array_push($recebimentos, $movAtual);
                    } else if ($movAtual['IDTIPOMOVIVE'] == self::MOVIMENTACAO_SAIDA) {
                        $troco += $movAtual['VRMOVIVEND'];
                    }
                }

                $printerParams = $this->impressaoUtil->buscaParametrosImpressora($IDMODEIMPRES);
                $texto = '';
                $texto .= self::cabecalhoCupomNF($printerParams, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $NMRAZSOCFILI, $NRINSJURFILI, $DSENDEFILI, $NMBAIRFILI, $NMMUNICIPIO, $SGESTADO, $CDINSCESTA, $CDINSCMUNI);
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'Extrato No. ' . $NRNOTAFISCALCE) . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'CUPOM FISCAL ELETRONICO - SAT') . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->imprimeLinha($printerParams, $texto);
                if (empty($NRINSCRCONS)) {
                    $texto .= $this->impressaoUtil->centraliza($printerParams, 'CONSUMIDOR NAO IDENTIFICADO') . $printerParams['comandoEnter'];
                } else {
                    if (!empty($NMCONSVEND)) {
                        $texto .= $this->impressaoUtil->centraliza($printerParams, $NMCONSVEND) . $printerParams['comandoEnter'];
                    }
                    if (strlen($NRINSCRCONS) > 8) {
                    	// CPF
                    	$texto .= $this->impressaoUtil->centraliza($printerParams, 'CPF: ' . substr($NRINSCRCONS, 0, 3) . '.' . substr($NRINSCRCONS, 3, 3) . '.' . substr($NRINSCRCONS, 6, 3) . '-' . substr($NRINSCRCONS, 9, 2)) . $printerParams['comandoEnter'];
                    } else {
                    	// CNPJ
                    	$texto .= $this->impressaoUtil->centraliza($printerParams, 'CNPJ: ' . $NRINSCRCONS) . $printerParams['comandoEnter'];
                    }
                }
                $texto .= $this->impressaoUtil->centraliza($printerParams, '# | COD | DESC | QTD | UN | VL UNIT R$ | ST | ALIQ | VL ITEM R$') . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->imprimeLinha($printerParams, $texto);
                $texto .= self::itensCupomNF($produtos, $printerParams, $recebimentos, $troco);
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'OBSERVACOES DO CONTRIBUINTE') . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'TRIBUTACAO APROXIMADA R$ ' . $this->impressaoUtil->formataNumero($impostoFederal, 2) . ' FEDERAL E R$ ' . $this->impressaoUtil->formataNumero($impostoEstadual, 2) . ' ESTADUAL') . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->centraliza($printerParams, '(CONFORME LEI FEDERAL 12.741/2012) - FONTE IBPT') . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->imprimeLinha($printerParams, $texto);
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'SAT No. ' . substr($CDSERIESAT, 0, 3) . '.' . substr($CDSERIESAT, 3, 3) . '.' . substr($CDSERIESAT, 6, 3)) . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->centraliza($printerParams, $DTEMISSAONFCE->format('d/m/Y H:i:s')) . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->centraliza($printerParams, $NRACESSONFCE) . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->preencheLinha($printerParams, '', ' ');
                $this->impressaoUtil->imprimeTexto($texto, $IDMODEIMPRES, $portaUsbKiosk);
                $this->impressaoUtil->configuraCodigoBarras($IDMODEIMPRES, 40, 0, 0, 0, 25);
                $this->impressaoUtil->imprimeCodigoBarrasCODE128($IDMODEIMPRES, substr($NRACESSONFCE, 4, 22), $portaUsbKiosk);
                $this->impressaoUtil->imprimeCodigoBarrasCODE128($IDMODEIMPRES, substr($NRACESSONFCE, 26, 22), $portaUsbKiosk);
                $this->impressaoUtil->configuraCodigoBarras($IDMODEIMPRES, 60, 2, 2, 0, 174);
                $this->impressaoUtil->imprimeQrCode($IDMODEIMPRES, $DSQRCODENFCE, $portaUsbKiosk);
                $texto = $printerParams['comandoEnter'];
                $texto .= 'Filial: ' . $CDFILIAL . ' ' . $NMFILIAL . $printerParams['comandoEnter'];
                $texto .= 'Loja..: ' . $CDLOJA . ' ' . $NMLOJA . $printerParams['comandoEnter'];
                $texto .= 'Caixa.: ' . $CDCAIXA . ' ' . $NMCAIXA . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                $texto .= $this->impressaoUtil->centraliza($printerParams, 'Consulte o QRCODE deste extrato atraves do app DeOlhoNaNota') . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                $this->impressaoUtil->imprimeTexto($texto, $IDMODEIMPRES, $portaUsbKiosk);
                $texto = $this->impressaoUtil->centraliza($printerParams, 'SENHA: ' . $CDSENHAPED, 24) . $printerParams['comandoEnter'];
                $this->impressaoUtil->imprimeTexto($texto, $IDMODEIMPRES, $portaUsbKiosk, 0, 0, 1, 1, 3);
                $texto = $printerParams['comandoEnter'] . $printerParams['comandoEnter'] . $printerParams['comandoEnter'] . $printerParams['comandoEnter'];
                $this->impressaoUtil->imprimeTexto($texto, $IDMODEIMPRES, $portaUsbKiosk);
                if ($IDMODEIMPRES != 20) {
                    $this->impressaoUtil->cortaPapel($IDMODEIMPRES, $portaUsbKiosk);
                }
                $this->impressaoUtil->fechaPorta($IDMODEIMPRES, $portaUsbKiosk);

                $result = array(
                    'error' => false
                );
            } else {
                $result = $resposta;
            }
        } else {
            $result = array(
                'error' => true,
                'message' => 'Impressora SAT não parametrizada para o caixa.'
            );
        }
        return $result;
    }

    const GET_DADOS_FILIAL = "
        SELECT F.NRINSJURFILI, UPPER(F.NMRAZSOCFILI) NMRAZSOCFILI, UPPER(F.NMFILIAL) NMFILIAL, UPPER(EF.DSENDEFILI) DSENDEFILI,
               EF.NRENDEFILISEP,UPPER(EF.NMBAIRFILI) NMBAIRFILI,M.CDMUNICIBGE,UPPER(M.NMMUNICIPIO) NMMUNICIPIO, EF.SGESTADO,
               EF.NRCEPFILI, P.CDPAISBACEN,UPPER(P.NMPAIS) NMPAIS,F.CDINSCESTA,F.CDINSCMUNI,E.CDCNAE,PE.CDSITUCRT, ES.CDESTADOIBGE
          FROM FILIAL F, ENDEFILI EF, MUNICIPIO M, PAIS P, PARAMEMPRESA PE, EMPRESA E, ESTADO ES
         WHERE F.CDFILIAL      = :CDFILIAL
           AND F.NRORG         = :NRORG
           AND EF.CDFILIAL     = F.CDFILIAL
           AND EF.NRORG        = F.NRORG
           AND EF.IDTPENDEFILI = 'P'
           AND ES.SGESTADO     = EF.SGESTADO
           AND ES.CDPAIS       = EF.CDPAIS
           AND M.CDMUNICIPIO   = EF.CDMUNICIPIO
           AND M.SGESTADO      = EF.SGESTADO
           AND P.CDPAIS        = EF.CDPAIS
           AND E.CDEMPRESA     = F.CDEMPRESA
           AND E.NRORG         = F.NRORG
           AND PE.CDEMPRESA    = E.CDEMPRESA
           AND PE.NRORG        = E.NRORG
    ";

    const BUSCA_DADOS_IMPRESSORA = "
        SELECT P.NRSEQIMPRLOJA, I.IDMODEIMPRES, P.CDPORTAIMPR, L.CDLOJA, UPPER(L.NMLOJA) AS NMLOJA, UPPER(C.NMCAIXA) AS NMCAIXA
          FROM CAIXA C JOIN IMPRLOJA P
                         ON C.CDFILIAL = P.CDFILIAL
                        AND C.CDLOJA = P.CDLOJA
                        AND C.NRSEQIMPRLOJA3 = P.NRSEQIMPRLOJA
                       JOIN IMPRESSORA I
                         ON P.CDIMPRESSORA = I.CDIMPRESSORA
                       JOIN LOJA L
                         ON L.CDLOJA = C.CDLOJA
                        AND L.CDFILIAL = C.CDFILIAL
         WHERE C.CDFILIAL = :CDFILIAL
           AND C.CDCAIXA = :CDCAIXA
           AND C.NRORG = :NRORG
    ";

    const BUSCA_DADOS_CAIXA = "
        SELECT MAX(DTABERCAIX) AS DTABERCAIX
          FROM TURCAIXA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRORG = :NRORG
    ";

    const BUSCA_RECEBIMENTOS_ORACLE = "
        SELECT T.DSIMPFISCAL, T.NMTIPORECE, M.VRMOVIVEND, M.IDTIPOMOVIVE
          FROM MOVCAIXA M JOIN TIPORECE T
                            ON M.CDTIPORECE = T.CDTIPORECE
         WHERE M.CDFILIAL = :CDFILIAL
           AND M.CDCAIXA = :CDCAIXA
           AND M.DTABERCAIX = :DTABERCAIX
           AND M.NRSEQVENDA = :NRSEQVENDA
           AND M.NRORG = :NRORG
           AND M.IDTIPOMOVIVE IN ('E', 'S')
    ";

    const BUSCA_RECEBIMENTOS_MSDE = "
        SELECT T.DSIMPFISCAL, T.NMTIPORECE, M.VRMOVIVEND, M.IDTIPOMOVIVE
          FROM MOVCAIXA M JOIN TIPORECE T
                            ON M.CDTIPORECE = T.CDTIPORECE
         WHERE M.CDFILIAL = :CDFILIAL
           AND M.CDCAIXA = :CDCAIXA
           AND M.DTABERCAIX = CONVERT(DATETIME, :DTABERCAIX, 120)
           AND M.NRSEQVENDA = :NRSEQVENDA
           AND M.NRORG = :NRORG
           AND M.IDTIPOMOVIVE IN ('E', 'S')
    ";

    const BUSCA_PRODUTOS = "
        SELECT P.CDARVPROD, P.NMPRODUTO, P.SGUNIDADE, I.QTPRODVEND,
               I.VRUNITVEND, I.VRDESITVEND, I.VRACRITVEND, A.IDTPIMPOSFIS, A.VRPEALIMPFIS
          FROM ITEMVENDA I JOIN PRODUTO P
                             ON I.CDPRODUTO = P.CDPRODUTO
                           JOIN ALIQIMPFIS A
                             ON I.CDFILIAL = A.CDFILIAL
                            AND I.CDPRODUTO = A.CDPRODUTO
         WHERE I.CDFILIAL = :CDFILIAL
           AND I.CDCAIXA = :CDCAIXA
           AND I.NRSEQVENDA = :NRSEQVENDA
           AND I.NRORG = :NRORG
    ";

    private function itensCupomNF($produtos, $printerParams, $recebimentos, $troco) {
        $corpo = '';
        $qtdItens = count($produtos);
        $totalVendaSemDesconto = 0;
        $descontoVenda = 0;
        foreach ($produtos as $index => $item) {
            $totalVendaSemDesconto += ($item['VRUNITVEND'] * $item['QTPRODVEND']) + $item['VRACRITVEND'];
            $descontoVenda += $item['VRDESITVEND'];

            $nrItem = str_pad($index + 1, 3, '0', STR_PAD_LEFT);
            $newItem = $nrItem . " " . $item['CDARVPROD'] . " " . $item['NMPRODUTO'];

            if ($item['SGUNIDADE'] != 'UN') {
                $qtd = $this->impressaoUtil->formataNumero($item['QTPRODVEND'], 3);
            } else {
                $qtd = intval($item['QTPRODVEND']);
            }
            $valorImposto = $item['VRPEALIMPFIS'] > 0 ?  $this->impressaoUtil->formataNumero($item['VRPEALIMPFIS'], 2) : $item['VRPEALIMPFIS'];
            $impostoItem = $item['IDTPIMPOSFIS'] . ' ' . $valorImposto . '%';
            $valorItemCalculado = $this->impressaoUtil->formataNumero($item['QTPRODVEND'] * $item['VRUNITVEND'], 2);
            $finalItem = $qtd . ' ' . $item['SGUNIDADE'] . " X " . $this->impressaoUtil->formataNumero($item['VRUNITVEND'], 2) . ' ' . $impostoItem . ' ' . $valorItemCalculado;
            $textoAlinhado = $this->impressaoUtil->alinhaInicioFim($printerParams, $newItem, $finalItem);
            $corpo .=  $textoAlinhado . $printerParams['comandoEnter'];
            $total = $item['QTPRODVEND'] * round($item['VRUNITVEND'], 2);
            if ($item['VRDESITVEND'] > 0) {
                $corpo .= $this->impressaoUtil->preencheLinha($printerParams, 'Desconto no item ' . $nrItem . ' - '. $item['VRDESITVEND'], ' ');
            }
        }
        $corpo .= $this->impressaoUtil->preencheLinha($printerParams, 'Vr. Total Pago: = ' . $this->impressaoUtil->formataNumero($totalVendaSemDesconto - $descontoVenda, 2), ' ');
        $corpo .= $this->impressaoUtil->preencheLinha($printerParams, 'Valor de Troco: = ' . $this->impressaoUtil->formataNumero($troco, 2), ' ');
        $inicio = 'SUBTOTAL R$ ';
        $final = $this->impressaoUtil->formataNumero($totalVendaSemDesconto, 2);
        $textoAlinhado = $this->impressaoUtil->alinhaInicioFim($printerParams, $inicio, $final);
        $corpo .= $textoAlinhado . $printerParams['comandoEnter'];
        if ($descontoVenda > 0) {
            $corpo .= 'DESCONTOS R$ ' . $this->impressaoUtil->formataNumero($descontoVenda, 2) . $printerParams['comandoEnter'];
        }
        $inicio = 'TOTAL R$ ';
        $final = $this->impressaoUtil->formataNumero($totalVendaSemDesconto - $descontoVenda, 2);
        $textoAlinhado = $this->impressaoUtil->alinhaInicioFim($printerParams, $inicio, $final);
        $corpo .= $textoAlinhado . $printerParams['comandoEnter'];
        foreach ($recebimentos as $recebimentoAtual) {
            $nomeTiporece = $recebimentoAtual['NMTIPORECE'];
            $valorRecebimento = $this->impressaoUtil->formataNumero($recebimentoAtual['VRMOVIVEND'], 2);
            $textoAlinhado = $this->impressaoUtil->alinhaInicioFim($printerParams, $nomeTiporece, $valorRecebimento);
            $corpo .= $textoAlinhado . $printerParams['comandoEnter'];
        }
        $corpo .= $this->impressaoUtil->imprimeLinha($printerParams, $corpo);
        return $corpo;
    }

    public function cabecalhoCupomNF($printerParams, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $NMRAZSOCFILI, $NRINSJURFILI, $DSENDEFILI, $NMBAIRFILI, $NMMUNICIPIO, $SGESTADO, $CDINSCESTA, $CDINSCMUNI) {
        $cabecalho = '';
        $cabecalho .= $this->impressaoUtil->centraliza($printerParams, $NMRAZSOCFILI) . $printerParams['comandoEnter'];
        $cabecalho .= $this->impressaoUtil->centraliza($printerParams, $DSENDEFILI) . $printerParams['comandoEnter'];
        $cabecalho .= $this->impressaoUtil->centraliza($printerParams, $NMBAIRFILI . ' - ' . $NMMUNICIPIO . ' - ' . $SGESTADO) . $printerParams['comandoEnter'];
        $cabecalho .= $this->impressaoUtil->centraliza($printerParams, 'CNPJ: ' . $NRINSJURFILI . ' IE: ' . $CDINSCESTA . ' IM: '. $CDINSCMUNI) . $printerParams['comandoEnter'];
        $cabecalho .= $this->impressaoUtil->imprimeLinha($printerParams, $cabecalho);
        return $cabecalho;
    }

}