<?php

namespace Odhen\API\Service;

class Venda {

	protected $entityManager;
	protected $notaFiscalService;
	protected $estoque;
	protected $getParametro;
	protected $util;
	protected $sat;
	protected $databaseUtil;
	protected $precoService;

 	public function __construct(
 		\Doctrine\ORM\EntityManager $entityManager,
 		\Odhen\API\Service\NotaFiscal $notaFiscal,
 		\Odhen\API\Service\Estoque $estoque,
 		\Odhen\API\Util\GetParametro $getParametro,
 		\Odhen\API\Util\Util $util,
 		\Odhen\API\Service\SAT $sat,
 		\Odhen\API\Util\Database $databaseUtil,
 		\Odhen\API\Service\Preco $precoService) {

		$this->entityManager = $entityManager;
		$this->notaFiscalService = $notaFiscal;
		$this->estoque = $estoque;
		$this->getParametro = $getParametro;
		$this->util = $util;
		$this->sat = $sat;
		$this->databaseUtil = $databaseUtil;
		$this->precoService = $precoService;
	}

	const APROVADA = 'O';
    const ENTRADA = 'E';
    const SINCRONIZA_ESTOQUE = 'S';
	const ITEM_APROVADO = 'A';
	const ITEM_ABASTECIMENTO = 'A';
	const TIPO_NFCE = 'FNC';
	const TIPO_SAT = 'SAT';
	const OBS_ADICIONA_PRODUTO = 'A';

	private $refreshTaxVRTOTTRIBIBPT;

	public function venda($NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA, $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA, $IDTPEMISVEND, $TOTALVENDA, $ITEMVENDA, $FORMADEPAGAMENTO, $NMCONSVEND, $CDNSUHOSTTEF, $NRINSCRCONS, $CDSENHAPED, $NRSEQVENDA, $VRTROCOVEND, $EMAIL, $IDENTIFICACAO, $VRDESCVENDA, $IDMONGO, $NRDATABASE, $CDCLIENTE, $CDCONSUMIDOR, $simulaImpressao, $simulateSaleValidation) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$precosProdutos = self::validaBuscaPrecosProdutos($ITEMVENDA, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);

			if ($precosProdutos['error'] == false) {
				$ITEMVENDA = $precosProdutos['produtosComPreco'];
				$impostos = self::calculaImpostoParaImpressao($NRORG, $CDFILIAL, $ITEMVENDA, $TOTALVENDA, $VRDESCVENDA, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);
				$VRTOTTRIBIBPT = $impostos['VRTOTTRIBIBPT'];
				$impostoEstadual = $impostos['impostoEstadual'];

				$respostaVenda = self::realizaVenda(
					$NRORG,
					$CDFILIAL,
					$CDLOJA,
					$CDCAIXA,
					$CDVENDEDOR,
					$CDOPERADOR,
					$DTABERCAIX,
					$DTVENDA,
					$IDTPEMISVEND,
					$TOTALVENDA,
					$ITEMVENDA,
					$FORMADEPAGAMENTO,
					$NMCONSVEND,
					$CDNSUHOSTTEF,
					$NRINSCRCONS,
					$CDSENHAPED,
					$NRSEQVENDA,
					$VRTROCOVEND,
					$EMAIL,
					$IDENTIFICACAO,
					$VRDESCVENDA,
					$IDMONGO,
					$NRDATABASE,
					$VRTOTTRIBIBPT,
					$CDCLIENTE,
					$CDCONSUMIDOR
				);

				$dadosCaixa = self::buscaDadosCaixa($CDFILIAL, $CDCAIXA, $NRORG);

				$IDTPEMISSAOFOS = $dadosCaixa['IDTPEMISSAOFOS'];
				if (!$simulateSaleValidation) {
					if ($IDTPEMISSAOFOS == self::TIPO_SAT) {
						$respostaTransmissao = $this->sat->vendaSAT($CDFILIAL, $CDCAIXA, $respostaVenda['NRSEQVENDA'], $NRORG, $NRINSCRCONS, $NMCONSVEND, $CDSENHAPED, $CDOPERADOR, $VRTOTTRIBIBPT, $impostoEstadual, $simulaImpressao);
						if ($respostaTransmissao['error'] == false) {
							$result = array(
								'error' => false,
								'NRSEQVENDA' => $respostaVenda['NRSEQVENDA']
							);
						} else {
							$result = $respostaTransmissao;
						}
					} else if ($IDTPEMISSAOFOS == self::TIPO_NFCE) {
			    		$respostaTransmissao = $this->notaFiscalService->transmitirNFCe($NRORG, $CDCAIXA, $CDFILIAL, $DTABERCAIX, $CDLOJA, $DTVENDA, $VRDESCVENDA, $NRINSCRCONS, $ITEMVENDA, $FORMADEPAGAMENTO, $CDOPERADOR, $IDTPEMISVEND, $respostaVenda['NRSEQVENDA'], $CDNSUHOSTTEF, $VRTOTTRIBIBPT , $TOTALVENDA, $simulaImpressao);
			    		if ($respostaTransmissao['error'] == false) {
			    			$result = array(
								'error' => false,
								'NRSEQVENDA' => $respostaVenda['NRSEQVENDA']
							);
				    		if ($EMAIL['ENVEMAIL'] != null) {
				    			$this->util->sendEmailVenda($EMAIL['EMAIL'], $respostaTransmissao, $DTVENDA, $respostaVenda['NRSEQVENDA']);
				    		}
			    		} else {
			    			$result = $respostaTransmissao;
			    		}
					} else {
						$result = array(
							'error' => false,
							'NRSEQVENDA' => $respostaVenda['NRSEQVENDA']
						);
					}
				} else {
					$result = array(
						'error' => false,
						'NRSEQVENDA' => $respostaVenda['NRSEQVENDA']
					);
				}
				if ($result['error'] == true) {
					$respostaCancelamento = self::cancelaVenda($CDFILIAL, $CDCAIXA, $NRORG, $respostaVenda['NRSEQVENDA'], $CDOPERADOR);
					if ($respostaCancelamento['error'] == true) {
						$result['message'] = $result['message'] . ' - ' . $respostaCancelamento['message'];
					}
				}
			} else {
				$result = $precosProdutos;
			}

			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollBack();
			$result = array(
				'error' => true,
				'message' => $e->getMessage()
			);
 		}
 		return $result;
	}

	private function validaBuscaPrecosProdutos($produtosDaVenda, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR) {
		$retorno = array(
			'error' => false
		);
		foreach ($produtosDaVenda as &$produtoAtual) {
			if ($produtoAtual['IDTIPOCOMPPROD'] == '3') {
				// itens do combo
				foreach ($produtoAtual['itensCombo'] as &$itemComboAtual) {
					if ($produtoAtual['IDIMPPRODUTO'] == '2') {
						$precoItemCombo = $this->precoService->buscaPreco($CDFILIAL, $CDCLIENTE, $itemComboAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
						if ($precoItemCombo['error'] == false) {
							$itemComboAtual['VRUNITVEND'] = $precoItemCombo['PRECO'];
						} else {
							$retorno = $precoItemCombo;
							break 2;
						}
					}
					$observacoesComPreco = self::colocaPrecoNasObs($itemComboAtual['OBSERVACOES'], $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);
					if ($observacoesComPreco['error'] == false) {
						$itemComboAtual['OBSERVACOES'] = $observacoesComPreco['OBSERVACOES'];
					} else {
						$retorno = $observacoesComPreco;
						break 2;
					}
				}
			} else {
				// produto normal
				$precoItem = $this->precoService->buscaPreco($CDFILIAL, $CDCLIENTE, $produtoAtual['CDPRODUTO'], $CDLOJA, $CDCONSUMIDOR);
				if ($precoItem['error'] == false) {
					$produtoAtual['VRUNITVEND'] = $precoItem['PRECO'];
					$observacoesComPreco = self::colocaPrecoNasObs($produtoAtual['OBSERVACOES'], $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR);
					if ($observacoesComPreco['error'] == false) {
						$produtoAtual['OBSERVACOES'] = $observacoesComPreco['OBSERVACOES'];
					} else {
						$retorno = $observacoesComPreco;
						break;
					}
				} else {
					$retorno = $precoItem;
					break;
				}
			}
		}
		if ($retorno['error'] == false) {
			$retorno['produtosComPreco'] = $produtosDaVenda;
		}
		return $retorno;
	}

	private function colocaPrecoNasObs($observacoes, $CDFILIAL, $CDLOJA, $CDCLIENTE, $CDCONSUMIDOR) {
		$retorno = array(
			'error' => false
		);
		foreach ($observacoes as &$obsAtual) {
			$obsCDPRODUTO = self::buscaProdutoObs($obsAtual['CDGRPOCOR'], $obsAtual['CDOCORR']);
			if ($obsCDPRODUTO) {
				$precoObs = $this->precoService->buscaPreco($CDFILIAL, $CDCLIENTE, $obsCDPRODUTO, $CDLOJA, $CDCONSUMIDOR);
				if ($precoObs['error'] == false) {
					$obsAtual['VRUNITVEND'] = $precoObs['PRECO'];
				} else {
					$retorno = $precoObs;
					break;
				}
			}
		}
		if ($retorno['error'] == false) {
			$retorno['OBSERVACOES'] = $observacoes;
		}
		return $retorno;
	}

	private function cancelaVenda($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $CDOPERADOR) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRORG' => $NRORG,
			'NRSEQVENDA' => $NRSEQVENDA
		);
		$venda = $this->entityManager->getConnection()->fetchAll(self::BUSCA_VENDA, $params);
		$IDIMPVENDA = $venda[0]['IDIMPVENDA'] == 'I' ? 'I' : 'A';

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRORG' => $NRORG,
			'NRSEQVENDA' => $NRSEQVENDA,
			'CDOPERULTATU' => $CDOPERADOR,
			'IDIMPVENDA' => $IDIMPVENDA,
			'IDSITUVENDA' => 'C',
			'IDSTATUSNFCE' => 'R'
		);
		$this->entityManager->getConnection()->executeQuery(self::CANCELA_VENDA, $params);

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'CDOPERULTATU' => $CDOPERADOR,
			'NRORG' => $NRORG,
			'NRSEQVENDA' => $NRSEQVENDA,
			'IDTIPOMOVIVE' => 'C'
		);
		$this->entityManager->getConnection()->executeQuery(self::CANCELA_MOVCAIXA, $params);

		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRORG' => $NRORG,
			'NRSEQVENDA' => $NRSEQVENDA
		);
		$MOVICLIE = $this->entityManager->getConnection()->fetchAll(self::BUSCA_MOVICLIE, $params);

		// Caso venda for do tipo 'DÉBITO CONSUMIDOR'
		if (count($MOVICLIE) > 0) {
			$VRSALDOCONS = $MOVICLIE[0]['VRSALDOCONS'];
			$VRMOVCLI = $MOVICLIE[0]['VRMOVCLI'];
			$VRSALDOCONS = $VRSALDOCONS + $VRMOVCLI;
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDCAIXA' => $CDCAIXA,
				'CDOPERULTATU' => $CDOPERADOR,
				'NRORG' => $NRORG,
				'NRSEQVENDA' => $NRSEQVENDA,
				'IDTIPMOCVLI' => 'X',
				'VRSALDOCONS' => $VRSALDOCONS
			);
			$this->entityManager->getConnection()->executeQuery(self::CANCELA_MOVICLIE, $params);
		}

		$result = array(
			'error' => false
		);
		return $result;
	}

    const BUSCA_VENDA = "
        SELECT CDFILIAL, CDLOJA, CDCAIXA, DTENTRVENDA,
               DTVENDA, IDIMPVENDA, DTABERTUR, CDFILIALTUR,
               CDCAIXATUR, CDCLIENTE, IDSITUVENDA, NRSEQVENDA,
               NRORG, CDEMPRESA, VRDESCVENDA, CDOPERADOR,
               NRINSJURIEST, IDTPEMISVEND
          FROM VENDA
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRORG = :NRORG
           AND NRSEQVENDA = :NRSEQVENDA
    ";

    const CANCELA_VENDA = "
        UPDATE VENDA
           SET IDIMPVENDA = :IDIMPVENDA,
               IDSITUVENDA = :IDSITUVENDA,
               IDSTATUSNFCE = :IDSTATUSNFCE,
               CDOPERULTATU = :CDOPERULTATU,
               NRORGULTATU = :NRORG
         WHERE NRSEQVENDA = :NRSEQVENDA
           AND CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRORG = :NRORG
    ";

    const CANCELA_MOVCAIXA = "
        UPDATE MOVCAIXA
           SET IDTIPOMOVIVE = :IDTIPOMOVIVE,
               CDOPERULTATU = :CDOPERULTATU,
               NRORGULTATU = :NRORG
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRORG = :NRORG
           AND NRSEQVENDA = :NRSEQVENDA
    ";

    const BUSCA_MOVICLIE = "
        SELECT CDCLIENTE, NRSEQMOVCLI, NRSEQVENDA, NRSEQUMOVI,
               CDCONSUMIDOR, VRMOVCLI, VRSALDOCONS, DTMOVCLI,
               IDTIPMOCVLI
          FROM MOVICLIE
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRORG = :NRORG
           AND NRSEQVENDA = :NRSEQVENDA
    ";

    const CANCELA_MOVICLIE = "
        UPDATE MOVICLIE
           SET IDTIPMOCVLI = :IDTIPMOCVLI,
               VRSALDOCONS = :VRSALDOCONS,
               CDOPERULTATU = :CDOPERULTATU,
               NRORGULTATU = :NRORG
         WHERE CDFILIAL = :CDFILIAL
           AND CDCAIXA = :CDCAIXA
           AND NRORG = :NRORG
           AND NRSEQVENDA = :NRSEQVENDA
    ";

	private function buscaDadosCaixa($CDFILIAL, $CDCAIXA, $NRORG) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRORG' => $NRORG
		);
		return $this->entityManager->getConnection()->fetchAssoc(self::GET_DADOS_CAIXA, $params);
	}

	const GET_DADOS_CAIXA = "
		SELECT IDTPEMISSAOFOS, IDTPEQUSAT
		  FROM CAIXA
		 WHERE CDFILIAL = :CDFILIAL
		   AND CDCAIXA = :CDCAIXA
	       AND NRORG = :NRORG
	";

	private function trunc($val, $f = "2") {
	    if (($p = strpos($val, '.')) !== false) {
	        $val = floatval(substr($val, 0, $p + 1 + $f));
	    }
	    return $val;
	}

	private function calculaImpostoParaImpressao($NRORG, $CDFILIAL, $ITEMVENDA, $TOTALVENDA, $VRDESCVENDA) {
		$params = array(
			':NRORG' => $NRORG,
			':CDFILIAL'  => $CDFILIAL
		);
		$IDCUMUPISCOFIL = $this->entityManager->getConnection()->fetchAssoc(self::GET_IDCUMUPISCOFIL, $params);
		$IDCUMUPISCOFIL = $IDCUMUPISCOFIL['IDCUMUPISCOFIL'];

	    $VRTXSERVICO = 0;
	    $VRACRCOMANDA = 0;
   		$descontoTotal = 0 ;
   		$acrescimoTotal = 0 ;
   		$ICMSTotal = 0;
   		$pis = 0; // 3,14s?  HAHA I made a joke (the mathematic guy)
   		$cofins = 0;
   		$totalFederal = 0;
   		$totalEstadual = 0;

   		$itensImposto = self::separaItensQueSeraoCobrados($ITEMVENDA);
	    foreach ($itensImposto as $itemAtual) {
	    	$porcDescItem = 0;
			$porcAcreItem = 0;

	    	$TOTALITEM = self::trunc(($itemAtual['VRUNITVEND'] * $itemAtual['QTPRODVEND']) - $itemAtual['VRDESITVEND']);
	    	if ($TOTALVENDA > 0) {
		        $porcDescItem = self::trunc(($TOTALITEM * $VRDESCVENDA) / $TOTALVENDA);
		        $porcAcreItem = self::trunc(($TOTALITEM * $VRACRCOMANDA) / $TOTALVENDA);
	    	}
			$params = array(
				':CDPRODUTO' => $itemAtual['CDPRODUTO'],
				':NRORG' => $NRORG,
				':CDFILIAL' => $CDFILIAL
			);
	    	$impostoProduto = $this->entityManager->getConnection()->fetchAssoc(self::GET_IMPOSTOS_PRODUTO, $params);
		   	// Real(Nao Cumulativo) = 'S' e Presumido(Cumulativo) = 'N'
	        if ($impostoProduto['IDINCIDEPISCOF'] = 'N'){
		        $VALORPIS = 0.65;
		        $VALORCOFINS = 3;
	        }else{
	         	$VALORPIS = 1.65;
				$VALORCOFINS = 7.60;
	        }

	        $descontoTotal = $descontoTotal  + $porcDescItem;
		    $acrescimoTotal = $acrescimoTotal + $porcAcreItem;
		    $ICMSitem = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($impostoProduto['VRPEALIMPFIS'] / 100));
		    $ICMSTotal = $ICMSTotal + $ICMSitem;

		    if (($impostoProduto['VRALIQIBPT'] == 0) || (empty($impostoProduto['VRALIQIBPT']))) {
		    	$pis = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($VALORPIS / 100));
		    	$cofins = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($VALORCOFINS / 100));
		    	$totalFederal = $totalFederal + $ICMSitem + $pis  + $cofins;
		    } else {
		    	$IBPT = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($impostoProduto['VRALIQIBPT']) / 100);
		    	$totalFederal = $totalFederal + $IBPT;
		    }

		    if (($impostoProduto['VRALIQIBPTES'] == 0) || (empty($impostoProduto['VRALIQIBPTES']))) {
		    	$IBPTES = 0;
		    	$totalEstadual = $totalEstadual + $IBPTES;
		    } else {
		    	$IBPTES = (($TOTALITEM - $porcDescItem + $porcAcreItem) * ($impostoProduto['VRALIQIBPTES']) / 100);
		    	$totalEstadual = $totalEstadual + $IBPTES;
		    }
		}

	    return array(
			'VRTOTTRIBIBPT' => (self::trunc($totalFederal) + self::trunc($totalEstadual)),
			'impostoEstadual' => self::trunc($ICMSTotal)
	    );
	}

	const GET_IDCUMUPISCOFIL = "
		SELECT IDCUMUPISCOFIL
		  FROM FILIAL
		 WHERE CDFILIAL = :CDFILIAL
	       AND NRORG = :NRORG
	";

	const GET_IMPOSTOS_PRODUTO = "
		SELECT VRALIQIBPT, VRALIQIBPTES, IDINCIDEPISCOF, VRPEALIMPFIS
		  FROM ALIQIMPFIS
		 WHERE CDFILIAL = :CDFILIAL
		   AND CDPRODUTO = :CDPRODUTO
		   AND NRORG = :NRORG
	";

	private function separaItensQueSeraoCobrados($itens) {
		$itensImposto = array();
		foreach ($itens as $itemAtual) {
			// se for promoção e será cobrado os itens filhos
			if (($itemAtual['IDTIPOCOMPPROD'] == '3') && ($itemAtual['IDIMPPRODUTO'] == '2')) {
				foreach ($itemAtual['itensCombo'] as $itemComboAtual) {
					array_push($itensImposto, $itemComboAtual);
				}
			} else {
				array_push($itensImposto, $itemAtual);
			}
		}
		return $itensImposto;
	}

	private function realizaVenda($NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA, $CDVENDEDOR, $CDOPERADOR, $DTABERCAIX, $DTVENDA, $IDTPEMISVEND, $TOTALVENDA, $ITEMVENDA, $FORMADEPAGAMENTO, $NMCONSVEND, $CDNSUHOSTTEF, $NRINSCRCONS, $CDSENHAPED, $NRSEQVENDA, $VRTROCOVEND, $EMAIL, $IDENTIFICACAO, $VRDESCVENDA, $IDMONGO, $NRDATABASE, $VRTOTTRIBIBPT, $CDCLIENTE, $CDCONSUMIDOR) {
		$DTABERVENDA = $this->databaseUtil->getCurrentDateTime();

	    if (isset($IDENTIFICACAO)) {
	    	$NRPARCNEGOCIO = isset($IDENTIFICACAO['NRPARCNEGOCIOCLI']) ? $IDENTIFICACAO['NRPARCNEGOCIOCLI'] : null;
	    	$atributosIdent = isset($IDENTIFICACAO['ATRIBUTOVENDA']) ? $IDENTIFICACAO['ATRIBUTOVENDA'] : null;
	    } else {
	    	$NRPARCNEGOCIO = null;
	    	$atributosIdent = null;
	    }
	    $IDIMPVENDA = 'I'; // I (incluida) - A (alterada) - S (esportada com S)
	    $IDSITUVENDA = self::APROVADA;
	    $IDMONGO_SAAS = null;
	    if (isset($IDMONGO)) {
	    	$IDMONGO_SAAS = $NRDATABASE . '_' . $CDFILIAL . '_' . $NRORG . '_' . $IDMONGO;
	    }
	    // @TODO - MELHORAR ISSO
	    $IDORIGEMVENDA  = 'BAL_AUT';

	    // Dados consumidor
	    $NMCONSVEND = isset($NMCONSVEND) ? $NMCONSVEND : null;
		$NRINSCRCONS = isset($NRINSCRCONS) ? $NRINSCRCONS : null;
		$CDCLIENTE = isset($CDCLIENTE) ? $CDCLIENTE : null;
	    $CDCONSUMIDOR = isset($CDCONSUMIDOR) ? $CDCONSUMIDOR : null;
		$CDCCUSCLIE = isset($CDCCUSCLIE) ? $CDCCUSCLIE : null;

		// Dados TEF
		$CDNSUHOSTTEF = isset($CDNSUHOSTTEF) ? $CDNSUHOSTTEF : null;

	    $connection = $this->entityManager->getConnection();

	    $params = array(
	        'CDFILIAL' => $CDFILIAL,
	        'NRORG' => $NRORG
	    );
	    $dadosEmpresa = $this->entityManager->getConnection()->fetchAll(self::GET_EMPRESAFILIAL, $params);
	    $CDEMPRESA = $dadosEmpresa[0]['CDEMPRESA'];
	  	$NRINSJURFILI = $dadosEmpresa[0]['NRINSJURFILI'];

	    $params = array(
	        'CDFILIAL' => $CDFILIAL,
	        'NRORG' => $NRORG
	    );
	   	$dadosFilial = $this->entityManager->getConnection()->fetchAll(self::GET_ENDEFILI, $params);
	   	$SGESTADO = $dadosFilial[0]['SGESTADO'];

	    $this->refreshTaxVRTOTTRIBIBPT = array();

	    if ($NRPARCNEGOCIO != null) {
	    	$params = array(
				'NRORG'	=> $NRORG,
				'NRPARCNEGOCIO' => $NRPARCNEGOCIO
			);
	    	$resultCDCLIENTE = $this->entityManager->getConnection()->fetchAll(self::GET_CLIENTEPARC, $params);
	    	if (isset($resultCDCLIENTE[0])) {
	    		$CDCLIENTE = $resultCDCLIENTE[0]['CDCLIENTE'];
	    	}
	    } else {
	    	if (isset($IDENTIFICACAO['CDCLIENTE'])) {
	    		if ($IDENTIFICACAO['CDCLIENTE'] != null) {
	    			$CDCLIENTE = $IDENTIFICACAO['CDCLIENTE'];
	    			$CDCONSUMIDOR = $IDENTIFICACAO['CDCONSUMIDOR'];

	    			if ($CDCONSUMIDOR != '') {
	    				$params = array(
	    					'CDCONSUMIDOR' => $CDCONSUMIDOR,
							'CDCLIENTE' => $CDCLIENTE,
							'NRORG' => $NRORG
            			);
	    				$dadosConsumidor = $this->entityManager->getConnection()->fetchAll(self::GET_CONSUMIDOR, $params);
	    				$NMCONSVEND = $dadosConsumidor[0]['NMCONSUMIDOR'];
	    				$NRINSCRCONS = $dadosConsumidor[0]['NRCPFRESPCON'];
	    				$CDCCUSCLIE = $dadosConsumidor[0]['CDCCUSCLIE'];
	    			}
	    		} else {
	    			$CDCLIENTE = self::buscaClientePadrao($CDFILIAL, $NRORG);
	    		}
	    	} else {
	    		$CDCLIENTE = self::buscaClientePadrao($CDFILIAL, $NRORG);
		    }
	    }

	    //--- Generates a sale code
	    if (empty($NRSEQVENDA)) {
	    	$CDCONTADOR = 'PEDIDOVENDA' . $CDFILIAL . $CDCAIXA;
	    	$NRSEQVENDA = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 10);
	    }

		// trata NRVENDAEXT (segundo o João Fos)
		$dadosCaixa = self::buscaDadosCaixa($CDFILIAL, $CDCAIXA, $NRORG);
		$IDTPEMISSAOFOS = $dadosCaixa['IDTPEMISSAOFOS'];
		if ($IDTPEMISSAOFOS == self::TIPO_SAT) {
			$IDTPAMBNFCE = $dadosCaixa['IDTPEQUSAT'];
		} else {
			$PARAVEND = self::buscaPARAVEND($CDFILIAL, $NRORG);
			$IDTPAMBNFCE = $PARAVEND['IDAMBTRABNFCE'];
		}
		$CDCONTADOR = 'NFCE_' . $CDFILIAL . $CDCAIXA . '_AMB' . $IDTPAMBNFCE;
		$NRVENDAEXT = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 9);

	    //--- Build the sale params
	    $parametrosDaVenda = self::montaParametrosDaVenda($CDFILIAL, $CDLOJA, $CDCAIXA, $CDVENDEDOR, $DTVENDA, $DTABERVENDA, $CDEMPRESA, $SGESTADO, $IDIMPVENDA, $DTABERCAIX, $VRDESCVENDA, $CDCLIENTE, $CDCCUSCLIE, $IDSITUVENDA, $NRSEQVENDA, $NRORG, $IDTPEMISVEND, $CDOPERADOR, $NRINSJURFILI, $CDCONSUMIDOR, $NMCONSVEND, $NRINSCRCONS, $IDORIGEMVENDA, $IDMONGO_SAAS, $CDSENHAPED, $NRVENDAEXT);

	    //--- Persists the sale
   		$this->entityManager->getConnection()->executeQuery(self::INSERT_VENDA, $parametrosDaVenda);

	    //--- Saves the sale items
	    $cdprodutoItems = array();
	    foreach($ITEMVENDA as &$produto) {
	    	if ($produto['IDSITUITEM'] == self::ITEM_APROVADO) {
	    		$cdprodesto = null;
	    		$nrfeaturegrupo = null;
	    		if (isset($produto['DIMENSAO'])) {
	    			$dimensoes = $produto['DIMENSAO'];
	    			$nrfeaturegrupo = self::getNrfeatureGrupoByDimensoes($dimensoes);
    				$cdprodesto = self::getProdEstoByNrfeaturegrupo($nrfeaturegrupo);
	    		} else {
					$cdprodesto = $produto['CDPRODUTO'];
				}
				$produto['NRFEATUREGRUPO'] = $nrfeaturegrupo;
				$produto['CDPRODESTO'] = $cdprodesto;
				$cdprodutoItems[] = $cdprodesto;
			}
	    }

	    $NRLANCESTQ = null;
	    $PARAVEND = self::buscaPARAVEND($CDFILIAL, $NRORG);
	    $IDESTTEMPOREAL = $PARAVEND['IDESTTEMPOREAL'];
	    if ($IDESTTEMPOREAL == self::SINCRONIZA_ESTOQUE) {
	  	  	$paramsEstqFili = $this->estoque->getParamsEstqFili($CDFILIAL, $NRORG);
    		if ($paramsEstqFili['IDCTRLESTQ']) {
    			$prodsCntrEstq = $this->estoque->prodsVendCntrlEst($NRORG, $CDFILIAL, $cdprodutoItems);
    			if ($prodsCntrEstq) {
    				$dslancestq = 'LANCTO. REF. À VENDA: ' . $NRSEQVENDA . ' DO CAIXA: ' . $CDCAIXA;
    				$idtplancto = 'S';
    				$NRLANCESTQ = $this->estoque->geraLancamentoEstq($CDFILIAL, $NRORG, $CDOPERADOR, $DTVENDA, $CDFILIAL, $dslancestq, $idtplancto);
	   			}
    		}
	    }

	    $NRORDITCUPFIS = 0;
	    $vr = $this->calcVrSale($ITEMVENDA);
	    $vrSalePerValidItem = $vr[0];
	    $vrSaleWithNoDesc = $vr[1];
	    foreach ($ITEMVENDA as $item) {
    		$NRORDITCUPFIS = $NRORDITCUPFIS + 1;
    		$NRORDITCUPFIS = str_pad($NRORDITCUPFIS, 5, '0', STR_PAD_LEFT);
	    	if ($item['IDSITUITEM'] == self::ITEM_APROVADO) {
	   			self::insertSaleItem(
	   				$CDFILIAL,
	   				$CDLOJA,
	   				$CDCAIXA,
	   				$NRORG,
	   				$NRSEQVENDA,
	   				$item,
	   				$NRORDITCUPFIS,
	   				$nrfeaturegrupo,
	   				$CDOPERADOR,
	   				$IDTPEMISVEND,
	   				$TOTALVENDA,
	   				$VRDESCVENDA,
	   				$vrSalePerValidItem,
	   				$vrSaleWithNoDesc,
	   				null,
	   				null,
	   				$CDVENDEDOR,
	   				$VRTOTTRIBIBPT,
	   				$CDCLIENTE,
	   				$CDCONSUMIDOR,
	   				$IDESTTEMPOREAL
	   			);
			} else {
				self::insertSaleItemCanc($CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $NRSEQVENDA, $item, $NRORDITCUPFIS);
			}
	    }

	    self::refreshITVENDAIMPOS($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA);

	    //----- Saves the sale identification atributes ------
	    $cdcontadorAtribVND = 'ATRIBVND' . $NRORG;
	    if ($atributosIdent){
   	  		$nrrelacatributo = $this->util->geraCodigo($connection, $cdcontadorAtribVND, $NRORG, count($atributosIdent), 1, 10);
		    foreach ($atributosIdent as $key => $atributo) {
		    	self::insertAtribsIdent($nrrelacatributo++, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, null, $key, $atributo, $NRORG, $DTVENDA, $CDOPERADOR);
		    }
	    }

	    //----- atualiza estoque ------
	    if ($IDESTTEMPOREAL == self::SINCRONIZA_ESTOQUE) {
	    	if ($NRLANCESTQ !== null) {
	    		$dtini = date_create_from_format('d/m/Y H:i:s', $DTVENDA);
	    		$dtini = $dtini->format('d/m/Y');
	    		$dtfin = date_create_from_format('d/m/Y H:i:s', $DTVENDA);
	    		$dtfin = $dtfin->format('d/m/Y');
	    		$this->estoque->atualizaEstoque($CDFILIAL, $CDFILIAL, $NRLANCESTQ, $dtini, $dtfin);
	    	}
	    }
	    $FORMADEPAGAMENTO = self::mergePaymentModes($FORMADEPAGAMENTO);
	    foreach ($FORMADEPAGAMENTO as $paymentMode) {
	   		if (!empty($paymentMode['NRSEQMOV'])) {
	   			self::updateIntegracaomov($paymentMode['NRSEQMOV'], $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG);
	   		}
   	  		if (isset($paymentMode['ATRIBUTOSRECEBIMENTO'])) {
				$atributosPayment = $paymentMode['ATRIBUTOSRECEBIMENTO'];
   	  			$nrrelacatributo = $this->util->geraCodigo($connection, $cdcontadorAtribVND, $NRORG, count($atributosPayment), 1, 10);
   	  			foreach ($atributosPayment as $key => $atributo) {
		    		self::insertAtribsIdent($nrrelacatributo++, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $paymentMode['CDTIPORECE'], $key, $atributo, $NRORG, $DTVENDA, $CDOPERADOR);
		   		}
   	  		}

	    	$nrseqmovi = self::insertPayment($paymentMode, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $DTVENDA, $NRSEQVENDA, $NRORG, $CDOPERADOR, $CDCLIENTE, $VRTROCOVEND, $CDNSUHOSTTEF);
	    	$idtipoRece = self::getIdtiporece($paymentMode, $NRORG);

	    	//---- save movclie debito consumidor

	    	if ($idtipoRece == 'A') {
	    		$cdcontadorMovclie = 'MOVCLIE' . $CDCLIENTE;
	    		$nrseqmovclie      = $this->util->geraCodigo($connection, $cdcontadorMovclie, $NRORG, 1, 10);
	    		$vrmovcli          = $paymentMode['VALOR'];
				$cdtiporece        = $paymentMode['CDTIPORECE'];
				$vrsaldocons       = self::getSaldoConsumidor(array(
					'CDCLIENTE' => $CDCLIENTE,
					'CDCONSUMIDOR' => $CDCONSUMIDOR,
					'NRORG' => $NRORG
				));

				if ($vrsaldocons == null) {
					$novoSaldoCons = $vrmovcli * (-1);
				} else {
					$vrmovcli = $vrmovcli * (-1);
					$novoSaldoCons = $vrsaldocons + $vrmovcli;
					$vrmovcli = $vrmovcli * (-1);
				}
				$novoSaldoCons = $novoSaldoCons;
				$vrmovcli = $vrmovcli;

				// @toDo: Verificar como os quatro campos abaixo são calculados
				$vrmovpescons = 0; 
				$vrsaldpecons = 0;
				$vrpesubsidio = 0;
				$vrtetosubsid = 0;

	    		self::insertMovclie(
	    			$CDFILIAL,
	    			$CDCAIXA,
	    			$DTABERCAIX,
	    			$nrseqmovclie,
	    			$CDCLIENTE,
	    			$CDCONSUMIDOR,
	    			$vrmovcli,
					$DTVENDA,
					$NRSEQVENDA,
					$DTABERCAIX,
					$nrseqmovi,
					$cdtiporece,
					$novoSaldoCons,
					$CDOPERADOR,
					$NRORG,
					$vrmovpescons,
			    	$vrsaldpecons,
			    	$vrpesubsidio,
			    	$vrtetosubsid
			    );
	    	}
	   		if (!empty($paymentMode['NRSEQMOV'])) {
	   			self::updateIntegracaomov($paymentMode['NRSEQMOV'], $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG);
	   		}
	    }

	    if ($VRDESCVENDA > 0) {
	    	self::prorateDiscount($ITEMVENDA, $VRDESCVENDA, $TOTALVENDA);
	    }

	    return array(
	    	'NRSEQVENDA' => $NRSEQVENDA
    	);
	}

    const GET_EMPRESAFILIAL = "
		SELECT F.CDINSCESTA, E.CDEMPRESA, E.CDPAIS, E.SGESTADO, E.CDMUNICIPIO, F.NRINSJURFILI
		  FROM FILIAL F, EMPRESA E
		 WHERE F.CDFILIAL = :CDFILIAL
		   AND F.NRORG    = :NRORG
		   AND E.NRORG    = F.NRORG
		   AND E.CDEMPRESA = F.CDEMPRESA
    ";

    const GET_ENDEFILI = "
        SELECT *
           FROM ENDEFILI
          WHERE CDFILIAL     = :CDFILIAL
            AND IDTPENDEFILI = 'P'
            AND NRORG        = :NRORG
    ";

    const GET_CLIENTEPARC = "
      SELECT CDCLIENTE
        FROM CLIENTE
       WHERE NRORG    = :NRORG
         AND NRPARCNEGOCIO = :NRPARCNEGOCIO
    ";

    const GET_CONSUMIDOR = "
        SELECT NMCONSUMIDOR, NRCPFRESPCON ,CDCCUSCLIE
        FROM CONSUMIDOR
        WHERE CDCONSUMIDOR = :CDCONSUMIDOR
            AND CDCLIENTE  = :CDCLIENTE
            AND NRORG      = :NRORG
    ";

    const INSERT_VENDA = "
		INSERT INTO VENDA
		(CDFILIAL, CDLOJA, CDCAIXA, DTENTRVENDA,
		DTVENDA, DTABERVENDA, DTFECHAVENDA, IDIMPVENDA,
		DTABERTUR, CDFILIALTUR, CDCAIXATUR, CDCLIENTE,
		IDSITUVENDA, NRSEQVENDA, NRORG, CDEMPRESA,
		SGESTADO, VRDESCVENDA, CDOPERADOR, NRINSJURIEST,
		CDCONSUMIDOR, CDCCUSCLIE, NMCONSVEND, NRINSCRCONS,
		CDOPERINCLUSAO, CDOPERULTATU, IDTPEMISVEND, IDORIGEMVENDA,
		CDPREFINTGCOLVEN, DTHRREALCONCONS, IDTIPOVENDA,
		IDVENDEBCONS, NRPESMESAVENDA, CDSENHAPED, CDVENDEDOR,
		NRSEQVENDAMSDE, NRVENDAEXT)
		VALUES
		(:CDFILIAL, :CDLOJA, :CDCAIXA, :DTENTRVENDA,
		:DTVENDA, :DTABERVENDA, :DTFECHAVENDA, :IDIMPVENDA,
		:DTABERTUR, :CDFILIALTUR, :CDCAIXATUR, :CDCLIENTE,
		:IDSITUVENDA, :NRSEQVENDA, :NRORG, :CDEMPRESA,
		:SGESTADO, :VRDESCVENDA, :CDOPERADOR, :NRINSJURIEST,
		:CDCONSUMIDOR, :CDCCUSCLIE, :NMCONSVEND, :NRINSCRCONS,
		:CDOPERADOR, :CDOPERADOR, :IDTPEMISVEND, :IDORIGEMVENDA,
		:CDPREFINTGCOLVEN, :DTHRREALCONCONS, :IDTIPOVENDA,
		:IDVENDEBCONS, :NRPESMESAVENDA, :CDSENHAPED, :CDVENDEDOR,
		:NRSEQVENDA, :NRVENDAEXT)
    ";

	private function defineTipoProduto($CDPRODPROMOCAO, $IDTIPOCOMPPROD, $IDIMPPRODUTO) {
		$tipoProduto = 'normal';
		if (!empty($CDPRODPROMOCAO)) {
			$tipoProduto = 'filho';
		} else if (($IDTIPOCOMPPROD == '3') && ($IDIMPPRODUTO == '2')) {
			$tipoProduto = 'promocaoInsereFilhos';
		} else if (($IDTIPOCOMPPROD == '3') && ($IDIMPPRODUTO == '1')) {
			$tipoProduto = 'promocaoInserePai';
		}
		return $tipoProduto;
	}

	private function insertSaleItem($CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $NRSEQVENDA, $item, $NRORDITCUPFIS, &$nrfeaturegrupo, $CDOPERADOR, $IDTPEMISVEND, $totalVenda, $VRDESCVENDA, $vrSalePerValidItem, $vrSaleWithNoDesc, $CDPRODPROMOCAO, $NRSEQPRODCOMIT, $CDVENDEDOR, $VRTOTTRIBIBPT, $CDCLIENTE, $CDCONSUMIDOR, $IDESTTEMPOREAL) {
		$connection = $this->entityManager->getConnection();
		$CDCONTADOR = 'ITEMVENDA' . $CDFILIAL . $CDCAIXA . $NRSEQVENDA;
		$NRSEQUITVEND = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 3);

		foreach ($item['OBSERVACOES'] as $currentObs) {
			$obsCDPRODUTO = self::buscaProdutoObs($currentObs['CDGRPOCOR'], $currentObs['CDOCORR']);
			if ($obsCDPRODUTO) {
				$NRORDITCUPFIS = $NRORDITCUPFIS + 1;
				$NRORDITCUPFIS = str_pad($NRORDITCUPFIS, 5, '0', STR_PAD_LEFT);
	    		$itemObs = self::montaItemObs($obsCDPRODUTO, $currentObs['QTPRODVEND'], $currentObs['VRUNITVEND']);
				self::insertSaleItem(
					$CDFILIAL,
				 	$CDLOJA,
				 	$CDCAIXA,
				 	$NRORG,
				 	$NRSEQVENDA,
				 	$itemObs,
				 	$NRORDITCUPFIS,
				 	$nrfeaturegrupo,
				 	$CDOPERADOR,
				 	$IDTPEMISVEND,
				 	$totalVenda,
				 	$VRDESCVENDA,
				 	$vrSalePerValidItem,
				 	$vrSaleWithNoDesc,
				 	$obsCDPRODUTO,
				 	$NRSEQPRODCOMIT,
				 	$CDVENDEDOR,
				 	$VRTOTTRIBIBPT,
				 	$CDCLIENTE,
				 	$CDCONSUMIDOR,
				 	$IDESTTEMPOREAL
				);
			}
		}

		if (empty($item['IDTIPOCOMPPROD'])) {
			$item['IDTIPOCOMPPROD'] = null;
		}
		if (empty($item['IDIMPPRODUTO'])) {
			$item['IDIMPPRODUTO'] = null;
		}
		$tipoProduto = self::defineTipoProduto($CDPRODPROMOCAO, $item['IDTIPOCOMPPROD'], $item['IDIMPPRODUTO']);

		if ($tipoProduto == 'promocaoInserePai') {
			foreach ($item['itensCombo'] as $currentComboItem) {
				self::insereITEMVENDAEST(
					$CDFILIAL,
					$CDCAIXA,
					$NRSEQVENDA,
					$NRSEQUITVEND,
					$currentComboItem['CDPRODUTO'],
					$currentComboItem['QTPRODVEND'],
					$currentComboItem['VRUNITVEND'],
					$currentComboItem['VRDESITVEND'],
					null,
					null
				);
				foreach ($currentComboItem['OBSERVACOES'] as $currentObs) {
					$obsCDPRODUTO = self::buscaProdutoObs($currentObs['CDGRPOCOR'], $currentObs['CDOCORR']);
					if ($obsCDPRODUTO) {
						$NRORDITCUPFIS = $NRORDITCUPFIS + 1;
						$NRORDITCUPFIS = str_pad($NRORDITCUPFIS, 5, '0', STR_PAD_LEFT);
			    		$itemObs = self::montaItemObs($obsCDPRODUTO, $currentObs['QTPRODVEND'], $currentObs['VRUNITVEND']);
						self::insertSaleItem(
							$CDFILIAL,
						 	$CDLOJA,
						 	$CDCAIXA,
						 	$NRORG,
						 	$NRSEQVENDA,
						 	$itemObs,
						 	$NRORDITCUPFIS,
						 	$nrfeaturegrupo,
						 	$CDOPERADOR,
						 	$IDTPEMISVEND,
						 	$totalVenda,
						 	$VRDESCVENDA,
						 	$vrSalePerValidItem,
						 	$vrSaleWithNoDesc,
						 	$obsCDPRODUTO,
						 	$NRSEQPRODCOMIT,
						 	$CDVENDEDOR,
						 	$VRTOTTRIBIBPT,
						 	$CDCLIENTE,
						 	$CDCONSUMIDOR,
						 	$IDESTTEMPOREAL
						);
					}

					self::insereOBSITEMVENDAEST(
						$CDFILIAL,
						$CDCAIXA,
						$NRSEQVENDA,
						$NRSEQUITVEND,
						$currentComboItem['CDPRODUTO'],
						$currentObs['CDGRPOCOR'],
						$currentObs['CDOCORR']
					);
				}
			}
		}

		// se for promoção e será cobrado os itens filhos
		if ($tipoProduto == 'promocaoInsereFilhos') {
			$CDCONTADOR = 'ITEMVENDAPROMO' . $CDFILIAL . $CDCAIXA . $NRSEQVENDA;
			$NRSEQPRODCOMIT = $this->util->geraCodigo($connection, $CDCONTADOR, $NRORG, 1, 3);
			foreach ($item['itensCombo'] as $currentComboItem) {
				$NRORDITCUPFIS = $NRORDITCUPFIS + 1;
				$NRORDITCUPFIS = str_pad($NRORDITCUPFIS, 5, '0', STR_PAD_LEFT);
				self::insertSaleItem(
					$CDFILIAL,
				 	$CDLOJA,
				 	$CDCAIXA,
				 	$NRORG,
				 	$NRSEQVENDA,
				 	$currentComboItem,
				 	$NRORDITCUPFIS,
				 	$nrfeaturegrupo,
				 	$CDOPERADOR,
				 	$IDTPEMISVEND,
				 	$totalVenda,
				 	$VRDESCVENDA,
				 	$vrSalePerValidItem,
				 	$vrSaleWithNoDesc,
				 	$item['CDPRODUTO'],
				 	$NRSEQPRODCOMIT,
				 	$CDVENDEDOR,
				 	$VRTOTTRIBIBPT,
				 	$CDCLIENTE,
				 	$CDCONSUMIDOR,
				 	$IDESTTEMPOREAL
				);
			}
		} else {
			$cdproduto = $item['CDPRODUTO'];
			$qtprodvend = $item['QTPRODVEND'];
			$vrunitvend = $item['VRUNITVEND'];
			$idtipoitem = $item['IDTIPOITEM'];
			$idsituitem = $item['IDSITUITEM'];
			$vrdesitvend = $item['VRDESITVEND'];
		  	$dadosProdParams = array(
				'CDPRODUTO' => $cdproduto,
				'NRORG' => $NRORG
			);
		  	$dadosProd = $this->entityManager->getConnection()->fetchAll(self::GET_DADOSPROD, $dadosProdParams);

		  	$sgunidade = $dadosProd[0]['SGUNIDADE'];
		  	$cdclasfisc = $dadosProd[0]['CDCLASFISC'];
			$cdarvprod = $dadosProd[0]['CDARVPROD'];

			$vrpercvend = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
			$vraliqimpr = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO4
			$nratraproditve = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
			$vrrattxserv = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
			$vrratdescven = '0'; // VERIFICAR COMO CALCULA ESSE CAMPO
			$dthrpedido = date('d/m/Y H:i:s'); // verificar se é o horário do front ou não (em caso de Sync pode dar diferença de horário)

			$baseCalcItVend = ($qtprodvend * $vrunitvend) - $item['VRDESITVEND'] + $item['VRACRITVEND'];
			$saleItemParams = self::buildSaleItemParams($CDFILIAL, $CDLOJA, $CDCAIXA, $qtprodvend, $vrunitvend, $item['VRDESITVEND'], $cdproduto, $NRORG, $NRSEQVENDA, $NRSEQUITVEND, $item['VRACRITVEND'], $NRORDITCUPFIS, $vrpercvend, $vraliqimpr, $nratraproditve, $vrrattxserv, $vrratdescven, $dthrpedido, $CDPRODPROMOCAO, $NRSEQPRODCOMIT, $CDVENDEDOR);

			$this->entityManager->getConnection()->executeQuery(self::INSERT_ITEMVENDA, $saleItemParams);

			foreach ($item['OBSERVACOES'] as $currentObs) {
				self::insereOBSITEMVENDA(
					$CDFILIAL,
					$CDCAIXA,
					$NRSEQVENDA,
					$NRSEQUITVEND,
					$currentObs['CDGRPOCOR'],
					$currentObs['CDOCORR']
				);
			}

			if ($item['IDTIPOITEM'] == self::ITEM_ABASTECIMENTO) {

				$dataconclusaoabastecimento = $item['DATACONCLUSAOABASTECIMENTO'];
				$nrbico = $item['NRBICO'];
				$vrencerranteinicial = $item['VRENCERRANTEINICIAL'];
				$vrencerrantefinal = $item['VRENCERRANTEFINAL'];

				$getBombaParams = array(
					'NRBICO' => $nrbico
				);

				$nrbombaResult = $this->entityManager->getConnection()->fetchAll(self::GET_BOMBA_BY_BICO, $getBombaParams);
				$nrbomba = $nrbombaResult[0]['NRBOMBA'];
				$cdbomba = $nrbombaResult[0]['CDBOMBA'];
				$cdbico = $nrbombaResult[0]['CDBICO'];

				$getTanquebicohParams = array(
					'NRSEQBICO' => $nrbico,
					'DTABASTECIMENTO' => $this->databaseUtil->convertToDateTime($dataconclusaoabastecimento . ':00')
				);
				$tanquebicohResult = $this->entityManager->getConnection()->fetchAll(self::GET_TANQUEBICOH_BY_BICO, $getTanquebicohParams);
				$CDFILIALalmoxarifado = $tanquebicohResult[0]['CDFILIALALMOXARIFADO'];
				$cdalmoxarifado = $tanquebicohResult[0]['CDALMOXARIFADO'];
				$nritemvendaauxResult = $this->entityManager->getConnection()->fetchAll(self::GET_NOVO_NRITEMVENDAUXILIAR);
				$nritemvendauxiliar = $nritemvendaauxResult[0]['NRITEMVENDAUXILIAR'];
				$getTipocombustivelParams = array(
					'CDPRODUTO' => $cdproduto,
					'NRORG'		=> $NRORG
				);
				$nrtipocombustivelResult = $this->entityManager->getConnection()->fetchAll(self::GET_TIPOCOMBUSTIVEL_BY_PRODUTO, $getTipocombustivelParams);
				$nrtipocombustivel = null;
				if (isset($nrtipocombustivelResult[0]['NRTIPOCOMBUSTIVEL'])) {
					$nrtipocombustivel = $nrtipocombustivelResult[0]['NRTIPOCOMBUSTIVEL'];
				}

				$auxSaleItemParams = self::buildAuxSaleItemParams($CDFILIAL, $CDCAIXA, $nritemvendauxiliar, $CDFILIALalmoxarifado, $cdalmoxarifado, $dataconclusaoabastecimento, $nrtipocombustivel, $nrbomba, $nrbico, $NRORG, $NRSEQVENDA, $NRSEQUITVEND, $vrencerranteinicial, $vrencerrantefinal);
				$this->entityManager->getConnection()->executeQuery(self::INSERT_ITEMVENDAAUXILIAR, $auxSaleItemParams);
			}

			if (isset($item['NRFEATUREGRUPO'])) {
				$nrfeaturegrupo = $item['NRFEATUREGRUPO'];
				if ($nrfeaturegrupo != null) {
					$nritemvendaauxResult = $this->entityManager->getConnection()->fetchAll(self::GET_NOVO_NRITEMVENDAUXILIAR);
					$nritemvendauxiliar = $nritemvendaauxResult[0]['NRITEMVENDAUXILIAR'];
					$auxSaleItemFeatureParams = self::buildAuxSaleItemFeatureParams($nritemvendauxiliar, $CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $NRSEQUITVEND, $nrfeaturegrupo, $CDOPERADOR);
					$this->entityManager->getConnection()->executeQuery(self::INSERT_ITEMVENDAAUXILIAR_FEATUREGRUPO, $auxSaleItemFeatureParams);
				}
			}
			//Insere os possiveis beneficios
			if(!empty($item['beneficios'])){
				$item['beneficios'] = $item['beneficios'][0];
				$CDCAMPANHA = $item['beneficios']['CDCAMPANHA'];
				$NRSEQPRODCAMP = $item['beneficios']['NRSEQPRODCAMP'];
				$NRSEQBENEFICIO = $item['beneficios']['NRSEQBENEFICIO'];
				$QTDEBENEFICIO = $item['beneficios']['QTDEBENEFICIO'];
				$params = array(
					':QTDEBENEFICIO'  => (string)$QTDEBENEFICIO,
					':CDCLIENTE'      => $CDCLIENTE,
					':CDCONSUMIDOR'   => $CDCONSUMIDOR,
					':CDCAMPANHA'     => $CDCAMPANHA,
					':NRSEQUITVEND'   => $NRSEQUITVEND,
					':NRSEQPRODCAMP'  => $NRSEQPRODCAMP,
					':NRSEQBENEFICIO' => $NRSEQBENEFICIO
				);
				$this->entityManager->getConnection()->executeQuery(self::UPDATE_MOVBENEFICIOCTR, $params);
				$NRSEQBENECONS = $this->util->geraCodigo($this->entityManager->getConnection(), 'MOVBENEFICIOCONS', $NRORG, 1, 10);
				$params = array(
					':CDCAMPANHA'     => $CDCAMPANHA,
					':NRSEQBENECONS'  => $NRSEQBENECONS,
					':NRSEQPRODCAMP'  => $NRSEQPRODCAMP,
					':NRSEQBENEFICIO' => $NRSEQBENEFICIO,
					':CDCLIENTE'      => $CDCLIENTE,
					':CDCONSUMIDOR'   => $CDCONSUMIDOR,
					':DTUTBENECONS'   => $this->databaseUtil->getCurrentDateTime(),
					':IDIMPTMOV'      => 'A',//A de aalterado, quando subir a venda vira N
					':CDFILIAL'       => $CDFILIAL,
					':CDCAIXA'        => $CDCAIXA,
					':NRSEQVENDA'     => $NRSEQVENDA,
					':NRSEQUITVEND'   => $NRSEQUITVEND
				);
				$this->entityManager->getConnection()->executeQuery(self::INSERT_MOVBENEFICIOCONS, $params);
			}
			// TRATAMENTO DOS IMPOSTOS
			if (!isset($item['IMPOSTOS'])) {
				$this->calculaImposto($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $item, $IDTPEMISVEND, $baseCalcItVend, $totalVenda, $VRDESCVENDA, $vrSalePerValidItem, $vrSaleWithNoDesc, $VRTOTTRIBIBPT);
			}

			// @todo - verificar com Thais
			$NRLANCESTQ = null;
			if ($IDESTTEMPOREAL == self::SINCRONIZA_ESTOQUE) {
				if ($NRLANCESTQ !== null) {
					$keys = array(
						'NRLANCESTQ'   => $NRLANCESTQ,
						'NRSEQVENDA'   => $NRSEQVENDA,
						'NRSEQUITVEND' => $NRSEQUITVEND
					);
					$this->estoque->updateProductStock($item, $NRORG, $CDFILIAL, $CDLOJA, $CDCAIXA, $keys, $DTVENDA, null, null, null, null, $CDOPERADOR);
				}
			}
		}
	}

	const GET_DADOSPROD = "
        SELECT P.CDARVPROD, P.NMPRODUTO, P.CDCLASFISC, P.SGUNIDADE
          FROM PRODUTO P
         WHERE P.NRORG = :NRORG
           AND P.CDPRODUTO = :CDPRODUTO
    ";

    const INSERT_ITEMVENDA = "
        INSERT INTO ITEMVENDA
        (CDFILIAL, CDLOJA, CDCAIXA, QTPRODVEND, VRUNITVEND,
        VRDESITVEND, CDPRODUTO, NRORG, NRSEQVENDA, NRSEQUITVEND,
        VRACRITVEND, VRUNITVENDCL, NRORDITCUPFIS,
        VRPERCVEND, VRALIQIMPR, NRATRAPRODITVE, VRRATTXSERV,
        VRRATDESCVEN, DTHRPEDIDO, CDCAIXAPEDVEN, CDPRODPROMOCAO,
        NRSEQPRODCOMIT, CDVENDEDOR, IDORIGPEDVEN, QTPRODREFIL)
        VALUES
        (:CDFILIAL, :CDLOJA, :CDCAIXA, :QTPRODVEND, :VRUNITVEND,
        :VRDESITVEND, :CDPRODUTO, :NRORG, :NRSEQVENDA,
        :NRSEQUITVEND, :VRACRITVEND, :VRUNITVENDCL, :NRORDITCUPFIS,
        :VRPERCVEND, :VRALIQIMPR, :NRATRAPRODITVE, :VRRATTXSERV,
        :VRRATDESCVEN, :DTHRPEDIDO, :CDCAIXAPEDVEN, :CDPRODPROMOCAO,
        :NRSEQPRODCOMIT, :CDVENDEDOR, :IDORIGPEDVEN, :QTPRODREFIL)
    ";

    const GET_BOMBA_BY_BICO = "
        SELECT BICO.NRBOMBA,BICO.CODIGOBICO AS CDBICO, BOMBA.CODIGOBOMBA AS CDBOMBA
          FROM VND_BICO BICO, VND_BOMBA BOMBA
         WHERE BICO.NRSEQBICO  = :NRBICO
           AND BICO.NRBOMBA    = BOMBA.NRBOMBA
    ";

    const GET_TANQUEBICOH_BY_BICO = "
        SELECT TB.CDALMOXARIFE AS CDALMOXARIFADO , TB.CDFILIAL AS CDFILIALALMOXARIFADO
          FROM (
                SELECT MIN(:DTABASTECIMENTO) - DTATIVACAO) AS DATEDIFF, NRSEQBICO
                  FROM VND_TANQUEBICOH
                 WHERE NRSEQBICO = :NRSEQBICO
                   AND :DTABASTECIMENTO,'DD/MM/YY HH24:MI:SS') >= DTATIVACAO
              GROUP BY NRSEQBICO
            ) DIFFS, VND_TANQUEBICOH TB
          WHERE DIFFS.NRSEQBICO = TB.NRSEQBICO
            AND (:DTABASTECIMENTO - TB.DTATIVACAO) = DIFFS.DATEDIFF
    ";

    const GET_NOVO_NRITEMVENDAUXILIAR = "
        SELECT (NVL(MAX(NRITEMVENDAUXILIAR),0) + 1) AS  NRITEMVENDAUXILIAR
          FROM VND_ITEMVENDAAUXILIAR
    ";

    const GET_TIPOCOMBUSTIVEL_BY_PRODUTO = "
        SELECT PP.NRTIPOCOMBUSTIVEL, TC.CODCOMBUSTIVEL
          FROM VND_PARAMETROSPRODUTO PP, VND_TIPOCOMBUSTIVEL TC
         WHERE PP.CDPRODUTO         = :CDPRODUTO
           AND PP.NRORG     = :NRORG
           AND PP.NRTIPOCOMBUSTIVEL = TC.NRTIPOCOMBUSTIVEL(+)
    ";

    const INSERT_ITEMVENDAAUXILIAR = "
        INSERT INTO VND_ITEMVENDAAUXILIAR
            (CDFILIAL,CDCAIXA,NRITEMVENDAUXILIAR,CDFILIALALMOXARIFADO,
            CDALMOXARIFADO,DATACONCLUSAOABASTECIMENTO,NRTIPOCOMBUSTIVEL,NRBOMBA,
            NRBICO,NRORG,NRSEQVENDA,NRSEQUITVEND,
            VRENCERRANTEINICIAL,VRENCERRANTEFINAL)
         VALUES (:CDFILIAL,:CDCAIXA,:NRITEMVENDAUXILIAR,:CDFILIALALMOXARIFADO,
         	:CDALMOXARIFADO,:DATACONCLUSAOABASTECIMENTO,:NRTIPOCOMBUSTIVEL,:NRBOMBA,
         	:NRBICO,:NRORG,:NRSEQVENDA,:NRSEQUITVEND,
         	:VRENCERRANTEINICIAL,:VRENCERRANTEFINAL)
    ";

    const INSERT_ITEMVENDAAUXILIAR_FEATUREGRUPO = "
        INSERT INTO VND_ITEMVENDAAUXILIAR
                   (NRITEMVENDAUXILIAR,CDFILIAL,CDCAIXA,NRORG,NRSEQVENDA,NRSEQUITVEND,NRFEATUREGRUPO,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU)
            VALUES (:NRITEMVENDAUXILIAR,:CDFILIAL,:CDCAIXA,:NRORG,:NRSEQVENDA,:NRSEQUITVEND,:NRFEATUREGRUPO,:NRORGINCLUSAO,:CDOPERINCLUSAO,:NRORGULTATU,:CDOPERULTATU)
    ";
	const UPDATE_MOVBENEFICIOCTR = "
        UPDATE MOVBENEFICIOCTR
		   SET QTDEBENEFICIO  = :QTDEBENEFICIO
		 WHERE CDCLIENTE      = :CDCLIENTE
		   AND CDCONSUMIDOR   = :CDCONSUMIDOR
		   AND CDCAMPANHA     = :CDCAMPANHA
		   AND NRSEQPRODCAMP  = :NRSEQPRODCAMP
		   AND NRSEQPRODCAMP  = :NRSEQPRODCAMP
		   AND NRSEQBENEFICIO = :NRSEQBENEFICIO
    ";
	const INSERT_MOVBENEFICIOCONS = "
        INSERT INTO MOVBENEFICIOCONS
			(CDCAMPANHA, NRSEQBENECONS, NRSEQPRODCAMP, NRSEQBENEFICIO, CDCLIENTE,
		 	 CDCONSUMIDOR, DTUTBENECONS, IDIMPTMOV, CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND)
		VALUES
			(:CDCAMPANHA, :NRSEQBENECONS, :NRSEQPRODCAMP, :NRSEQBENEFICIO, :CDCLIENTE,
		 	 :CDCONSUMIDOR, :DTUTBENECONS, :IDIMPTMOV, :CDFILIAL, :CDCAIXA, :NRSEQVENDA, :NRSEQUITVEND)
    ";
	private function montaItemObs($CDPRODUTO, $QTPRODVEND, $VRUNITVEND) {
		return array(
            'CDPRODUTO' => $CDPRODUTO,
    		'QTPRODVEND' => $QTPRODVEND,
    		'VRUNITVEND' => $VRUNITVEND,
			'VRDESITVEND' => 0,
    		'VRACRITVEND' => 0,
    		'IDSITUITEM' => 'A',
    		'IDTIPOITEM' => null,
    		'OBSERVACOES' => array()
		);
	}

    private function buscaProdutoObs($CDGRPOCOR, $CDOCORR) {
		$result = false;
		$params = array(
	    	'CDGRPOCOR' => $CDGRPOCOR,
	    	'CDOCORR' => $CDOCORR
   		);
    	$OCORRENCIA = $this->entityManager->getConnection()->fetchAssoc(self::BUSCA_OCORRENCIA, $params);
    	if ($OCORRENCIA['IDCONTROLAOBS'] == self::OBS_ADICIONA_PRODUTO) {
    		$result = $OCORRENCIA['CDPRODUTO'];
    	}
    	return $result;
    }

	const BUSCA_OCORRENCIA = "
		SELECT *
		  FROM OCORRENCIA
		 WHERE CDGRPOCOR = :CDGRPOCOR
		   AND CDOCORR = :CDOCORR
	";

    private function insereOBSITEMVENDA($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $CDGRPOCOR, $CDOCORR) {
    	$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRSEQVENDA' => $NRSEQVENDA,
			'NRSEQUITVEND' => $NRSEQUITVEND,
			'CDGRPOCOR' => $CDGRPOCOR,
			'CDOCORR' => $CDOCORR
		);
		$this->entityManager->getConnection()->executeQuery(self::INSERT_OBSITEMVENDA, $params);
    }

    const INSERT_OBSITEMVENDA = "
		INSERT INTO OBSITEMVENDA
		(CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND,
		CDGRPOCOR, CDOCORR)
		VALUES
		(:CDFILIAL, :CDCAIXA, :NRSEQVENDA, :NRSEQUITVEND,
		:CDGRPOCOR, :CDOCORR)
    ";

	private function insereOBSITEMVENDAEST($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $CDPRODUTO, $CDGRPOCOR, $CDOCORR) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRSEQVENDA' => $NRSEQVENDA,
			'NRSEQUITVEND' => $NRSEQUITVEND,
			'CDPRODUTO' => $CDPRODUTO,
			'CDGRPOCOR' => $CDGRPOCOR,
			'CDOCORR' => $CDOCORR
		);
		$this->entityManager->getConnection()->executeQuery(self::INSERT_OBSITEMVENDAEST, $params);
	}

	const INSERT_OBSITEMVENDAEST = "
		INSERT INTO OBSITEMVENDAEST
		(CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND,
		CDPRODUTO, CDGRPOCOR, CDOCORR)
		VALUES
		(:CDFILIAL, :CDCAIXA, :NRSEQVENDA, :NRSEQUITVEND,
		:CDPRODUTO, :CDGRPOCOR, :CDOCORR)
	";

    private function insereITEMVENDAEST($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $CDPRODUTO, $QTITVENDAEST, $VRUNVENDAEST, $VRDESITVENDAEST, $NRATRAPRODITES, $DSOBSPEDDIGITE) {
		$params = array(
			'CDFILIAL' => $CDFILIAL,
			'CDCAIXA' => $CDCAIXA,
			'NRSEQVENDA' => $NRSEQVENDA,
			'NRSEQUITVEND' => $NRSEQUITVEND,
			'CDPRODUTO' => $CDPRODUTO,
			'QTITVENDAEST' => $QTITVENDAEST,
			'VRUNVENDAEST' => $VRUNVENDAEST,
			'VRDESITVENDAEST' => $VRDESITVENDAEST,
			'NRATRAPRODITES' => $NRATRAPRODITES,
			'DSOBSPEDDIGITE' => $DSOBSPEDDIGITE
		);
	  	$this->entityManager->getConnection()->executeQuery(self::INSERE_ITEMVENDAEST, $params);
	}

	const INSERE_ITEMVENDAEST = "
		INSERT INTO ITEMVENDAEST
		(CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND,
		CDPRODUTO, QTITVENDAEST, VRUNVENDAEST, VRDESITVENDAEST,
		NRATRAPRODITES, DSOBSPEDDIGITE)
		VALUES
		(:CDFILIAL, :CDCAIXA, :NRSEQVENDA, :NRSEQUITVEND,
		:CDPRODUTO, :QTITVENDAEST, :VRUNVENDAEST, :VRDESITVENDAEST,
		:NRATRAPRODITES, :DSOBSPEDDIGITE)
	";

	private function calculaImposto($CDFILIAL, $CDCAIXA, $NRSEQVENDA, $NRSEQUITVEND, $item, $IDTPEMISVEND, $baseCalcItVend, $totalVenda, $VRDESCVENDA, $vrSalePerValidItem, $vrSaleWithNoDesc, $VRTOTTRIBIBPT) {
		$descItemInVenda = 0; //desconto rateado da venda no item
		$acreItemInVenda = 0; //acrescimo rateado da venda no item. por enquanto n existe acrescimo na venda;

		$getAliqImpFisParams = array(
			'CDFILIAL' => $CDFILIAL,
			'CDPRODUTO'  => $item['CDPRODUTO']
		);

		$aliqImpFisResult = $this->entityManager->getConnection()->fetchAll(self::GET_ALIQIMPFIS_IMPOSTO, $getAliqImpFisParams);
		$aliqImpFisResult = $aliqImpFisResult[0];
		$prodItem['QTPRODCOMVEN'] = $item['QTPRODVEND'];
		$prodItem['VRPRECCOMVEN'] = $item['VRUNITVEND'];
		$prodItem['VRUNITVENDCL'] = 0;
		$prodItem['VRDESCCOMVEN'] = $item['VRDESITVEND'];
		$prodItem['VRACRCOMVEN'] = $item['VRACRITVEND'];
		$prodItem['VRPEALIMPFIS'] = 0;
		$prodItem['ITEMPREC'] = (($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - $item['VRDESITVEND']);

		$doDescItem = 0;
		$doAcreItem = 0;

		// CASO A VENDA TIVER DESCONTO
		$percDesc = 0;
		$subTotalAliq = 0;
		$vrTotalVenda = $vrSalePerValidItem;
		$vrDescAliq = 0;

		if (($VRDESCVENDA != 0) && ($VRDESCVENDA != 0.0)) {
			// Percentual do desconto ou acréscimo no cupom
			$percDesc = (($VRDESCVENDA / $vrTotalVenda) * 100);

			// SubTotal por alíquota
			$subTotalAliq = $vrSaleWithNoDesc;

			// Desconto e acréscimo por alíquota
			$vrDescAliq = (($percDesc * $subTotalAliq) / 100);

			if ($totalVenda > 0){
				if ($IDTPEMISVEND != 'E') {
					$doDescItem = round(($prodItem['ITEMPREC'] * $vrDescAliq / $subTotalAliq), 2);
				} else {
					$doDescItem = ($prodItem['ITEMPREC'] * $vrDescAliq / $subTotalAliq);
				}
				if (((($prodItem['ITEMPREC'] - $doDescItem) == 0) || ($prodItem['ITEMPREC'] - $doDescItem) == 0) && (($IDTPEMISVEND == 'S') || ($IDTPEMISVEND == 'N'))) {
					$doDescItem = (($prodItem['ITEMPREC'] * $VRDESCVENDA) / $totalVenda);
				}
			}
		}

		if (($vrSaleWithNoDesc != 0) && ($vrSaleWithNoDesc != 0)) {
			$descItemInVenda = (($prodItem['ITEMPREC'] * $VRDESCVENDA) / $vrSaleWithNoDesc);
		}

		$paramsItvendaimpos = array();
		$paramsItvendaimpos['CDFILIAL'] = $CDFILIAL;
		$paramsItvendaimpos['CDCAIXA'] = $CDCAIXA;
		$paramsItvendaimpos['NRSEQVENDA'] = $NRSEQVENDA;
		$paramsItvendaimpos['NRSEQUITVEND'] = $NRSEQUITVEND;
		$paramsItvendaimpos['NRSEQITIMPOS'] = '001';
        $paramsItvendaimpos['CDIMPOSTO'] = $aliqImpFisResult['CDIMPOSTO'];
        $paramsItvendaimpos['VRPEALPRODIT'] = $aliqImpFisResult['VRPEALIMPFIS'];
        $paramsItvendaimpos['VRIMPOPRODIT'] = 0;
        $paramsItvendaimpos['CDIMPOSTOEX'] = null;
        $paramsItvendaimpos['CDCSTPRODI'] = 0;
        $paramsItvendaimpos['CDCSTPRODPC'] = 0;
        $paramsItvendaimpos['CDCFOPPROD'] = 0;
        $paramsItvendaimpos['VRPERCOFINS'] = 0;
        $paramsItvendaimpos['VRPERPIS'] = 0;
        $paramsItvendaimpos['IDMODALBASECALC'] = 0;
        $paramsItvendaimpos['VRBASECALCREDUZ'] = 0;
        $paramsItvendaimpos['VRIMPOPRODREDUZ'] = 0;
        $paramsItvendaimpos['VRBASECALCICMS'] = 0;
        $paramsItvendaimpos['VRBCREDUZICMS'] = 0;
        $paramsItvendaimpos['VRPRBCREDUICMS'] = 0;
        $paramsItvendaimpos['VRIMPPIS'] = 0;
        $paramsItvendaimpos['VRIMPCOFINS'] = 0;
        $paramsItvendaimpos['VRTOTTRIBIBPT'] = 0;

        // validado com o Fos
		$vrImpoProdIt = round((((((($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - $doDescItem + $doAcreItem) - ((($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - $doDescItem + $doAcreItem) * $aliqImpFisResult['VRPERCREDUCAO'] / 100)) / 100) * $aliqImpFisResult['VRPEALIMPFIS'])), 2);
		$vrBaseCalcReduz = ((($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - ($prodItem['VRDESCCOMVEN'] + $doDescItem - $doAcreItem)) / 100) * $aliqImpFisResult['VRPERCREDUCAO'];
		$paramsItvendaimpos['VRIMPOPRODIT'] = ($IDTPEMISVEND == 'S') ? $vrImpoProdIt : $vrImpoProdIt;

		/*
    	if ($aliqImpFisResult['IDTRATIMPO'] == 'A') {
			$vrImpoProdIt = ((($prodItem['QTPRODCOMVEN'] *
								$prodItem['VRPRECCOMVEN'] + $prodItem['VRUNITVENDCL']) -
									($prodItem['VRDESCCOMVEN'] + $doDescItem)) / 100) *
									$aliqImpFisResult['VRPEALIMPFIS'];
            $vrBaseCalcReduz = ((($prodItem['QTPRODCOMVEN'] *
								$prodItem['VRPRECCOMVEN']) -
									( $prodItem['VRDESCCOMVEN'] + $doDescItem)) / 100) *
								$aliqImpFisResult['VRPERCREDUCAO'];
		} else {
			$vrImpoProdIt = (((($prodItem['QTPRODCOMVEN'] *
								$prodItem['VRPRECCOMVEN']) - $doDescItem + $doAcreItem) -
									((($prodItem['QTPRODCOMVEN'] *
								$prodItem['VRPRECCOMVEN']) - $doDescItem + $doAcreItem) *
									$aliqImpFisResult['VRPERCREDUCAO'] / 100)) / 100) *
							$aliqImpFisResult['VRPEALIMPFIS'];
        }
        */

        $paramsItvendaimpos['CDIMPOSTOEX'] = isset($aliqImpFisResult['CDINTIMPOSTO']) ? $aliqImpFisResult['CDINTIMPOSTO'] : null;
    	$paramsItvendaimpos['CDCSTPRODI'] = $aliqImpFisResult['CDCSTICMS'];
       	$paramsItvendaimpos['CDCSTPRODPC'] = $aliqImpFisResult['CDCSTPISCOF'];
        $paramsItvendaimpos['CDCFOPPROD'] = $aliqImpFisResult['CDCFOPPFIS'];
        $paramsItvendaimpos['VRPERCOFINS'] = $aliqImpFisResult['VRALIQCOFINS'];
        $paramsItvendaimpos['VRPERPIS'] = $aliqImpFisResult['VRALIQPIS'];
        $paramsItvendaimpos['IDMODALBASECALC'] = $aliqImpFisResult['IDMODALBASECALC'];
        $paramsItvendaimpos['VRBASECALCREDUZ'] = $vrBaseCalcReduz;
        $paramsItvendaimpos['VRIMPOPRODREDUZ'] = ($vrBaseCalcReduz / 100) * $aliqImpFisResult['VRPEALIMPFIS'];
        $paramsItvendaimpos['VRBASECALCICMS'] = ($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) + $prodItem['VRACRCOMVEN'] - $doDescItem;
        $paramsItvendaimpos['VRBCREDUZICMS'] = ((($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - $doDescItem) - ((($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - $doDescItem) * ($aliqImpFisResult['VRPERCREDUCAO'] / 100)));
        $paramsItvendaimpos['VRPRBCREDUICMS'] = isset($aliqImpFisResult['VRPERCREDUCAO']) ? $aliqImpFisResult['VRPERCREDUCAO'] : 0;
        if ($aliqImpFisResult['VRALIQPIS'] != '') {
			$doVrImpPis = (($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - $doDescItem) * ($aliqImpFisResult['VRALIQPIS'] / 100);
			$paramsItvendaimpos['VRIMPPIS'] = ($IDTPEMISVEND == 'S' ? round($doVrImpPis, 2) : $doVrImpPis);
		} else {
			$doVrImpPis = 0;
			$paramsItvendaimpos['VRIMPPIS'] = ($IDTPEMISVEND == 'S') ? round($doVrImpPis, 2) : $doVrImpPis;
		}

        if ($aliqImpFisResult['VRALIQCOFINS'] != '') {
        	$doVrImpCofins = (($prodItem['QTPRODCOMVEN'] * $prodItem['VRPRECCOMVEN']) - $doDescItem) * ($aliqImpFisResult['VRALIQCOFINS'] / 100);
			$paramsItvendaimpos['VRIMPCOFINS'] = ($IDTPEMISVEND == 'S') ? round($doVrImpCofins, 2) : $doVrImpCofins;
        } else {
        	$doVrImpCofins = 0;
			$paramsItvendaimpos['VRIMPCOFINS'] = ($IDTPEMISVEND == 'S') ? round($doVrImpCofins, 2) : $doVrImpCofins;
        }

        if ($aliqImpFisResult['VRALIQIBPT'] == '0' || $aliqImpFisResult['VRALIQIBPT'] == null) {
			if ($aliqImpFisResult['IDINCIDEPISCOF'] == 'N') {
         		$vrPis 	  = 0.65;
         		$vrCofins = 3;
         	} else {
         		$vrPis    = 1.65;
         		$vrCofins = 7.60;
         	}
			$icms 	= ($prodItem['ITEMPREC'] - $descItemInVenda + $acreItemInVenda) * ($aliqImpFisResult['VRPEALIMPFIS'] / 100);
         	$pis 	= ($prodItem['ITEMPREC'] - $descItemInVenda + $acreItemInVenda) * ($vrPis / 100);
         	$cofins = ($prodItem['ITEMPREC'] - $descItemInVenda + $acreItemInVenda) * ($vrCofins / 100);
			$paramsItvendaimpos['VRTOTTRIBIBPT'] = $icms + $pis + $cofins;
        } else {
        	$paramsItvendaimpos['VRTOTTRIBIBPT'] = ($prodItem['ITEMPREC'] - $descItemInVenda + $acreItemInVenda) * ($aliqImpFisResult['VRALIQIBPT'] / 100);
		}
		array_push($this->refreshTaxVRTOTTRIBIBPT, $paramsItvendaimpos['VRTOTTRIBIBPT']);
		$paramsItvendaimpos['VRTOTTRIBIBPT'] = $paramsItvendaimpos['VRTOTTRIBIBPT'];

		// @todo - verificar isso
		// Recursos técnicos aplicados - (sugestão de Pimpolho Feliz)
		$paramsItvendaimpos['VRPERCOFINS'] = floatval($paramsItvendaimpos['VRPERCOFINS']);
		$paramsItvendaimpos['VRIMPOPRODIT'] = floatval($paramsItvendaimpos['VRIMPOPRODIT']);
		$paramsItvendaimpos['VRPEALPRODIT'] = floatval($paramsItvendaimpos['VRPEALPRODIT']);
		$paramsItvendaimpos['VRPERPIS'] = floatval($paramsItvendaimpos['VRPERPIS']);
		$paramsItvendaimpos['VRBASECALCREDUZ'] = $paramsItvendaimpos['VRBASECALCREDUZ'];
		$paramsItvendaimpos['VRIMPOPRODREDUZ'] = $paramsItvendaimpos['VRIMPOPRODREDUZ'];
		$paramsItvendaimpos['VRBASECALCICMS'] = $paramsItvendaimpos['VRBASECALCICMS'];
		$paramsItvendaimpos['VRBCREDUZICMS'] = $paramsItvendaimpos['VRBCREDUZICMS'];
		$paramsItvendaimpos['VRPRBCREDUICMS'] = $paramsItvendaimpos['VRPRBCREDUICMS'];
		$paramsItvendaimpos['VRIMPPIS'] = $paramsItvendaimpos['VRIMPPIS'];
		$paramsItvendaimpos['VRIMPCOFINS'] = $paramsItvendaimpos['VRIMPCOFINS'];
		$paramsItvendaimpos['VRTOTTRIBIBPT'] = $VRTOTTRIBIBPT;

   		$this->entityManager->getConnection()->executeQuery(self::INSERT_ITVENDAIMPOS, $paramsItvendaimpos);
	}

    const GET_ALIQIMPFIS_IMPOSTO = "
        SELECT
              AL.VRPEALIMPFIS, AL.CDIMPOSTO, IP.IDTPIMPOSFIS, IP.CDINTIMPOSTO, IP.IDTRATIMPO,
              AL.CDCSTICMS, AL.CDCSTPISCOF, AL.CDCFOPPFIS, AL.VRALIQPIS, AL.VRALIQCOFINS,
              AL.IDMODALBASECALC, AL.VRPERCREDUCAO, AL.VRALIQIBPT, AL.IDINCIDEPISCOF
            FROM
              ALIQIMPFIS AL, IMPOSTO IP
            WHERE (AL.CDFILIAL  = :CDFILIAL)
              AND (AL.CDPRODUTO = :CDPRODUTO)
              AND (AL.CDIMPOSTO = IP.CDIMPOSTO)
    ";

    const INSERT_ITVENDAIMPOS = "
     	INSERT INTO ITVENDAIMPOS
			(CDFILIAL, CDCAIXA, NRSEQVENDA, NRSEQUITVEND,
			NRSEQITIMPOS, CDIMPOSTO, VRPEALPRODIT, VRIMPOPRODIT,
			CDIMPOSTOEX, CDCSTPRODI, CDCSTPRODPC, CDCFOPPROD,
			VRPERCOFINS, VRPERPIS, IDMODALBASECALC, VRBASECALCREDUZ,
			VRIMPOPRODREDUZ, VRBASECALCICMS, VRBCREDUZICMS, VRPRBCREDUICMS,
			VRIMPPIS, VRIMPCOFINS, VRTOTTRIBIBPT)
  		 VALUES (:CDFILIAL, :CDCAIXA, :NRSEQVENDA, :NRSEQUITVEND,
	 		:NRSEQITIMPOS, :CDIMPOSTO, :VRPEALPRODIT, :VRIMPOPRODIT,
	 		:CDIMPOSTOEX, :CDCSTPRODI, :CDCSTPRODPC, :CDCFOPPROD,
	 		:VRPERCOFINS, :VRPERPIS, :IDMODALBASECALC, :VRBASECALCREDUZ,
	 		:VRIMPOPRODREDUZ, :VRBASECALCICMS, :VRBCREDUZICMS, :VRPRBCREDUICMS,
	 		:VRIMPPIS, :VRIMPCOFINS, :VRTOTTRIBIBPT)
    ";

	private function refreshITVENDAIMPOS($CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA) {
		if (count($this->refreshTaxVRTOTTRIBIBPT) > 1) {
			$vrtottribibpt = 0;
			foreach ($this->refreshTaxVRTOTTRIBIBPT as $number) {
				$vrtottribibpt += $number;
			}
			$params = array(
				'CDFILIAL' => $CDFILIAL,
				'CDCAIXA' => $CDCAIXA,
				'NRORG' => $NRORG,
				'NRSEQVENDA' => $NRSEQVENDA,
				'VRTOTTRIBIBPT' => $vrtottribibpt
			);
			$this->entityManager->getConnection()->executeQuery(self::UPDATE_ITVENDAIMPOS, $params);
		}
	}

    const UPDATE_ITVENDAIMPOS = "
        UPDATE ITVENDAIMPOS
        	SET VRTOTTRIBIBPT = :VRTOTTRIBIBPT
          WHERE CDFILIAL = :CDFILIAL
            AND CDCAIXA = :CDCAIXA
            AND NRSEQVENDA = :NRSEQVENDA
            AND NRORG = :NRORG
    ";

	private function insertSaleItemCanc($CDFILIAL, $CDLOJA, $CDCAIXA, $NRORG, $NRSEQVENDA, $item, $NRORDITCUPFISca) {

    	//--- Gets the firm properties
		$cdproduto = $item['CDPRODUTO'];
		$qtprodvendc = $item['QTPRODVEND'];
		$vrunitvendc = $item['VRUNITVEND'];
		$idtipoitemc = $item['IDTIPOITEM'];

		//---- TEMPORARIO
   		$vracritvendc = isset($item['VRACRITVEND']) ? $item['VRACRITVEND'] : 0;
   		//----
		$vrdesitvendc = $item['VRDESITVEND'];
		$CDCONTADOR = 'ITEMVENDA' . $CDFILIAL . $CDCAIXA . $NRSEQVENDA;
		$NRSEQITVENDC = $this->util->geraCodigo($this->entityManager->getConnection(), $CDCONTADOR, $NRORG, 1, 3);
		$saleItemCancParams = self::buildSaleItemCancParams($CDFILIAL, $CDCAIXA, $qtprodvendc, $vrunitvendc, $vrdesitvendc, $cdproduto, $NRORG, $NRSEQVENDA, $NRSEQITVENDC, $vracritvendc, $NRORDITCUPFISca);

		$this->entityManager->getConnection()->executeQuery(self::INSERT_ITEMVENDA_CANCELADO, $saleItemCancParams);
	}

    const INSERT_ITEMVENDA_CANCELADO = "
      INSERT INTO ITVENDACAN
              (CDFILIAL,CDCAIXA,QTPRODVENDC,VRUNITVENDC,VRDESITVENDC,CDPRODUTO,NRORG,NRSEQVENDA,NRSEQITVENDC,VRACRITVENDC,NRORGINCLUSAO,VRUNITVENCLC,NRORDITCUPFISCA)
       VALUES (:CDFILIAL,:CDCAIXA,:QTPRODVENDC,:VRUNITVENDC,:VRDESITVENDC,:CDPRODUTO,:NRORG,:NRSEQVENDA,:NRSEQITVENDC,:VRACRITVENDC,:NRORGINCLUSAO,:VRUNITVENCLC,:NRORDITCUPFISCA)
    ";

	private function insertPayment($payment, $CDFILIAL, $CDCAIXA, $DTABERCAIX, $DTVENDA, $NRSEQVENDA, $NRORG, $CDOPERADOR, $CDCLIENTE, $VRTROCOVEND, $CDNSUHOSTTEF) {
		$idtipomovive = self::ENTRADA;
		$vrmovivend   = $payment['VALOR'];
		$cdtiporece   = $payment['CDTIPORECE'];
  		$qtparcreceb  = $payment['NRPARCELAS'];

		// @toDo: Verificar como os dois campos abaixo são calculados
  		$vrmoviVeout = 0;
  		$vrcotindout = 0;

        $connection = $this->entityManager->getConnection();
  		// Gera código de pagamento
		$cdcontadorMovcaixa = 'MOVCAIXA' . $CDFILIAL . $CDCAIXA . $DTABERCAIX;
   	    $nrsequmovi = $this->util->geraCodigo($connection, $cdcontadorMovcaixa, $NRORG, 1, 10);

   		// Prepara parametros de pagamento
   		$paymentParams = self::buildPaymentParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrsequmovi, $DTVENDA, $idtipomovive, $vrmovivend, $qtparcreceb, $vrmoviVeout, $vrcotindout, $NRSEQVENDA, $NRORG, $cdtiporece, $CDOPERADOR, $CDCLIENTE, $CDNSUHOSTTEF);
   		// Faz a persistência do pagamento
   		$this->entityManager->getConnection()->executeQuery(self::INSERT_MOVCAIXA_SALE, $paymentParams);

   		// Monta troco, caso haver
   		if (!empty($VRTROCOVEND)) {
	    	foreach ($VRTROCOVEND as $troco) {
   				$nrsequmovi	= $this->util->geraCodigo($connection, $cdcontadorMovcaixa, $NRORG, 1, 10);
   				$paymentParams = self::buildPaymentTrocoParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrsequmovi, $DTVENDA, $troco['VALOR'], $NRSEQVENDA, $NRORG, $troco['CDTIPOTROCO'], $CDOPERADOR, $CDCLIENTE, $vrmoviVeout, $vrcotindout, $qtparcreceb);
    			$this->entityManager->getConnection()->executeQuery(self::INSERT_MOVCAIXA_SALE, $paymentParams);
	    	}
	    }
	    return $nrsequmovi;
	}

	const INSERT_MOVCAIXA_SALE = "
		INSERT INTO MOVCAIXA
		(CDFILIAL,CDCAIXA,DTABERCAIX,NRSEQUMOVI,
		DTHRINCMOV,IDTIPOMOVIVE,VRMOVIVEND,QTPARCRECEB,
		VRMOVIVEOUT,VRCOTINDOUT,NRSEQVENDA,NRORG,
		NRORGINCLUSAO,CDOPERINCLUSAO,CDCLIENTE,CDTIPORECE,
		DTMOVIMCAIXA, CDNSUHOSTTEF, NRSEQUMOVIMSDE)
		VALUES(:CDFILIAL,:CDCAIXA,:DTABERCAIX,:NRSEQUMOVI,
		:DTHRINCMOV,:IDTIPOMOVIVE,:VRMOVIVEND,:QTPARCRECEB,
		:VRMOVIVEOUT, :VRCOTINDOUT,:NRSEQVENDA,:NRORG,
		:NRORGINCLUSAO,:CDOPERINCLUSAO, :CDCLIENTE,:CDTIPORECE,
		:DTMOVIMCAIXA, :CDNSUHOSTTEF, :NRSEQUMOVI)
	";

	private function insertMovclie($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrseqmovclie, $CDCLIENTE, $CDCONSUMIDOR, $vrmovcli,
		$dtmovclie, $NRSEQVENDA, $DTABERCAIX, $nrsequmovi, $cdtiporece, $vrsaldocons, $CDOPERADOR, $NRORG,$vrmovpescons,
	    $vrsaldpecons, $vrpesubsidio, $vrtetosubsid){

		$paramsMovclie = self::buildMoviclieParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrseqmovclie, $CDCLIENTE, $CDCONSUMIDOR, $vrmovcli,
		$dtmovclie, $NRSEQVENDA, $DTABERCAIX, $nrsequmovi, $cdtiporece, $vrsaldocons, $CDOPERADOR, $NRORG,$vrmovpescons,
	    $vrsaldpecons, $vrpesubsidio, $vrtetosubsid);

		$this->entityManager->getConnection()->executeQuery(self::INSERT_MOVCLIE, $paramsMovclie);
	}

    const INSERT_MOVCLIE = "
	    INSERT INTO MOVICLIE
	    (CDFILIAL, CDCAIXA, DTABERCAIX, NRSEQMOVCLI,
	    CDCLIENTE, CDCONSUMIDOR, VRMOVCLI, DTMOVCLI,
	    DTMOVIMCLIE, IDTIPMOCVLI, NRSEQVENDA,NRSEQUMOVI,
	    CDTIPORECE, VRSALDOCONS, CDOPERADOR, NRORG,
	    DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU,
	    CDOPERULTATU, VRMOVPESCONS,VRSALDPECONS,VRPESUBSIDIO,
	    VRTETOSUBSID)
	    VALUES
	    (:CDFILIAL, :CDCAIXA, :DTABERCAIX, :NRSEQMOVCLI,
	    :CDCLIENTE, :CDCONSUMIDOR, :VRMOVCLI, :DTMOVCLI,
	    :DTMOVIMCLIE, :IDTIPMOCVLI, :NRSEQVENDA, :NRSEQUMOVI,
	    :CDTIPORECE, :VRSALDOCONS, :CDOPERADOR, :NRORG,
	    :DTINCLUSAO, :NRORGINCLUSAO, :CDOPERINCLUSAO, :NRORGULTATU,
	    :CDOPERULTATU,:VRMOVPESCONS,:VRSALDPECONS,:VRPESUBSIDIO,
	    :VRTETOSUBSID)
    ";

	private function insertAtribsIdent($nrrelacatributo, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $cdtiporece, $nratributovenda, $valoratributo, $NRORG, $DTVENDA, $CDOPERADOR) {
   		$atribsIdentParams = self::buildAtribsIdentParams($nrrelacatributo, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $cdtiporece, $nratributovenda, $valoratributo, $NRORG, $DTVENDA, $CDOPERADOR);
		$this->entityManager->getConnection()->executeQuery(self::INSERT_RELACATRIBUTOVENDA, $atribsIdentParams);
	}

	const INSERT_RELACATRIBUTOVENDA = "
        INSERT
        INTO VND_RELACATRIBUTOVENDA
          (
            NRRELACATRIBUTO,
            CDFILIAL,
            CDCAIXA,
            NRSEQVENDA,
            CDTIPORECE,
            NRATRIBUTOVENDA,
            VALORATRIBUTO,
            NRORG,
            DTINCLUSAO,
            NRORGINCLUSAO,
            CDOPERINCLUSAO,
            NRORGULTATU,
            CDOPERULTATU
          )
          VALUES
          (
            :NRRELACATRIBUTO,
            :CDFILIAL,
            :CDCAIXA,
            :NRSEQVENDA,
            :CDTIPORECE,
            :NRATRIBUTOVENDA,
            :VALORATRIBUTO,
            :NRORG,
            :DTINCLUSAO,
            :NRORGINCLUSAO,
            :CDOPERINCLUSAO,
            :NRORGULTATU,
            :CDOPERULTATU
          )
    ";

	private function getIdtiporece($payment, $NRORG) {
		$cdtiporece = $payment['CDTIPORECE'];
		$idtiporeceParams = array(
			'CDTIPORECE' => $cdtiporece,
			'NRORG' => $NRORG
		);
		$idtiporeceQuery = $this->entityManager->getConnection()->fetchAll(self::GET_IDTIPORECE, $idtiporeceParams);
		$idtiporeceSAAS = $idtiporeceQuery[0]['IDTIPORECE'];
		return $idtiporeceSAAS;
	}

    const GET_IDTIPORECE = "
        SELECT CDTIPORECE, NMTIPORECE, IDTIPORECE
          FROM TIPORECE
         WHERE CDTIPORECE = :CDTIPORECE
           AND NRORG      = :NRORG
    ";

	private function getProdEstoByNrfeaturegrupo($nrfeaturegrupo) {
		$cdprodesto = null;
		$getProdEstFeatureGruParams = self::buildGetProdEstFeatureGruParams($nrfeaturegrupo);
		$getProdEstFeatureGruResult = $this->entityManager->getConnection()->fetchAll(self::GET_PRODESTFEATUREGRU, $getProdEstFeatureGruParams);
		if (isset($getProdEstFeatureGruResult[0])) {
			$cdprodesto = $getProdEstFeatureGruResult[0]['CDPRODUTOESTOQUE'];
		}
		return $cdprodesto;
	}

    const GET_PRODESTFEATUREGRU = "
        SELECT CDPRODUTOESTOQUE
          FROM VND_PRODFEATUREGRUPOESTOQUE
         WHERE NRFEATUREGRUPO = :NRFEATUREGRUPO
    ";

	private function getNrfeatureGrupoByDimensoes($dimensoes) {
		$nrfeaturegrupo = null;
		$qtdimensoes = count($dimensoes);
		$nrprodutodimensaovalorArray = array();
		foreach ($dimensoes as $dimensao) {
			$nrprodutodimensaovalorArray[] = $dimensao['NRPRODUTODIMENSAOVALOR'];
		}
		$getNrfeaturegrupoParams = self::buildGetNrfeaturegrupoParams($nrprodutodimensaovalorArray, $qtdimensoes);

		$params = array($getNrfeaturegrupoParams['NRPRODUTODIMENSAOVALOR'], $getNrfeaturegrupoParams['QTDIMENSOES']);
        $types = array(\Doctrine\DBAL\Connection::PARAM_STR_ARRAY, \PDO::PARAM_INT);
        $dts = $this->entityManager->getConnection()->executeQuery(self::GET_NRFEATUREGRUPO_BY_DIMENSOES, $params, $types);
        $getNrfeaturegrupoResult = $dts->fetchAll();
		if (isset($getNrfeaturegrupoResult[0])) {
			$nrfeaturegrupo = $getNrfeaturegrupoResult[0]['NRFEATUREGRUPO'];
		}
		return $nrfeaturegrupo;
	}

    const GET_NRFEATUREGRUPO_BY_DIMENSOES = "
        SELECT PDVFG1.NRFEATUREGRUPO
          FROM ( SELECT NRFEATUREGRUPO, COUNT(NRFEATUREGRUPO) QTDIMENSOES
                   FROM VND_PRODUTODIMVALORFEATUREGRU
                  WHERE NRPRODUTODIMENSAOVALOR IN (?)
               GROUP BY NRFEATUREGRUPO
               ) PDVFG1,
               ( SELECT NRFEATUREGRUPO, COUNT(NRFEATUREGRUPO) QTDIMENSOES
                   FROM VND_PRODUTODIMVALORFEATUREGRU
               GROUP BY NRFEATUREGRUPO
               ) PDVFG2
         WHERE PDVFG1.QTDIMENSOES    = ?
           AND PDVFG2.NRFEATUREGRUPO = PDVFG1.NRFEATUREGRUPO
           AND PDVFG2.QTDIMENSOES    = PDVFG1.QTDIMENSOES
    ";

	private function getSaldoConsumidor($paramsConsumidor) {
		$saldoConsumidor = null;
		$consumidor = $this->entityManager->getConnection()->fetchAll(self::GET_SALDOCONSUMIDOR, $paramsConsumidor);
		if ($consumidor) {
			$saldoConsumidor = $consumidor[0]['VRSALDOCONS'];
		}
		return $saldoConsumidor;
	}

    const GET_SALDOCONSUMIDOR = "
   		SELECT CDCLIENTE, CDCONSUMIDOR, VRSALDOCONS
   		  FROM MOVICLIE
   		 WHERE NRSEQMOVCLI IN (SELECT MAX(NRSEQMOVCLI)
   		                         FROM MOVICLIE
   		                        WHERE (INSTR(:CDCLIENTE, MOVICLIE.CDCLIENTE ) > 0)
   		                       GROUP BY CDCLIENTE)
   		   AND (INSTR(:CDCLIENTE, MOVICLIE.CDCLIENTE) > 0)
   		GROUP BY CDCLIENTE, CDCONSUMIDOR, VRSALDOCONS
    ";

	private function buscaPARAVEND($CDFILIAL, $NRORG) {
		$params = array(
	    	'CDFILIAL' => $CDFILIAL,
	    	'NRORG' => $NRORG
   		);
    	return $this->entityManager->getConnection()->fetchAssoc(self::BUSCA_PARAVEND, $params);
	}

    const BUSCA_PARAVEND = "
        SELECT CDCLIENTE, CDCCUSCLIE, IDAMBTRABNFCE, IDESTTEMPOREAL
          FROM PARAVEND
         WHERE NRORG    = :NRORG
           AND CDFILIAL = :CDFILIAL
    ";

	private function montaParametrosDaVenda($CDFILIAL, $CDLOJA, $CDCAIXA, $CDVENDEDOR, $DTVENDA, $DTABERVENDA, $CDEMPRESA, $SGESTADO, $IDIMPVENDA, $DTABERCAIX, $VRDESCVENDA, $CDCLIENTE, $CDCCUSCLIE, $IDSITUVENDA, $NRSEQVENDA, $NRORG, $IDTPEMISVEND, $CDOPERADOR, $NRINSJURFILI, $CDCONSUMIDOR, $NMCONSVEND, $NRINSCRCONS, $IDORIGEMVENDA, $IDMONGO, $CDSENHAPED, $NRVENDAEXT) {
		if (empty($CDCCUSCLIE)) {
			$PARAVEND = self::buscaPARAVEND($CDFILIAL, $NRORG);
			$CDCCUSCLIE = $PARAVEND['CDCCUSCLIE'];
		}

		$parametrosDaVenda = array(
			'CDFILIAL' 		   => $CDFILIAL,
			'CDLOJA' 		   => $CDLOJA,
			'CDCAIXA' 		   => $CDCAIXA,
			'DTENTRVENDA' 	   => $this->databaseUtil->getCurrentDate(),
			'DTVENDA' 		   => $this->databaseUtil->convertToDateTime($DTVENDA),
			'DTABERVENDA' 	   => $DTABERVENDA,
			'DTFECHAVENDA'     => $this->databaseUtil->getCurrentDate(),
			'CDEMPRESA' 	   => $CDEMPRESA,
			'SGESTADO'         => $SGESTADO,
			'IDIMPVENDA' 	   => $IDIMPVENDA,
			'DTABERTUR' 	   => $this->databaseUtil->convertToDateTime($DTABERCAIX),
			'CDFILIALTUR' 	   => $CDFILIAL,
			'CDCAIXATUR' 	   => $CDCAIXA,
			'VRDESCVENDA' 	   => $VRDESCVENDA,
			'CDCLIENTE' 	   => $CDCLIENTE,
			'CDCCUSCLIE' 	   => $CDCCUSCLIE,
			'IDSITUVENDA' 	   => $IDSITUVENDA,
			'NRSEQVENDA' 	   => $NRSEQVENDA,
			'NRORG' 		   => $NRORG,
			'IDTPEMISVEND'     => $IDTPEMISVEND,
			'CDOPERADOR' 	   => $CDOPERADOR,
			'NRINSJURIEST'	   => $NRINSJURFILI,
			'CDCONSUMIDOR'	   => $CDCONSUMIDOR,
			'NMCONSVEND'	   => $NMCONSVEND,
			'NRINSCRCONS'	   => $NRINSCRCONS,
			'IDORIGEMVENDA'    => $IDORIGEMVENDA,
			'IDMONGO'		   => $IDMONGO,
			'DTHRREALCONCONS'  => $this->databaseUtil->getCurrentDate(),
			'CDPREFINTGCOLVEN' => 'TEK',
			'IDTIPOVENDA'	   => 'D',
			'IDVENDEBCONS'	   => 'N',
			'NRPESMESAVENDA'   => '1',
			'CDSENHAPED'       => $CDSENHAPED,
			'CDVENDEDOR'       => $CDVENDEDOR,
			'NRVENDAEXT'       => $NRVENDAEXT
 	    );

		return $parametrosDaVenda;
	}

	private function buildSaleItemParams($CDFILIAL, $CDLOJA, $CDCAIXA, $qtprodvend, $vrunitvend, $vrdesitvend, $cdproduto, $NRORG, $NRSEQVENDA, $nrsequitvend, $vracritvend, $NRORDITCUPFIS, $vrpercvend, $vraliqimpr, $nratraproditve, $vrrattxserv, $vrratdescven, $dthrpedido, $CDPRODPROMOCAO, $NRSEQPRODCOMIT, $CDVENDEDOR) {
		$saleItemParams = array(
			'CDFILIAL'       => $CDFILIAL,
			'CDLOJA'         => $CDLOJA,
			'CDCAIXA'        => $CDCAIXA,
			'QTPRODVEND'     => $qtprodvend,
			'VRUNITVEND'     => $vrunitvend,
			'VRDESITVEND'    => $vrdesitvend,
			'CDPRODUTO'      => $cdproduto,
			'NRORG'          => $NRORG,
			'NRSEQVENDA'     => $NRSEQVENDA,
			'NRSEQUITVEND'   => $nrsequitvend,
			'VRACRITVEND'    => $vracritvend,
			'VRUNITVENDCL'   => 0,
			'NRORDITCUPFIS'	 => $NRORDITCUPFIS,
			'VRPERCVEND'	 => $vrpercvend,
			'VRALIQIMPR'	 => $vraliqimpr,
			'NRATRAPRODITVE' => $nratraproditve,
			'VRRATTXSERV'	 => $vrrattxserv,
			'VRRATDESCVEN'	 => $vrratdescven,
			'DTHRPEDIDO'	 => $this->databaseUtil->convertToDateTime($dthrpedido),
			'CDCAIXAPEDVEN'	 => $CDCAIXA,
			'CDPRODPROMOCAO' => $CDPRODPROMOCAO,
			'NRSEQPRODCOMIT' => $NRSEQPRODCOMIT,
			'CDVENDEDOR'     => $CDVENDEDOR,
			'IDORIGPEDVEN'   => 'MOB', // @TODO - MELHORAR ISSO
			'QTPRODREFIL'    => 0
   		);
   		return $saleItemParams;
	}

	private function buildSaleItemCancParams($CDFILIAL, $CDCAIXA, $qtprodvendc, $vrunitvendc, $vrdesitvendc, $cdproduto, $NRORG, $NRSEQVENDA, $NRSEQITVENDC, $vracritvendc, $NRORDITCUPFISca) {
		$saleItemCancParams = array(
			'CDFILIAL'    	  => $CDFILIAL,
			'CDCAIXA'      	  => $CDCAIXA,
			'QTPRODVENDC'     => $qtprodvendc,
			'VRUNITVENDC'     => $vrunitvendc,
			'VRDESITVENDC'    => $vrdesitvendc,
			'CDPRODUTO'       => $cdproduto,
			'NRORG'           => $NRORG,
			'NRSEQVENDA'      => $NRSEQVENDA,
			'NRSEQITVENDC'    => $NRSEQITVENDC,
			'VRACRITVENDC'    => $vracritvendc,
			'NRORGINCLUSAO'	  => $NRORG,
			'VRUNITVENCLC'    => 0,
			'NRORDITCUPFISCA' => $NRORDITCUPFISca
   		);
   		return $saleItemCancParams;
	}

	private function buildPaymentParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrsequmovi, $DTVENDA, $idtipomovive, $vrmovivend, $qtparcreceb, $vrmoviVeout, $vrcotindout, $NRSEQVENDA, $NRORG, $cdtiporece, $CDOPERADOR, $CDCLIENTE, $CDNSUHOSTTEF) {
		$paymentParams = array(
			'CDFILIAL' 	   	 => $CDFILIAL,
			'CDCAIXA' 	   	 => $CDCAIXA,
			'DTABERCAIX'   	 => $this->databaseUtil->convertToDateTime($DTABERCAIX),
			'NRSEQUMOVI'   	 => $nrsequmovi,
			'DTHRINCMOV'	 => $this->databaseUtil->convertToDateTime($DTVENDA),
			'IDTIPOMOVIVE' 	 => $idtipomovive,
			'VRMOVIVEND'   	 => $vrmovivend,
			'QTPARCRECEB'    => $qtparcreceb,
			'VRMOVIVEOUT'    => $vrmoviVeout,
			'VRCOTINDOUT'    => $vrcotindout,
			'NRSEQVENDA'   	 => $NRSEQVENDA,
			'NRORG'			 => $NRORG,
			'CDTIPORECE'  	 => $cdtiporece,
			'DTMOVIMCAIXA'	 => $this->databaseUtil->getCurrentDate(),
			'CDOPERINCLUSAO' => $CDOPERADOR,
			'CDCLIENTE' 	 => $CDCLIENTE,
			'NRORGINCLUSAO'  => $NRORG,
			'CDNSUHOSTTEF'   => $CDNSUHOSTTEF
		);
		return $paymentParams;
	}

	private function buildMoviclieParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrseqmovclie, $CDCLIENTE, $CDCONSUMIDOR, $vrmovcli,
		$dtmovclie, $NRSEQVENDA, $DTABERCAIX, $nrseqmov, $cdtiporece, $vrsaldocons, $CDOPERADOR, $NRORG, $vrmovpescons,
	    $vrsaldpecons, $vrpesubsidio, $vrtetosubsid) {
		$movclieParams = array(
			'CDFILIAL' 	   	 => $CDFILIAL,
			'CDCAIXA' 	   	 => $CDCAIXA,
			'DTABERCAIX'   	 => $this->databaseUtil->convertToDateTime($DTABERCAIX),
			'NRSEQMOVCLI' 	 => $nrseqmovclie,
			'CDCLIENTE'      => $CDCLIENTE,
			'CDCONSUMIDOR'   => $CDCONSUMIDOR,
			'VRMOVCLI' 	     => $vrmovcli,
			'DTMOVCLI'       => $this->databaseUtil->convertToDateTime($dtmovclie),
			'DTMOVIMCLIE'    => $this->databaseUtil->convertToDate($dtmovclie),
			'IDTIPMOCVLI'    => 'D',
			'NRSEQVENDA'     => $NRSEQVENDA,
			'NRSEQUMOVI'     => $nrseqmov,
			'CDTIPORECE'     => $cdtiporece,
			'VRSALDOCONS'    => $vrsaldocons,
			'CDOPERADOR'     => $CDOPERADOR,
			'NRORG'          => $NRORG,
			'DTINCLUSAO'     => $dtmovclie,
			'NRORGINCLUSAO'  => $NRORG,
			'CDOPERINCLUSAO' => $CDOPERADOR,
			'NRORGULTATU'    => $NRORG,
			'VRMOVPESCONS' 	 => $vrmovpescons,
			'VRSALDPECONS' 	 => $vrsaldpecons,
			'VRPESUBSIDIO' 	 => $vrpesubsidio,
			'VRTETOSUBSID' 	 => $vrtetosubsid,
			'CDOPERULTATU'   => $CDOPERADOR,
			'DTINCLUSAO'     => $this->databaseUtil->getCurrentDate()
		);
		return $movclieParams;
	}

	private function buildPaymentTrocoParams($CDFILIAL, $CDCAIXA, $DTABERCAIX, $nrsequmovi, $DTVENDA, $vrmovivend, $NRSEQVENDA, $NRORG, $cdtiporece, $CDOPERADOR, $CDCLIENTE, $vrmoviVeout, $vrcotindout, $qtparcreceb) {
		$paymentParams = array(
			'CDFILIAL' 	   	 => $CDFILIAL,
			'CDCAIXA' 	   	 => $CDCAIXA,
			'DTABERCAIX'   	 => $DTABERCAIX,
			'NRSEQUMOVI'   	 => $nrsequmovi,
			'DTHRINCMOV'	 => $DTVENDA,
			'IDTIPOMOVIVE' 	 => "S",
			'VRMOVIVEND'   	 => $vrmovivend,
			'NRSEQVENDA'   	 => $NRSEQVENDA,
			'NRORG'			 => $NRORG,
			'CDTIPORECE'  	 => $cdtiporece,
			'DTMOVIMCAIXA'	 => $this->databaseUtil->getCurrentDate(),
			'CDOPERINCLUSAO' => $CDOPERADOR,
			'NRORGINCLUSAO'  => $NRORG,
			'CDCLIENTE'	 	 => $CDCLIENTE,
			'VRMOVIVEOUT'	 => $vrmoviVeout,
			'VRCOTINDOUT'    => $vrcotindout,
			'QTPARCRECEB'    => $qtparcreceb
		);
		return $paymentParams;
	}

	private function buildAuxSaleItemParams($CDFILIAL, $CDCAIXA, $nritemvendauxiliar, $CDFILIALalmoxarifado, $cdalmoxarifado, $dataconclusaoabastecimento, $nrtipocombustivel, $nrbomba, $nrbico, $NRORG, $NRSEQVENDA, $nrsequitvend, $vrencerranteinicial, $vrencerrantefinal) {
		$saleItemAuxParams = array(
			'CDFILIAL'    		  	     => $CDFILIAL,
			'CDCAIXA'      		 	     => $CDCAIXA,
			'NRITEMVENDAUXILIAR' 	     => $nritemvendauxiliar,
			'CDFILIALALMOXARIFADO'   	 => $CDFILIALalmoxarifado,
			'CDALMOXARIFADO'   		 	 => $cdalmoxarifado,
			'DATACONCLUSAOABASTECIMENTO' => $this->databaseUtil->convertToDateTime($dataconclusaoabastecimento),
			'NRTIPOCOMBUSTIVEL' 		 => $nrtipocombustivel,
			'NRBOMBA'	   				 => $nrbomba,
	        'NRBICO'      				 => $nrbico,
			'NRORG'        				 => $NRORG,
			'NRSEQVENDA'   				 => $NRSEQVENDA,
			'NRSEQUITVEND'				 => $nrsequitvend,
			'VRENCERRANTEINICIAL' 		 => $vrencerranteinicial,
			'VRENCERRANTEFINAL' 		 => $vrencerrantefinal
		);
		return $saleItemAuxParams;
	}

	private function buildGetNrfeaturegrupoParams($nrprodutodimensaovalorArray, $qtdimensoes) {
		$getNrfeaturegrupoParams = array(
			'NRPRODUTODIMENSAOVALOR' => $nrprodutodimensaovalorArray,
			'QTDIMENSOES' 			 => $qtdimensoes
		);
		return $getNrfeaturegrupoParams;
	}

	private function buildGetProdEstFeatureGruParams($nrfeaturegrupo) {
		$getProdEstFeatureGruParams = array(
			'NRFEATUREGRUPO' => $nrfeaturegrupo
		);
		return $getProdEstFeatureGruParams;
	}

   	private function buildAuxSaleItemFeatureParams($nritemvendauxiliar, $CDFILIAL, $CDCAIXA, $NRORG, $NRSEQVENDA, $nrsequitvend, $nrfeaturegrupo, $CDOPERADOR) {
		$auxSaleItemFeatureParams = array(
			'NRITEMVENDAUXILIAR' => $nritemvendauxiliar,
			'CDFILIAL' 			 => $CDFILIAL,
			'CDCAIXA'        	 => $CDCAIXA,
			'NRORG'    	     	 => $NRORG,
			'NRSEQVENDA'     	 => $NRSEQVENDA,
			'NRSEQUITVEND'   	 => $nrsequitvend,
			'NRFEATUREGRUPO' 	 => $nrfeaturegrupo,
			'NRORGINCLUSAO'	 	 => $NRORG,
			'NRORGULTATU'    	 => $NRORG,
			'CDOPERINCLUSAO' 	 => $CDOPERADOR,
			'CDOPERULTATU'	 	 => $CDOPERADOR
		);
		return $auxSaleItemFeatureParams;
	}

	private function buildAtribsIdentParams($nrrelacatributo, $CDFILIAL, $CDCAIXA, $NRSEQVENDA, $cdtiporece, $nratributovenda, $valoratributo, $NRORG, $DTVENDA, $CDOPERADOR) {
		$atribsIdentsParams = array(
			'NRRELACATRIBUTO' => $nrrelacatributo,
            'CDFILIAL'        => $CDFILIAL,
            'CDCAIXA'         => $CDCAIXA,
            'NRSEQVENDA'      => $NRSEQVENDA,
            'NRATRIBUTOVENDA' => $nratributovenda,
            'CDTIPORECE'      => $cdtiporece,
            'VALORATRIBUTO'   => $valoratributo,
            'NRORG'           => $NRORG,
            'DTINCLUSAO'      => $this->databaseUtil->convertToDateTime($DTVENDA),
            'NRORGINCLUSAO'   => $NRORG,
            'CDOPERINCLUSAO'  => $CDOPERADOR,
            'NRORGULTATU'     => $NRORG,
            'CDOPERULTATU'    => $CDOPERADOR
		);
		return $atribsIdentsParams;
	}

    private function updateIntegracaomov($nrseqmov, $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG) {
		$params = self::buildIntegracaomovParams($nrseqmov, $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG);
		$this->entityManager->getConnection()->executeQuery(self::UPDATE_NRSEQVENDA_VND_INTEGRACAOMOV, $params);
    }

    const UPDATE_NRSEQVENDA_VND_INTEGRACAOMOV = "
         UPDATE VND_INTEGRACAOMOV
            SET NRSEQVENDA = :NRSEQVENDA
         WHERE NRSEQMOV = :NRSEQMOV AND CDFILIAL = :CDFILIAL AND CDCAIXA = :CDCAIXA AND NRORG = :NRORG
     ";

    private function buildIntegracaomovParams($nrseqmov, $NRSEQVENDA, $CDFILIAL, $CDCAIXA, $NRORG) {
		return array(
			'NRSEQMOV'   => $nrseqmov,
            'CDFILIAL'   => $CDFILIAL,
            'CDCAIXA'    => $CDCAIXA,
            'NRSEQVENDA' => $NRSEQVENDA,
            'NRORG'      => $NRORG
		);
	}

	private function mergePaymentModes($paymentModes) {
		for ($i = 0; $i < count($paymentModes); $i++) {
			for($j = $i+1; $j < count($paymentModes); $j++) {
				$id = 0;
				if ($paymentModes[$i]['CDTIPORECE'] == $paymentModes[$j]['CDTIPORECE']) {
					$paymentModes[$i]['VALOR'] += $paymentModes[$j]['VALOR'];
					$id = 1;
				}
				if ($id == 1) {
					array_splice($paymentModes, $j, 1);
					$j--;
				}
			}
		}
		return $paymentModes;
	}

	private function buscaClientePadrao($CDFILIAL, $NRORG) {
		$CDCLIENTE = null;
		$getCliepadrao = array(
	    	'CDFILIAL' => $CDFILIAL,
	    	'NRORG' => $NRORG
   		);
    	$resultCDCLIENTE = $this->entityManager->getConnection()->fetchAll(self::GET_CLIENTEPADRAO, $getCliepadrao);
    	if (isset($resultCDCLIENTE[0])) {
			$CDCLIENTE = $resultCDCLIENTE[0]['CDCLIENTE'];
		}
		return $CDCLIENTE;
	}

    const GET_CLIENTEPADRAO = "
        SELECT CDCLIENTE
          FROM PARAVEND
         WHERE NRORG    = :NRORG
           AND CDFILIAL = :CDFILIAL
    ";

	private function prorateDiscount(&$saleItens, $saleDiscount, $saleValue) {
	    //Divide o desconto da venda nos valores unitários dos itens
	    //Dependendo do valor cria dízimas periódicas no valor unitário
	    $totalSale = $saleValue + $saleDiscount;
	    $percent = 1 - ($saleDiscount / $totalSale);
		foreach ($saleItens as &$item) {
			$vrUnit = $item['VRUNITVEND'];
			$qtProd = $item['QTPRODVEND'];
			$vrDes = $item['VRDESITVEND'];
			$discount = $vrDes/$qtProd;
			$vrUnit -= $discount;
			$vrUnit *= $percent;
			$vrUnit += $discount;
			$item['VRUNITVEND'] = $vrUnit;
		}
	}

	private function calcVrSale($items) {
		$vr = 0;
		$vrTotal = 0;
		$produtosCobrados = self::separaItensQueSeraoCobrados($items);
		foreach ($produtosCobrados as $item) {
			$qnt = $item['QTPRODVEND'];
			$vrPrecItem = $item['VRUNITVEND'];
			$vrDescItem = $item['VRDESITVEND'];
			$vrAcreItem = $item['VRACRITVEND'];
			if ((($qnt * $vrPrecItem) - $vrDescItem) != 0.01) {
				$vr += (($qnt * $vrPrecItem) - $vrDescItem + $vrAcreItem);
			}
			$vrTotal += (($qnt * $vrPrecItem) - $vrDescItem + $vrAcreItem);
		}
		return array($vr, $vrTotal);
	}

}