<?php
namespace Zeedhi\PaymentPlatform\Implementations\BoletoPHP;

use Zeedhi\PaymentPlatform\BankSlip\BankSlipPlatform;
use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;
use Zeedhi\PaymentPlatform\BankSlip\Store;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Exception;

class PaymentImpl implements BankSlipPlatform{
    
    /** @var Store */
    private $store;
    /** @var BankSlipRequireFiles */
    private $generateBankSlip;
    
    /**
     * PaymentImpl constructor.
     *
     * @param Store $store
     * @param BankSlipRequireFiles $generateBankSlip
     */
    public function __construct(Store $store, BankSlipRequireFiles $generateBankSlip){
        $this->store = $store;
        $this->generateBankSlip = $generateBankSlip;
    }
    
        
    /**
     * @param Address $address
     * 
     * @return String
     */
     private function addressStringify(Address $address) {
         return $address->getStreet()
                .' '.$address->getNumber()
                .' '.$address->getComplement()
                .' '.$address->getNeighborhood()
                .' '.$address->getCity()
                .' '.$address->getState()
                .' '.$address->getZipCode()
                .' '.$address->getCountry();
     }

    /**
     * @param BankSlipTransaction $bankSlipTransaction
     *
     * @return array
     */
    public function bankSlipTransaction(BankSlipTransaction $bankSlipTransaction){
        if ($bankSlipTransaction->getBank() == '237') {
            $dadosboleto["carteira"] = '06';  // Código da Carteira: pode ser 06 ou 03
            $dadosboleto["bankName"] = 'bradesco';
        } else if ($bankSlipTransaction->getBank() == '001') {
            $dadosboleto["carteira"] = '17';
            $dadosboleto["bankName"] = 'bb';
        } else if ($bankSlipTransaction->getBank() == '341') {
            $dadosboleto["carteira"] = '109';
            $dadosboleto["bankName"] = 'itau';
        } else if ($bankSlipTransaction->getBank() == '033') {
            $dadosboleto["carteira"] = '102';
            $dadosboleto["bankName"] = 'santander';
        } else if ($bankSlipTransaction->getBank() == '399') {
            $dadosboleto["carteira"] = 'CNR';
            $dadosboleto["bankName"] = 'hsbc';
        } else {
            throw Exception::unsupportedBank();
        }

        $prazoPagamento = $bankSlipTransaction->getDaysToAddInBankSlipExpirationDate(); //prazo para pagamento a partir do dia da geração
        $valorBoleto    = $bankSlipTransaction->getValue(); //Valor total do boleto
        $nossoNumero    = $this->store->getStoreBank()->getOurNumber(); //Número de registro da empresa no banco
        $nrDocumento    = $bankSlipTransaction->getDocumentNumber(); //Número que identifica o boleto

        $nomeCliente    = $bankSlipTransaction->getCustomer()->getName(); //Nome do cliente
        $endereco1      = $this->addressStringify($bankSlipTransaction->getCustomer()->getAddress()); //Endereço do cliente

        $nomeFantasia   = $this->store->getFancyName(); //Nome fantasia da loja
        $site           = $this->store->getWebSite(); //Site para contato da loja
        $cpfCnpj        = $this->store->getCpfCnpj(); //Cpf ou CNPJ da loja
        $enderecoLoja   = $this->addressStringify($this->store->getStoreAddress()); //Endereço da loja
        $cidade_uf      = $this->store->getStoreAddress()->getCity().' '.$this->store->getStoreAddress()->getState(); //Formato: Cidade/Estado
        $razaoSocial    = $this->store->getBusinessName(); // Razão social da loja

        $agenciaLoja    = $this->store->getStoreBank()->getAgency(); //Agencia da loja
        $agenciaDvLoja  = $this->store->getStoreBank()->getAgencyDigit(); //Dígito verificador da agencia
        $contaLoja      = $this->store->getStoreBank()->getAccount(); //Conta da loja
        $contaDvLoja    = $this->store->getStoreBank()->getAccountDigit(); //Dígito verificador da conta

        $data_venc = date("d/m/Y", time() + ($prazoPagamento * 86400));  // Prazo de X dias OU informe data: "13/04/2006";
        $valor_boleto = number_format(str_replace(",", ".", $valorBoleto), 2, ',', '');

        $dadosboleto["nosso_numero"] = $nossoNumero;  // Nosso numero sem o DV - REGRA: Máximo de 11 caracteres!
        $dadosboleto["numero_documento"] = $nrDocumento;    // Num do pedido ou do documento
        $dadosboleto["data_vencimento"] = $data_venc; // Data de Vencimento do Boleto - REGRA: Formato DD/MM/AAAA
        $dadosboleto["data_documento"] = date("d/m/Y"); // Data de emissão do Boleto
        $dadosboleto["data_processamento"] = date("d/m/Y"); // Data de processamento do boleto (opcional)
        $dadosboleto["valor_boleto"] = $valor_boleto;   // Valor do Boleto - REGRA: Com vírgula e sempre com duas casas depois da virgula

        // DADOS DO SEU CLIENTE
        $dadosboleto["sacado"] = $nomeCliente;
        $dadosboleto["endereco1"] = $endereco1;
        $dadosboleto["endereco2"] = '';

        // INFORMACOES PARA O CLIENTE
        $dadosboleto["demonstrativo1"] = $bankSlipTransaction->getInstructions();
        $dadosboleto["demonstrativo2"] = "";
        $dadosboleto["demonstrativo3"] = "";
        $dadosboleto["instrucoes1"] = "- Favor nao receber apos o vencimento";
        $dadosboleto["instrucoes2"] = $site;
        $dadosboleto["instrucoes3"] = "";
        $dadosboleto["instrucoes4"] = "";

        // DADOS OPCIONAIS DE ACORDO COM O BANCO OU CLIENTE
        $dadosboleto["quantidade"] = "001";
        $dadosboleto["valor_unitario"] = $valor_boleto;
        $dadosboleto["aceite"] = "";
        $dadosboleto["especie"] = "R$";
        $dadosboleto["especie_doc"] = "DS";

        // Bradesco: todos os campos
        // BB: Só utiliza agencia e conta, sem os dígitos
        // Itaú: não usa dígito da agência
        $dadosboleto["agencia"] = $agenciaLoja; // Num da agencia, sem digito
        $dadosboleto["agencia_dv"] = $agenciaDvLoja; // Digito do Num da agencia
        $dadosboleto["conta"] = $contaLoja;  // Num da conta, sem digito
        $dadosboleto["conta_dv"] = $contaDvLoja;     // Digito do Num da conta

        // DADOS PERSONALIZADOS - Bradesco
        $dadosboleto["conta_cedente"] = ""; // ContaCedente do Cliente, sem digito (Somente Números)
        $dadosboleto["conta_cedente_dv"] = ""; // Digito da ContaCedente do Cliente

        // DADOS PERSONALIZADOS - BB
        /*
        #################################################
        DESENVOLVIDO PARA CARTEIRA 18
        ALERTA: A CARTEIRA FOI TROCADA DE 18 PARA 17 (15/10/2015).

        - Carteira 18 com Convenio de 8 digitos
          Nosso número: pode ser até 9 dígitos

        - Carteira 18 com Convenio de 7 digitos
          Nosso número: pode ser até 10 dígitos

        - Carteira 18 com Convenio de 6 digitos
          Nosso número:
          de 1 a 99999 para opção de até 5 dígitos
          de 1 a 99999999999999999 para opção de até 17 dígitos

        #################################################
        */
        $dadosboleto["convenio"] = $this->store->getAgreement();  // Num do convênio - REGRA: 6 ou 7 ou 8 dígitos
        $dadosboleto["contrato"] = $this->store->getContract(); // Num do seu contrato
        $dadosboleto["formatacao_convenio"] = strlen($this->store->getAgreement()); // REGRA: 8 p/ Convênio c/ 8 dígitos, 7 p/ Convênio c/ 7 dígitos, ou 6 se Convênio c/ 6 dígitos

        // FORMATACAO NOSSO NUMERO
        // REGRA: Usado apenas p/ Convênio c/ 6 dígitos: informe 1 se for NossoNúmero de até 5 dígitos ou 2 para opção de até 17 dígitos
        if (strlen($this->store->getAgreement()) == 6) {
            if (strlen($nossoNumero) <= 5) {
                $dadosboleto["formatacao_nosso_numero"] = '1';
            } else {
                $dadosboleto["formatacao_nosso_numero"] = '2';
            }
        }
        // FIM DADOS PERSONALIZADOS BB

        // SEUS DADOS
        $dadosboleto["identificacao"] = $nomeFantasia;
        $dadosboleto["cpf_cnpj"] = $cpfCnpj;
        $dadosboleto["endereco"] = $enderecoLoja;
        $dadosboleto["cidade_uf"] = $cidade_uf;
        $dadosboleto["cedente"] = $razaoSocial;


        /*
        033 - Santander
        399 - HSBC
        */
        if ($bankSlipTransaction->getBank() == '033') {
            // santander
            $dadosboleto["codigo_cliente"] = $this->store->getStoreBank()->getAssignorNumber();
            $dadosboleto["ponto_venda"] = $agenciaLoja;
            $dadosboleto["carteira_descricao"] = "COBRANÇA SIMPLES - CSR";
        } else if ($bankSlipTransaction->getBank() == '399') {
            $dadosboleto["codigo_cedente"] = $this->store->getStoreBank()->getAssignorNumber();
        } else if ($bankSlipTransaction->getBank() == '237') {
            $dadosboleto["codigo_cedente"] = substr($this->store->getStoreBank()->getAssignorNumber(),0,7);
            $dadosboleto["codigo_cedente_dv"] = substr($this->store->getStoreBank()->getAssignorNumber(),7);
        }

        $this->handleGenerateBankSlip($dadosboleto);
    }        

    /**
     * @param array $bankSlipTransaction
     *
     */
    public function handleGenerateBankSlip ($bankSlipTransaction) {
        $this->generateBankSlip->bankSlipConstruct($bankSlipTransaction);
    }

    /**
     * @param Store $store
     *
     * @return PaymentImpl
     */
    public static function factory(Store $store) {
        return new self($store, new BankSlipRequireFiles);
    }
}