<?php

namespace Zeedhi\PaymentPlatform\Implementations\Cielo;

use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;
use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;
use Zeedhi\PaymentPlatform\Revoke\Revoke;
use Zeedhi\PaymentPlatform\Consult\Consult;
use Zeedhi\PaymentPlatform\RequestURL;
use Zeedhi\PaymentPlatform\Address;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\Exception;

class PaymentImplTest extends \PHPUnit_Framework_TestCase {
    
    protected $paymentImplCielo;
    protected $merchantKey;
    protected $merchantId;

    public function setUp(){
        $this->httpRequest = $this->getMockBuilder(HttpRequest::class)
							 ->setMethods(array('setUrl', 'setOption', 'execute', 'getInfo', 'close'))
            				 ->disableOriginalConstructor()
            				 ->getMock();

        $this->paymentImplCielo = new PaymentImpl(
            $this->merchantKey,
            $this->merchantId,
            $this->httpRequest,
            PaymentImpl::ENDPOINT_SANDBOX
        );
    }

    public function testSuccessNonInstantBuyTransaction() {
        $jsonResponse = '{
            "Payment": {
                "ServiceTaxAmount": 0,
                "Installments": 1,
                "Interest": 0,
                "Capture": false,
                "Authenticate": false,
                "Recurrent": false,
                "CreditCard": {
                    "SaveCard": false,
                    "CardToken": "L0N+g2hPywCHnI4bkRs4f/xzxcKEof0Cuo3Owkl/TVE=",
                    "Brand": "Amex"
                },
                "Tid": "0405021924668",
                "ProofOfSale": "1924668",
                "AuthorizationCode": "706352",
                "Provider": "Simulado",
                "Amount": 100,
                "ReceivedDate": "2018-04-05 14:19:24",
                "Status": 1,
                "IsSplitted": false,
                "ReturnMessage": "Operation Successful",
                "ReturnCode": "4",
                "PaymentId": "e72546b7-c710-4832-a8c2-e5ba63ebf2a8",
                "Type": "CreditCard",
                "Currency": "BRL",
                "Country": "BRA",
                "Links": [
                    {
                        "Method": "GET",
                        "Rel": "self",
                        "Href": "https://apiquerysandbox.cieloecommerce.cielo.com.br/1/sales/e72546b7-c710-4832-a8c2-e5ba63ebf2a8"
                    },
                    {
                        "Method": "PUT",
                        "Rel": "capture",
                        "Href": "https://apisandbox.cieloecommerce.cielo.com.br/1/sales/e72546b7-c710-4832-a8c2-e5ba63ebf2a8/capture"
                    },
                    {
                        "Method": "PUT",
                        "Rel": "void",
                        "Href": "https://apisandbox.cieloecommerce.cielo.com.br/1/sales/e72546b7-c710-4832-a8c2-e5ba63ebf2a8/void"
                    }
                ]
            }
        }';
        $response = array(
            "status" => "4",
            "message" => "Operation Successful",
            "nsu" => "1924668"
        );
        
        $HTTPStatusCode = '201';

        $this->httpRequest->expects($this->once())
        			 ->method('setUrl');

        $this->httpRequest->expects($this->exactly(10))
        			 ->method('setOption');

		$this->httpRequest->expects($this->once())
        			 ->method('execute')
        			 ->willReturn($jsonResponse);

        $this->httpRequest->expects($this->once())
        			 ->method('getInfo')
        			 ->willReturn($HTTPStatusCode);

        $this->httpRequest->expects($this->once())
        			 ->method('close');
        $creditCard = new CreditCard("TEKNISA TESTES", "0000000000000001", "12", "2021", "123", "Visa");
        $customer = new Customer();
        $nonInstantBuyTransaction = new NonInstantBuyTransaction("100.00", "2014111703", $creditCard, $customer);
        $theResponse = $this->paymentImplCielo->nonInstantBuyTransaction($nonInstantBuyTransaction);
        $this->assertEquals($theResponse, $response);
    }

    public function testNonInstantBuyTransactionWithInvalidYear() {
        
        $this->expectException(Exception::class);
        $this->expectExceptionMessage("HTTP Status: 400. API Cielo informa o seguinte erro: Credit Card Expiration Date is invalid");
        $this->expectExceptionCode(Exception::API_CIELO_ERROR);
        $invalidYear = "18";
        $creditCard = new CreditCard("TEKNISA TESTES", "0000000000000001", "12", $invalidYear, "123", "Visa");
        $customer = new Customer();
        $HTTPStatusCode = '400';

        $jsonResponse = '[
            {
              "Code": 126,
              "Message": "Credit Card Expiration Date is invalid"
            }
          ]';
        
        $nonInstantBuyTransaction = new NonInstantBuyTransaction("100.00", "2014111703", $creditCard, $customer);

		$this->httpRequest->expects($this->once())
        ->method('setUrl');
        
        $this->httpRequest->expects($this->exactly(10))
        ->method('setOption');
        
		$this->httpRequest->expects($this->once())
        ->method('execute')
        ->willReturn($jsonResponse);
        
        $this->httpRequest->expects($this->once())
        ->method('getInfo')
        ->willReturn($HTTPStatusCode);
        
        $this->httpRequest->expects($this->once())
        ->method('close');
        $this->paymentImplCielo->nonInstantBuyTransaction($nonInstantBuyTransaction);
    }

    public function testSuccessCreateToken() {
        $creditCard = new CreditCard("TEKNISA TESTES", "0000000000000001", "12", "2018", "123", "Visa");
        $customer = new Customer();

        $jsonResponse = '{"CardToken":"f380ba4d-1715-4019-8980-726ba3fcd85b","Links":{"Method":"GET","Rel":"self","Href":"https://apiquerysandbox.cieloecommerce.cielo.com.br/1/card/f380ba4d-1715-4019-8980-726ba3fcd85b"}}';
        $finalResponse = array(
            "InstantBuyKey" => "f380ba4d-1715-4019-8980-726ba3fcd85b"
        );
        $HTTPStatusCode = '200';
        
        $this->httpRequest->expects($this->once())
        ->method('setUrl');
        
        $this->httpRequest->expects($this->exactly(10))
        ->method('setOption');
        
		$this->httpRequest->expects($this->once())
        ->method('execute')
        ->willReturn($jsonResponse);
        
        $this->httpRequest->expects($this->once())
        ->method('getInfo')
        ->willReturn($HTTPStatusCode);
        
        $this->httpRequest->expects($this->once())
        ->method('close');

        $token = $this->paymentImplCielo->createToken($creditCard, $customer);
        $this->assertEquals($token, $finalResponse);
    }

    public function testCreateTokenWithInvalidDate(){
        $this->expectException(Exception::class);
        $this->expectExceptionMessage("HTTP Status: 400. API Cielo informa o seguinte erro: Credit Card Expiration Date is invalid");
        $this->expectExceptionCode(Exception::API_CIELO_ERROR);
        $invalidYear = "18";
        $creditCard = new CreditCard("TEKNISA TESTES", "0000000000000001", "12", $invalidYear, "123", "Visa");
        $customer = new Customer();
        $HTTPStatusCode = '400';

        $jsonResponse = '[
            {
              "Code": 126,
              "Message": "Credit Card Expiration Date is invalid"
            }
          ]';

        $this->httpRequest->expects($this->once())
        ->method('setUrl');
        
        $this->httpRequest->expects($this->exactly(10))
        ->method('setOption');
        
		$this->httpRequest->expects($this->once())
        ->method('execute')
        ->willReturn($jsonResponse);
        
        $this->httpRequest->expects($this->once())
        ->method('getInfo')
        ->willReturn($HTTPStatusCode);
        
        $this->httpRequest->expects($this->once())
        ->method('close');
        
        $this->paymentImplCielo->createToken($creditCard, $customer);
    }

    public function testInstantBuyTransaction() {
        $customer = new Customer();
        $instantBuyTransaction = new InstantBuyTransaction("100.00", "0000001", "f380ba4d-1715-4019-8980-726ba3fcd85b", $customer);
        $instantBuyTransaction->setFlag("Visa");

        $HTTPStatusCode = '200';
        $jsonResponse = '{
            "Payment": {
                "ServiceTaxAmount": 0,
                "Installments": 1,
                "Interest": 0,
                "Capture": false,
                "Authenticate": false,
                "Recurrent": false,
                "CreditCard": {
                    "SaveCard": false,
                    "CardToken": "L0N+g2hPywCHnI4bkRs4f/xzxcKEof0Cuo3Owkl/TVE=",
                    "Brand": "Amex"
                },
                "Tid": "0405021924668",
                "ProofOfSale": "1924668",
                "AuthorizationCode": "706352",
                "Provider": "Simulado",
                "Amount": 100,
                "ReceivedDate": "2018-04-05 14:19:24",
                "Status": 1,
                "IsSplitted": false,
                "ReturnMessage": "Operation Successful",
                "ReturnCode": "4",
                "PaymentId": "e72546b7-c710-4832-a8c2-e5ba63ebf2a8",
                "Type": "CreditCard",
                "Currency": "BRL",
                "Country": "BRA",
                "Links": [
                    {
                        "Method": "GET",
                        "Rel": "self",
                        "Href": "https://apiquerysandbox.cieloecommerce.cielo.com.br/1/sales/e72546b7-c710-4832-a8c2-e5ba63ebf2a8"
                    },
                    {
                        "Method": "PUT",
                        "Rel": "capture",
                        "Href": "https://apisandbox.cieloecommerce.cielo.com.br/1/sales/e72546b7-c710-4832-a8c2-e5ba63ebf2a8/capture"
                    },
                    {
                        "Method": "PUT",
                        "Rel": "void",
                        "Href": "https://apisandbox.cieloecommerce.cielo.com.br/1/sales/e72546b7-c710-4832-a8c2-e5ba63ebf2a8/void"
                    }
                ]
            }
        }';
        $response = array(
            "status" => "4",
            "message" => "Operation Successful",
            "nsu" => "1924668"
        );

        $this->httpRequest->expects($this->once())
        ->method('setUrl');
        
        $this->httpRequest->expects($this->exactly(10))
        ->method('setOption');
        
		$this->httpRequest->expects($this->once())
        ->method('execute')
        ->willReturn($jsonResponse);
        
        $this->httpRequest->expects($this->once())
        ->method('getInfo')
        ->willReturn($HTTPStatusCode);
        
        $this->httpRequest->expects($this->once())
        ->method('close');

        $theResponse = $this->paymentImplCielo->instantBuyTransaction($instantBuyTransaction);
        $this->assertEquals($theResponse, $response);
    }   
}