describe('TabbedWidgetController', function() {
    var tabbedWidgetController;
    var metaDataFactory;
    var container;
    var $scope;
    var $rootScope;
    var templateManager;
    var screenService;
    var $timeout;
    var eventEngine;
    var zhPromise;

    var baseContainer = ZhHelper.baseContainerWithTwoWidgets;

    beforeEach(function() {
        module(function($provide) {
            //Providing a new $timeout because $timeout.flush is not flushing pending $timeout calls
            $provide.constant('$timeout', setTimeout);
            $provide.constant('$q', Q);
        });
        module('ZeedhiApp');
        module('ZeedhiServices');

        inject(function(_$rootScope_, $controller, _templateManager_, _ScreenService_, _$timeout_, _eventEngine_, _metaDataFactory_, _ZHPromise_) {
            metaDataFactory = _metaDataFactory_;
            screenService = _ScreenService_;
            templateManager = _templateManager_;
            $timeout = _$timeout_;
            eventEngine = _eventEngine_;
            zhPromise = _ZHPromise_;

            container = metaDataFactory.containerFactory(baseContainer());
            $rootScope = _$rootScope_;
            _$rootScope_.swipes = [];

            $scope = _$rootScope_.$new();
            $scope.$parent = _$rootScope_;
            $scope.$parent.widget = $scope.widget = container.widgets[0];

            $controller('SwipeController', {
                $scope: _$rootScope_.$new()
            });

            tabbedWidgetController = new TabbedWidgetController($scope, _templateManager_, _ScreenService_, _$timeout_, _eventEngine_, _ZHPromise_);
        });
    });

    it('setDefaultTab: should set the first child widget if no tab is passed', function(done) {
        var parentWidget = $scope.$parent.widget;
        parent.currentWidget = null;
        $scope.setDefaultTab().then(function() {
            expect(parentWidget.currentWidget).toBeDefined();
            expect(parentWidget.currentWidget).toEqual(container.getWidget('imageChild'));
            expect(parentWidget.currentWidget.isActive()).toBeTruthy();
            done();
        });
    });

    it('setDefaultTab: should set the passed tab as the currentWidget', function(done) {
        var parentWidget = $scope.$parent.widget;
        parent.currentWidget = null;
        $scope.setDefaultTab('imageSecondChild').then(function() {
            expect(parentWidget.currentWidget).toBeDefined();
            expect(parentWidget.currentWidget).toEqual(container.getWidget('imageSecondChild'));
            expect(parentWidget.currentWidget.isActive()).toBeTruthy();
            done();
        });
    });

    it('$scope.widget.activate(): the tabbedWidgetController should activate the child active tabbed too', function(done) {
        var parentWidget = $scope.$parent.widget;
        var imageSecondChild = container.getWidget('imageSecondChild');
        $scope.setDefaultTab(imageSecondChild).then(function() {
            container.getWidget('tabbedWidget').deactivate();
            imageSecondChild.deactivate();
            parent.currentWidget = imageSecondChild;
            container.getWidget('tabbedWidget').activate();
            expect(imageSecondChild.isActive()).toBeTruthy();
            done();
        });
    });

    it('$scope.$broadcast("destroy"): the tabbedWidgetController should remove his onWidgetActivate listener', function() {
        var lastListenersLength = $scope.widget.onActivatelistener.length;
        $scope.$broadcast('$destroy');

        expect($scope.widget.onActivatelistener.length).toBe(lastListenersLength - 1);
    });

    it('onClickTab: should set the currentWidget and activate the childWidget', function() {
        var parentWidget = $scope.$parent.widget;
        var childWidget = container.getWidget('imageSecondChild');
        childWidget.deactivate();
        parent.currentWidget = null;
        $scope.onClickTab(parentWidget, childWidget);

        expect(parentWidget.currentWidget).toBeDefined();
        expect(parentWidget.currentWidget).toEqual(childWidget);
        expect(childWidget.isActive()).toBeTruthy();
    });

    it('onClickTab: should close the swipes  that are greater if the clicked tab is different from the current clicked tab', function(done) {
        var parentWidget = $scope.$parent.widget;
        var childWidget = container.getWidget('imageSecondChild');
        screenService.openSwipe(childWidget);
        screenService.openSwipe(container.getWidget('imageChild'));

        expect($rootScope.swipes.length).toBe(2);

        childWidget.deactivate();
        parent.currentWidget = null;
        var tabClickPromise = $scope.onClickTab(parentWidget, childWidget);

        expect(tabClickPromise).toBeDefined();
        tabClickPromise.then(function() {
            expect(parentWidget.currentWidget).toBeDefined();
            expect(parentWidget.currentWidget).toEqual(childWidget);
            expect(childWidget.isActive()).toBeTruthy();
            expect($rootScope.swipes.length).toBe(1);
            done();
        });
    });

    it('isTabEnabled: should return true if the tab is Active', function() {
        var parentWidget = $scope.$parent.widget;
        var childWidget = container.getWidget('imageSecondChild');
        childWidget.activate();
        expect($scope.isTabEnabled(parentWidget, childWidget)).toBeTruthy();

        tabbedWidgetController._enabledTabs = [childWidget];
        expect($scope.isTabEnabled(parentWidget, childWidget)).toBeTruthy();
    });

    it('isTabEnabled: should return false if the tab is not active', function() {
        var parentWidget = $scope.$parent.widget;
        var childWidget = container.getWidget('imageSecondChild');
        childWidget.setEnabled(false);
        expect($scope.isTabEnabled(parentWidget, childWidget)).toBeFalsy();
    });

    it('setEnabledTabs: should reset the enabled tabs and set the tabs passed as arguments', function() {
        var parentWidget = $scope.$parent.widget;
        $scope.widget.setEnabledTabs(['imageChild', 'imageSecondChild']);
        expect(tabbedWidgetController._enabledTabs.length).toBe(2);
        expect(tabbedWidgetController._enabledTabs[0]).toEqual(container.getWidget('imageChild'));
        expect(tabbedWidgetController._enabledTabs[1]).toEqual(container.getWidget('imageSecondChild'));
    });

    it('clearEnabledTabs: should clear the enabled tabs', function() {
        var parentWidget = $scope.$parent.widget;
        $scope.widget.setEnabledTabs(['imageChild', 'imageSecondChild']);
        expect(tabbedWidgetController._enabledTabs.length).toBe(2);
        expect(tabbedWidgetController._enabledTabs[0]).toEqual(container.getWidget('imageChild'));
        expect(tabbedWidgetController._enabledTabs[1]).toEqual(container.getWidget('imageSecondChild'));

        $scope.widget.clearEnabledTabs();
        expect(tabbedWidgetController._enabledTabs.length).toBe(0);
    });

    it('updateTabbed: should trigger widget onEnterTab when changing the parent currentWidget', function() {
        var onEnterTabCallCount = 0;
        var parentWidget = $scope.$parent.widget;
        var widgetToSetCurrent = container.getWidget('imageChild');
        $scope.widget.currentWidget = null;
        widgetToSetCurrent.onEnterTab = function() {
            onEnterTabCallCount++;
        };
        $scope.$apply();
        expect(onEnterTabCallCount).toBe(0);
        $scope.widget.currentWidget = container.getWidget('imageChild');
        $scope.$apply();
        expect(onEnterTabCallCount).toBe(1);
    });

    it('$scope.tabbedWidget.edit: should change widget original function', function(){
        var parentWidget = $scope.$parent.widget;
        expect(parentWidget.mode).toBe('list');
        parentWidget.edit();
        expect(parentWidget.mode).toBe('edit');
        expect(parentWidget.formTemplate).toBe('widget/tabbedWidget.html');
        expect(parentWidget.childTemplate).toBe('widget/form.html');
    });

    it('$scope.tabbedWidget.view: should change widget original function', function(){
        var parentWidget = $scope.$parent.widget;
        expect(parentWidget.mode).toBe('list');
        parentWidget.view();
        expect(parentWidget.mode).toBe('view');
        expect(parentWidget.formTemplate).toBe('widget/tabbedWidget.html');
        expect(parentWidget.childTemplate).toBe('widget/form-view.html');
    });

    it('Property _tabbedMethodsUpdated must be changed to prevent override methods multiple times', function() {
        container.widgets[0]._tabbedMethodsUpdated = undefined;
        $scope = $rootScope.$new();
        $scope.$parent = $rootScope;
        $scope.$parent.widget = $scope.widget = container.widgets[0];

        var parentWidget = $scope.$parent.widget;
        expect(parentWidget._tabbedMethodsUpdated).toBe(undefined);
        spyOn(parentWidget, 'edit');

        parentWidget.innerEdit = parentWidget.edit;

        tabbedWidgetController = new TabbedWidgetController($scope, templateManager, screenService, $timeout, eventEngine, zhPromise);
        expect(parentWidget._tabbedMethodsUpdated).toBe(true);
        expect(parentWidget.innerEdit).not.toBe(parentWidget.edit);

        parentWidget.edit();
        expect(parentWidget.innerEdit).toHaveBeenCalled();

        parentWidget.secondEdit = parentWidget.edit;

        tabbedWidgetController = new TabbedWidgetController($scope, templateManager, screenService, $timeout, eventEngine, zhPromise);
        expect(parentWidget.secondEdit).toBe(parentWidget.edit);
    });

});
