
// FILE: src/CarouselWidgetCreator.js
function CarouselWidgetCreator(metaDataFactory, ScreenService) {

	this.createCaroulselWidget = function(gridWidget, functionsToInsertOnWidget){
		var caroulsel = {
			"id": 1,
			"container": gridWidget.container,
			"name": "carouselWidget",
			"showAsModal": true,
			"isVisible": true,
			"template": "widget/carousel.html",
			"actions": []
		};
		var caroulselFactored = metaDataFactory.widgetFactory(caroulsel);
		caroulselFactored.widgets = createChildrenCaroulselWidget(gridWidget, caroulselFactored, functionsToInsertOnWidget);
		return caroulselFactored;
	};

	var createChildrenCaroulselWidget = function(gridWidget, caroulselWidget, functionsToInsertOnWidget){
		var children = [
			createPreferencesWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsPreferencesListWidget),
			createPreferenceDetailsWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsPreferenceFieldsListWidget),
			createPropertyPreferencesWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsFieldsPropertiesListWidget),
			createNewPreferenceWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsNewPreferenceWidget),
			createChangePreferenceTypeWidget(gridWidget, caroulselWidget, functionsToInsertOnWidget.functionsChangePreferenceTypeWidget)
		];
		return children;
	};

	function createPreferencesWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 1,
			"showAsModal": true,
			"container": gridWidget.container,
			"name": "popUpPreferencesList",
			"label": ScreenService.i18n("Layouts"),
			"isVisible": true,
			"template": "widget/grid.html",
			"showCheckbox": true,
			"actions": actionsPreferencesWidget,
			"floatingControl": {
				"refreshAction": false,
				"customizationAction": {
					"layout": false,
					"filter": false,
					"view": false
				},
				"actions": []
			},
			"events": [
				{
					"id": 2,
					"name": "WidgetOnEnter",
					"code": "args.owner.preferencesListOnEnter(args.owner);"
				},
				{
					"id": 3,
					"name": "WidgetAfterMoveRow",
					"code": "args.owner.openEditPreference(args.owner, args.row);"
				}
			],
			"parentWidget": parentWidget,
			"dataSource": {
				"data": [],
				"localStorage": false,
				"rest": false
			},
			"fields": [
				{
					"id": 1,
					"name": "label",
					"label": ScreenService.i18n("Layout name"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"id": 1,
					"name": "preferenceTypeLabel",
					"label": ScreenService.i18n("Type"),
					"showOnList": true,
					"template": "field/text-edit.html"
				},
				{
					"name": "pdfReportOrientation",
					"label": ScreenService.i18n("PDF Report Orientation"),
					"isVisible": true,
					"showOnList": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": true,
					"dataSource": {
						"localStorage": false,
						"data":[
							{
								"value": "portrait",
								"label": "Portrait"
							},
							{
								"value": "landscape",
								"label": "Landscape"
							}
						]
					}
				},
				{
					"id": 4,
					"name": "defaultPreference",
					"isVisible": false,
					"showOnList": true,
					"label": ScreenService.i18n("Default"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"hideCheckAllButton": true,
					"events": [{
						"id": 1,
						"name": "FieldOnChange",
						"code": "args.owner.field.widget.setAsDefaultPreference(args.row, args.owner.field.widget);"
					}]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	function createPreferenceDetailsWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 2,
			"showAsModal": true,
			"container": gridWidget.container,
			"name": "PreferenceDetailsWidget",
			"showDescriptionOrder": false,
			"label": "teste",
			"labelAction": true,
			"isVisible": true,
			"template": "widget/grid.html",
			"showCheckbox": true,
			"actions": actionsPreferenceDetailsWidget,
			"events": eventsPreferenceDetailWidget,
			"styles": [
				{
					"cssClass" : "last-field",
					"condition" : "row.sequenceInList == row.qtdRows"
				},
				{
					"cssClass" : "first-field",
					"condition" : "row.sequenceInList == 1"
				}
			],
			"parentWidget": parentWidget,
			"dataSource": {
				"data": []
			},
			"fields": [
				{
					"id": 1,
					"label": ScreenService.i18n("Column name"),
					"name": "label",
					"showOnList": true
				},
				{
					"id": 4,
					"name": "visibility",
					"isVisible": true,
					"showOnList": true,
					"label": ScreenService.i18n("Column visibility"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"events": [{
						"id": 1,
						"name": "FieldOnChange",
						"code": "args.owner.field.widget.changePreferenceVisibility(args.row, args.owner.field.widget);"
					}]
				},
				{
					"id": 4,
					"name": "fixed",
					"isVisible": true,
					"showOnList": true,
					"label": ScreenService.i18n("Fix column"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"events": []
				},
				{
					"id": 4,
					"name": "grouped",
					"isVisible": true,
					"showOnList": true,
					"label": ScreenService.i18n("Group by column"),
					"template": "field/checkbox.html",
					"align" : "center",
					"valueToTrue": true,
					"valueToFalse": false,
					"inlineEditable" : "checkbox",
					"events": []
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	function createPropertyPreferencesWidget(gridWidget, parentWidget, functionsToInject) {
		var widget = metaDataFactory.widgetFactory({
			"id": 9999,
			"name": "properties_widget",
			"showAsModal": true,
			"container": gridWidget.container,
			"parentWidget": parentWidget,
			"label": ScreenService.i18n("Column properties") + " - ",
			"showDescriptionOrder": false,
			"template": "widget/form.html",
			"isVisible": true,
			"popupNoBlock": false,
			"fields": propertiesField,
			"dataSource": [],
			"dataSourceFilter": [],
			"events": [],
			"actions": actionsPropertiesWidget
		});
		return Util.extend(widget, functionsToInject);
	}

	function createNewPreferenceWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 1,
			"name": "newPreferenceWidget",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Create a new layout"),
			"template": "widget/form.html",
			"parentWidget": parentWidget,
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "name",
					"label": ScreenService.i18n("Name"),
					"class": "6 col-md-6",
					"isVisible": true,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/text-edit.html",
					"validations": {
						"required": {
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "type",
					"label": ScreenService.i18n("Type"),
					"class": "3 col-md-3",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data":[]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				},
				{
					"name": "pdfReportOrientation",
					"label": ScreenService.i18n("PDF Report Orientation"),
					"class": "3 col-md-3",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": true,
					"dataSource": {
						"localStorage": false,
						"data":[
							{
								"value": "portrait",
								"label": "Portrait"
							},
							{
								"value": "landscape",
								"label": "Landscape"
							}
						]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.closePopupAndRunDigest()"
					}]
				},
				{
					"id": 1,
					"name": "addPreference",
					"label": ScreenService.i18n("Continue"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "args.owner.widget.createNewPreference(args.owner.widget)"
						}
					]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	function createChangePreferenceTypeWidget(gridWidget, parentWidget, functionsToInject){
		var widget = metaDataFactory.widgetFactory({
			"id": 8998,
			"name": "changePreferenceTypeWidget",
			"showAsModal": true,
			"container": gridWidget.container,
			"label": ScreenService.i18n("Change layout type"),
			"template": "widget/form.html",
			"parentWidget": parentWidget,
			"showDescriptionOrder": false,
			"isVisible": true,
			"fields": [
				{
					"name": "preferenceType",
					"label": ScreenService.i18n("New layout type"),
					"class": "12",
					"isVisible": true,
					"readOnly": false,
					"showOnList": true,
					"showOnForm": true,
					"showOnView": true,
					"template": "field/select-native.html",
					"valueField": "value",
					"descriptionField": "label",
					"events": [],
					"hideEmptyValue": false,
					"dataSource": {
						"data":[]
					},
					"validations": {
						"required":{
							"message": ScreenService.i18n("Required Field")
						}
					}
				}
			],
			"events": [],
			"actions":[
				{
					"id": 1,
					"name": "canceladd",
					"label": ScreenService.i18n("Cancel"),
					"isVisible": true,
					"hideIcon": true,
					"showAsAction": "back",
					"events": [{
						"id": 1,
						"name": "ActionEvent",
						"code": "args.owner.widget.closePopupAndRunDigest()"
					}]
				},
				{
					"id": 1,
					"name": "addPreference",
					"label": ScreenService.i18n("Save"),
					"isVisible": true,
					"showAsAction": "never",
					"events": [
						{
							"id": 1,
							"name": "ActionEvent",
							"code": "args.owner.widget.saveTransferLayoutClick(args.owner.widget)"
						}
					]
				}
			]
		});
		return Util.extend(widget, functionsToInject);
	}

	var actionsPreferencesWidget = [
		{
			"id": 2,
			"name": "NEW",
			"isVisible": true,
			"showAsAction": "always",
			"icon": "plus icon-white",
			"tooltip": ScreenService.i18n("New"),
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.openNewPreferenceWidget(args.owner.widget)"
			}]
		},
		{
			"id": 3,
			"name": "cancelAction",
			"label": ScreenService.i18n("Close"),
			"showAsAction": "back",
			"hideIcon": true,
			"isVisible": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.closePopupAndRunDigest();"
			}]
		},
		{
			"id": 4,
			"name": "deletePreference",
			"isVisible": true,
			"icon": "trash",
			"showAsAction": "checked_rows",
			"condition": "row.acl_edit",
			"tooltip": ScreenService.i18n("Delete preference"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.deletePreferenceClick(args.owner.widget);"
				}
			]
		}
	];

	var actionsPropertiesWidget = [
		{
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"hideIcon": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.backToPreviousWidget(args.owner.widget)"
			}]
		}, {
			"label": ScreenService.i18n("OK"),
			"showAsAction": "never",
			"isVisible": true,
			"showOnForm": true,
			"showOnList": true,
			"events": [{
				"id": 1,
				"name": "ActionEvent",
				"code": "args.owner.widget.saveFieldChanges(args.owner.widget, args.row)"
			}]
		}
	];

	var actionsPreferenceDetailsWidget = [
		{
			"id": 0,
			"name": "saveAndApply",
			"isVisible": true,
			"showAsAction": "never",
			"label": ScreenService.i18n("Save & Apply"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.saveAndApplyPreference(args.owner.widget)"
				}
			]
		},
		{
			"id": 2,
			"name": "backAction",
			"label": ScreenService.i18n("Cancel"),
			"showAsAction": "back",
			"isVisible": true,
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.backToPreviousWidget(args.owner.widget)"
				}
			]
		},
		{
			"id": 3,
			"name": "moveUpField",
			"isVisible": true,
			"icon": "up",
			"showAsAction":"checked_rows",
			"tooltip": ScreenService.i18n("Move field up"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.changeOrder(-1, args.owner.widget);"
				}
			]
		},
		{
			"id": 4,
			"name": "moveDownField",
			"isVisible": true,
			"icon": "down",
			"showAsAction": "checked_rows",
			"tooltip": ScreenService.i18n("Move field down"),
			"events": [
				{
					"id": 1,
					"name": "ActionEvent",
					"code": "args.owner.widget.changeOrder(1, args.owner.widget);"
				}
			]
		}
	];

	var eventsPreferenceDetailWidget = [
		{
			"id": 1,
			"name": "WidgetAfterMoveRow",
			"code": "args.owner.openPopupPreferencesProperty(args.owner, args.row)"
		}
	];

	var propertiesField = [
		{
			"name": "label",
			"label": ScreenService.i18n("Label"),
			"class": "12 col-md-12",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/text-edit.html",
			"events": []
		},
		{
			"name": "align",
			"label": ScreenService.i18n("Alignment"),
			"class": "12 col-md-12",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/select-native.html",
			"valueField": "value",
			"descriptionField": "value",
			"events": [],
			"dataSource": {
				"data":[
					{ "value": "left" },
					{ "value": "center" },
					{ "value": "right" }
				]
			}
		},
		{
			"name": "expression",
			"label": ScreenService.i18n("Group expression"),
			"class": "12 col-md-12",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/select-native.html",
			"valueField": "value",
			"descriptionField": "value",
			"defaultOption": "left",
			"events": [],
			"dataSource": {
				"data":[
					{ "value": "COUNT" },
					{ "value": "SUM" },
					{ "value": "AVG" }
				]
			}
		},
		{
			"name": "minWidth",
			"label": ScreenService.i18n("Minimun width (px)"),
			"class": "6 col-md-6",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/number.html",
			"spin": true,
			"minValue": 0,
			"events": []
		},
		{
			"name": "maxWidth",
			"label": ScreenService.i18n("Maximum width (px)"),
			"class": "6 col-md-6",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/number.html",
			"minValue": 0,
			"spin": true,
			"events": []
		},
		{
			"name": "reportColumnSize",
			"label": ScreenService.i18n("PDF Report Column Size"),
			"class": "6 col-md-6",
			"isVisible": true,
			"readOnly": false,
			"showOnList": true,
			"showOnForm": true,
			"showOnView": true,
			"template": "field/text.html",
			"events": []
		}
	];

}

Configuration(function(ContextRegister) {
	ContextRegister.register("carouselWidgetCreator", CarouselWidgetCreator);
});


// FILE: src/LayoutPreferences.js
function LayoutPreferences(templateManager, ScreenService, Query, ZHPromise, carouselWidgetCreator, eventAggregator, zhCustomizationsService, eventEngine, containerMerger, preferencesUserService) {

	var self = this;
	var protectedMethods = {};

	var widget, preferenceActivated;

	var fieldDefaultProperties = {
		isVisible: true,
		showOnList: true,
		visibility: true,
		grouped: false,
		fixed: false
	};

	eventAggregator.subscribe("OnListStartUp", getAndApplyDefaultPreference);

	function getAndApplyDefaultPreference(eventData) {
		if (eventData.widget) {
			var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
			self.getDefaultPreferenceByUser(eventData.widget, userRegistered).then(function(defaultPreference) {
				if (defaultPreference) {
					self.applyLayout(defaultPreference, eventData.widget);
				} else {
					eventData.widget.customizationsApplied = eventData.widget.customizationsApplied || {};
					eventData.widget.customizationsApplied.layout = "default";
				}
			});
		}
	}

	this.getDefaultPreferenceByUser = function(widget, userId) {
		return zhCustomizationsService.getWidgetCustomizations(widget, "layout").then(function(preferences) {
			return preferences.filter(function(preference) {
				return preference.mapUserIdDefault[userId];
			}).shift();
		});
	};

	this.startConfig = function(editingWidget) {
		widget = editingWidget;
		ScreenService.openPopup(carouselWidgetCreator.createCaroulselWidget(editingWidget, functionsAsArguments));
	};

	this.restoreDefault = function(editingWidget) {
		self.restoreDefaultLayout(editingWidget);
	};

	this.preferencesListOnEnter = function(widgetPreferenceList) {
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		return zhCustomizationsService.getWidgetCustomizations(widget, "layout").then(function(preferences) {
			preferences.forEach(function(preference) {
				preference.defaultPreference = preference.mapUserIdDefault[userRegistered] || false;
				preference.preferenceTypeLabel = zhCustomizationsService.getLabelRepositoryType(preference.customizationRepository);
			});
			widgetPreferenceList.dataSource.data = preferences;
		});
	};

	this.applyLayout = function(layout, widget) {
		if (widget.backupPDFOrientation === undefined) {
			widget.backupPDFOrientation = widget.reportOrientation || 'portrait';
		}

		widget.reportOrientation = layout.pdfReportOrientation || widget.backupPDFOrientation;
		containerMerger.merge(templateManager.container, layout.containerDiff, "original");
		if (!widget.customizationsApplied) {
			widget.customizationsApplied = {};
		}
		widget.customizationsApplied.layout = layout._id;
		redrawWidget(widget);
	};


	function applyLayoutEventCallback(eventData) {
		var customization = eventData.e.customization;
		var editingWidget = eventData.e.editingWidget;
		self.restoreDefaultLayout(editingWidget);
		if (!customization.isDefaultOption) {
			self.applyLayout(customization, editingWidget);
		}
	}

	this.restoreDefaultLayout = function(widget) {
		widget.reportOrientation = widget.backupPDFOrientation || 'portrait';
		widget.customizationsApplied = widget.customizationsApplied || {};
		widget.customizationsApplied.layout = "default";

		containerMerger.restore(templateManager.container, "original");
		redrawWidget(widget);
	};

	function redrawWidget(widget) {
		if (widget.redraw) {
			widget.redraw(true);
		}
		templateManager.updateTemplate();
	}

	eventEngine.bindEvent(zhCustomizationsService, "LayoutPreferenceOnClick", applyLayoutEventCallback);

	var openNewPreferenceWidget = function(widget) {
		var caroulsel = widget.parentWidget;
		var popupAddPreference = caroulsel.widgets[3];
		var typeField = popupAddPreference.getField("type");
		var allowedPreferences = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		typeField.dataSource.data = allowedPreferences;
		popupAddPreference.newRow();
		ScreenService.openPopup(popupAddPreference);
	};

	var openPopupSelectDestinyRepository = function(preferencesListWidget) {
		var changePreferenceTypeWidget = preferencesListWidget.parentWidget.widgets[4];
		changePreferenceTypeWidget.newRow();
		var allowedPreferences = zhCustomizationsService.getCustomizationsRepositoriesTypes();
		changePreferenceTypeWidget.getField("preferenceType").dataSource.data = allowedPreferences;
		var toTransferLayouts = preferencesListWidget.getCheckedRows();
		var isSomeLayoutConnectedToAnVisuzalization = false;
		var promises = toTransferLayouts.map(function(layout) {
			return zhCustomizationsService.getCustomizationView(widget, layout.customizationRepository, layout.customizationType, layout._id).then(function(connectedView) {
				if (connectedView) {
					isSomeLayoutConnectedToAnVisuzalization = true;
				}
			});
		});
		ZHPromise.all(promises).then(function() {
			if (!isSomeLayoutConnectedToAnVisuzalization) {
				ScreenService.openPopup(changePreferenceTypeWidget);
			} else {
				ScreenService.showMessage(ScreenService.i18n("This layout is connected with a customizations. It can't be transferred."));
			}
		});
	};

	var saveTransferLayoutClick = function(widgetChangePreferenceType) {
		var preferencesListWidget = widgetChangePreferenceType.parentWidget.widgets[0];
		var toChangePreferences = preferencesListWidget.getCheckedRows();
		var newLayoutType = widgetChangePreferenceType.getField("preferenceType").value();
		zhCustomizationsService.transferCustomizationsRepository(widget, toChangePreferences, newLayoutType).then(function() {
			self.preferencesListOnEnter(preferencesListWidget);
			ScreenService.closePopup();
		});
	};

	var applyLayoutActionClick = function(row) {
		self.applyLayout(row, widget);
		ScreenService.closePopup(true);
		widget.activate();
	};

	var closePopupAndRunDigest = function() {
		ScreenService.closePopup();
		widget.activate();
		ScreenService.updateTemplate();
	};

	var createNewPreference = function(widget) {
		if (widget.isValid()) {
			var carousel = widget.parentWidget;
			var preferencesListWidget = carousel.widgets[0];
			var preferenceName = widget.currentRow.name;
			var customizationRepository = widget.currentRow.type;
			var pdfReportOrientation = widget.currentRow.pdfReportOrientation;
			var preferenceHeader = createPreferenceHeader(preferenceName, customizationRepository, pdfReportOrientation);

			ScreenService.closePopup();
			templateManager.updateTemplate();
			self.preferencesListOnEnter(preferencesListWidget).then(function() {
				openEditPreference(preferencesListWidget, preferenceHeader);
			});
		}
	};

	var openEditPreference = function(widgetBefore, row) {
		activatePreference(row);
		var indexNextWidget = widgetBefore.parentWidget.getNextIndex();
		var nextWidget = widgetBefore.parentWidget.widgets[indexNextWidget];
		nextWidget.dataSource.data = getFieldEditingWidgetAsDataSource(widget);
		var widgetDifferences = getEditingWidgetPath(row.containerDiff.widgets);
		nextWidget.propertiesToChange = {};
		nextWidget.label = row.label;
		nextWidget.dataSource.data = sortFieldsOrder(nextWidget.dataSource.data.map(function(field, index) {
			delete field.__isSelected;
			delete field.__leftActions;
			delete field.__notSelectable;
			delete field.__rightActions;
			if (!field.sequenceInList) {
				field.sequenceInList = index + 1;
			}
			field.qtdRows = nextWidget.dataSource.data.length;
			if (!Util.isEmptyOrBlank(widgetDifferences.fields)) {
				var fieldPreference = filterObject(widgetDifferences.fields, field.id, "id");
				if (fieldPreference.obj && Object.keys(fieldPreference.obj).length) {
					return Util.extend(field, fieldPreference.obj, true);
				}
			}
			return Util.extend(fieldDefaultProperties, field, true);
		}));
		widgetBefore.parentWidget.nextWidget();
	};

	function getFieldEditingWidgetAsDataSource(gridWidget) {
		return gridWidget.fields.map(function(field) {
			var data = {
				"id": field.id,
				"name": field.name,
				"label": field.label,
				"align": field.align,
				"isVisible": field.isVisible,
				"showOnList": field.showOnList,
				"isVisibleOnReport": field.isVisible && field.showOnList,
				"visibility": field.isVisible && field.showOnList,
				"grouped": field.grouped || false,
				"fixed": field.fixed || false,
				"skipPreference": field.skipPreference || false
			};
			return data;
		}).filter(function(field) {
			return !field.skipPreference;
		});
	}

	var changeOrder = function(operator, widget) {
		var lastUnchecked;
		if (operator === 1) {
			widget.dataSource.data.reverse();
		}

		widget.dataSource.data.forEach(function(row, index) {
			if (row.__isSelected) {
				if (lastUnchecked && ((row.sequenceInList > 1 && operator == -1) || (row.sequenceInList < widget.dataSource.data.length && operator == 1))) {
					row.sequenceInList += operator;
					if (lastUnchecked.sequenceInList == row.sequenceInList) {
						lastUnchecked.sequenceInList -= operator;
					}
				}
			} else {
				lastUnchecked = row;
			}
		});
		widget.dataSource.data = sortFieldsOrder(widget.dataSource.data);
	};

	var sortFieldsOrder = function(nextWidgetDataSource) {
		return nextWidgetDataSource.sort(function(firstField, secondField) {
			return firstField.sequenceInList - secondField.sequenceInList;
		});
	};

	var activatePreference = function(preference) {
		preferenceActivated = preference;
	};

	var changePreferenceVisibility = function(row, widget) {
		row.isVisible = true;
		row.showOnList = row.isVisibleOnReport = row.visibility;
	};

	var setAsDefaultPreference = function(row, widget) {
		var toUpdate = [];
		var userRegistered = preferencesUserService.getPreferenceUserRegistered() || "default";
		var preferenceSetAsDefault = widget.dataSource.data.filter(function(preference) {
			return preference.mapUserIdDefault[userRegistered] === true && preference._id !== row._id;
		}).shift();
		if (preferenceSetAsDefault) {
			preferenceSetAsDefault.mapUserIdDefault[userRegistered] = false;
			removeNotInterestingProperties(preferenceSetAsDefault);
			toUpdate.push(preferenceSetAsDefault);
		}
		removeNotInterestingProperties(row);
		row.mapUserIdDefault[userRegistered] = !row.mapUserIdDefault[userRegistered];
		toUpdate.push(row);
		zhCustomizationsService.saveCustomization(toUpdate, 'layout').then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				ScreenService.hideLoader();
				ScreenService.updateTemplate();
			});
		});
	};

	var deletePreferenceClick = function(widget) {
		return zhCustomizationsService.removeCustomizations(widget.getCheckedRows()).then(function() {
			self.preferencesListOnEnter(widget).then(function() {
				templateManager.updateTemplate();
			});
		});
	};

	var createPreferenceHeader = function(preferenceName, customizationRepository, pdfReportOrientation) {
		var header = {
			"label": preferenceName,
			"widgetId": widget.id,
			"containerId": widget.container.id,
			"customizationRepository": customizationRepository,
			"unsync_id": new Date().getTime(),
			"customizationType": "layout",
			"mapUserIdDefault": {
				"default": false
			},
			"defaultPreference": false,
			"pdfReportOrientation": pdfReportOrientation,
			"containerDiff": createDiffDataStructure(widget.container, widget)
		};
		return header;
	};

	var createDiffDataStructure = function(container, widget) {
		var containerDiff = {};
		containerDiff.id = container.id;
		createWidgetsDataStructure(widget.id, container.widgets, containerDiff);
		return containerDiff;
	};

	var createWidgetsDataStructure = function(widgetId, widgets, containerDiff) {
		containerDiff.widgets = [];
		createWidgetsObjects(containerDiff.widgets, widgets, widgetId);
		if (Util.isArray(widgets) && widgets.length > 0) {
			widgets.forEach(function(eachWidget) {
				createWidgetsDataStructure(widgetId, eachWidget.widgets, getWidgetObjectById(containerDiff.widgets, eachWidget.id));
			});
		}
	};

	var getWidgetObjectById = function(widgets, widgetId) {
		return widgets.filter(function(eachWidgetObject) {
			return eachWidgetObject.id == widgetId;
		}).shift();
	};

	var createWidgetsObjects = function(widgetsDiff, widgetsArray, widgetId) {
		if (Util.isArray(widgetsArray) && widgetsArray.length > 0) {
			widgetsArray.forEach(function(eachWidget) {
				var properties = {
					"id": eachWidget.id
				};
				if (eachWidget.id == widgetId) {
					properties.fields = [];

				}
				widgetsDiff.push(properties);
			});
		}
	};

	var getEditingWidgetPath = function(widgetsArray) {
		var foundWidget = getWidgetObjectById(widgetsArray, widget.id);
		if (!foundWidget) {
			widgetsArray.some(function(eachWidget) {
				foundWidget = getEditingWidgetPath(eachWidget.widgets);
				return foundWidget;
			});
		}
		return foundWidget;
	};

	var updatePreferenceData = function(fields, preferencesWidgetList) {
		var caroulselWidget = preferencesWidgetList.parentWidget;
		var editingWidget = getEditingWidgetPath(preferenceActivated.containerDiff.widgets);
		removeNotInterestingProperties(preferenceActivated);
		if (Util.isArray(fields)) {
			editingWidget.fields = fields;
		} else {
			var index = filterObject(editingWidget.fields, fields.id, "id").index;
			editingWidget.fields[index] = fields;
		}
		return zhCustomizationsService.saveCustomization(preferenceActivated, 'layout').then(function() {
			self.preferencesListOnEnter(preferencesWidgetList).then(function() {
				caroulselWidget.previousWidget();
				templateManager.updateTemplate();
			});
		});
	};

	var removeNotInterestingProperties = function(object) {
		Object.keys(object).forEach(function(key) {
			if (~key.indexOf('__')) {
				delete object[key];
			}
		});
	};

	var saveFieldSettings = function(widgetPreferenceFields) {
		ScreenService.showLoader();
		var caroulselWidget = widgetPreferenceFields.parentWidget;
		var widgetPreferenceList = caroulselWidget.widgets[0];
		var fields = widgetPreferenceFields.dataSource.data;
		return updatePreferenceData(fields, widgetPreferenceList).then(function() {
			ScreenService.hideLoader();
		});
	};

	var saveAndApplyPreference = function(widgetPreferenceFields) {
		return saveFieldSettings(widgetPreferenceFields).then(function() {
			applyLayoutActionClick(preferenceActivated);
		});
	};

	var openPopupPreferencesProperty = function(widget, row) {
		var indexNextWidget = widget.parentWidget.getNextIndex();
		var nextWidget = widget.parentWidget.widgets[indexNextWidget];
		nextWidget.currentFieldId = row.id;
		nextWidget.setCurrentRow(row);
		nextWidget.label = ScreenService.i18n("Column properties") + " - " + row.label;
		widget.parentWidget.nextWidget();
	};

	var backToPreviousWidget = function(widget) {
		widget.parentWidget.previousWidget();
	};

	var saveFieldChanges = function(widgetProperties, row) {
		var preferencesListWidget = widgetProperties.parentWidget.widgets[0];
		return updatePreferenceData(row, preferencesListWidget);
	};

	//Controller Utilities
	var filterObject = function(obj, value, key) {
		var filtered = obj.filter(function(preference) {
			return preference[key] == value;
		});
		var toReturn = {};
		toReturn.index = obj.indexOf(filtered[0]);
		toReturn.obj = filtered[0];
		return toReturn;
	};

	var functionsPreferencesListWidget = {
		"openEditPreference": openEditPreference,
		"deletePreferenceClick": deletePreferenceClick,
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"setAsDefaultPreference": setAsDefaultPreference,
		"applyLayoutActionClick": applyLayoutActionClick,
		"openNewPreferenceWidget": openNewPreferenceWidget,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository,
		"preferencesListOnEnter": self.preferencesListOnEnter
	};

	var functionsPreferenceFieldsListWidget = {
		"backToPreviousWidget": backToPreviousWidget,
		"changePreferenceVisibility": changePreferenceVisibility,
		"openPopupPreferencesProperty": openPopupPreferencesProperty,
		"saveFieldSettings": saveFieldSettings,
		"changeOrder": changeOrder,
		"saveAndApplyPreference": saveAndApplyPreference
	};

	var functionsFieldsPropertiesListWidget = {
		"backToPreviousWidget": backToPreviousWidget,
		"saveFieldChanges": saveFieldChanges
	};

	var functionsNewPreferenceWidget = {
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"createNewPreference": createNewPreference
	};

	var functionsChangePreferenceTypeWidget = {
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"openPopupSelectDestinyRepository": openPopupSelectDestinyRepository,
		"saveTransferLayoutClick": saveTransferLayoutClick
	};

	var functionsTransferCustomizationRepository = {
		"closePopupAndRunDigest": closePopupAndRunDigest,
		"saveTransferLayoutClick": saveTransferLayoutClick
	};

	var functionsAsArguments = {
		"functionsPreferencesListWidget": functionsPreferencesListWidget,
		"functionsPreferenceFieldsListWidget": functionsPreferenceFieldsListWidget,
		"functionsFieldsPropertiesListWidget": functionsFieldsPropertiesListWidget,
		"functionsNewPreferenceWidget": functionsNewPreferenceWidget,
		"functionsChangePreferenceTypeWidget": functionsChangePreferenceTypeWidget
	};
}

Configuration(function(ContextRegister) {
	ContextRegister.register("zhLayoutPreferences", LayoutPreferences);
});